@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -236,10 +236,14 @@ ActionType action_get_type(const gchar *action_str)
 			case 'h':
 				action_type |= ACTION_USER_HIDDEN_STR;
 				break;
+			case '%':
+				/* literal '%' */
+				break;
 			default:
 				action_type = ACTION_ERROR;
 				break;
 			}
+			p++;
 		} else if (p[0] == '|') {
 			if (p[1] == '\0')
 				action_type |= ACTION_PIPE_OUT;
@@ -321,6 +325,10 @@ static gchar *parse_action_cmd(gchar *action, MsgInfo *msginfo,
 					g_string_append(cmd, user_hidden_str);
 				p++;
 				break;
+			case '%':
+				g_string_append_c(cmd, p[1]);
+				p++;
+				break;
 			default:
 				g_string_append_c(cmd, p[0]);
 				g_string_append_c(cmd, p[1]);
@@ -840,14 +848,14 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 					   "Pipe creation failed.\n%s"),
 					g_strerror(errno));
 			/* Closing fd = -1 fails silently */
-			close(chld_in[0]);
-			close(chld_in[1]);
-			close(chld_out[0]);
-			close(chld_out[1]);
-			close(chld_err[0]);
-			close(chld_err[1]);
-			close(chld_status[0]);
-			close(chld_status[1]);
+			(void)close(chld_in[0]);
+			(void)close(chld_in[1]);
+			(void)close(chld_out[0]);
+			(void)close(chld_out[1]);
+			(void)close(chld_err[0]);
+			(void)close(chld_err[1]);
+			(void)close(chld_status[0]);
+			(void)close(chld_status[1]);
 			return NULL; /* Pipe error */
 		}
 	}
@@ -861,7 +869,7 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			perror("setpgid");
 
 #ifdef GDK_WINDOWING_X11
-		close(ConnectionNumber(gdk_display));
+		(void)close(ConnectionNumber(gdk_display));
 #endif /* GDK_WINDOWING_X11 */
 
 		gch_pid = fork();
@@ -875,21 +883,21 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 				    (ACTION_PIPE_IN |
 				     ACTION_USER_IN |
 				     ACTION_USER_HIDDEN_IN)) {
-					close(fileno(stdin));
-					dup  (chld_in[0]);
+					(void)close(fileno(stdin));
+					(void)dup  (chld_in[0]);
 				}
-				close(chld_in[0]);
-				close(chld_in[1]);
+				(void)close(chld_in[0]);
+				(void)close(chld_in[1]);
 
-				close(fileno(stdout));
-				dup  (chld_out[1]);
-				close(chld_out[0]);
-				close(chld_out[1]);
+				(void)close(fileno(stdout));
+				(void)dup  (chld_out[1]);
+				(void)close(chld_out[0]);
+				(void)close(chld_out[1]);
 
-				close(fileno(stderr));
-				dup  (chld_err[1]);
-				close(chld_err[0]);
-				close(chld_err[1]);
+				(void)close(fileno(stderr));
+				(void)dup  (chld_err[1]);
+				(void)close(chld_err[0]);
+				(void)close(chld_err[1]);
 			}
 
 			cmdline[0] = "sh";
@@ -909,24 +917,24 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			_exit(1);
 		} else if (gch_pid < (pid_t) 0) { /* Fork error */
 			if (sync)
-				write(chld_status[1], "1\n", 2);
+				(void)write(chld_status[1], "1\n", 2);
 			perror("fork");
 			_exit(1);
 		} else { /* Child */
 			if (sync) {
-				close(chld_in[0]);
-				close(chld_in[1]);
-				close(chld_out[0]);
-				close(chld_out[1]);
-				close(chld_err[0]);
-				close(chld_err[1]);
-				close(chld_status[0]);
+				(void)close(chld_in[0]);
+				(void)close(chld_in[1]);
+				(void)close(chld_out[0]);
+				(void)close(chld_out[1]);
+				(void)close(chld_err[0]);
+				(void)close(chld_err[1]);
+				(void)close(chld_status[0]);
 
 				debug_print("Child: waiting for grandchild\n");
 				waitpid(gch_pid, NULL, 0);
 				debug_print("Child: grandchild ended\n");
-				write(chld_status[1], "0\n", 2);
-				close(chld_status[1]);
+				(void)write(chld_status[1], "0\n", 2);
+				(void)close(chld_status[1]);
 			}
 			_exit(0);
 		}
@@ -944,13 +952,13 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		return NULL;
 	}
 
-	close(chld_in[0]);
+	(void)close(chld_in[0]);
 	if (!(children->action_type &
 	      (ACTION_PIPE_IN | ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
-		close(chld_in[1]);
-	close(chld_out[1]);
-	close(chld_err[1]);
-	close(chld_status[1]);
+		(void)close(chld_in[1]);
+	(void)close(chld_out[1]);
+	(void)close(chld_err[1]);
+	(void)close(chld_status[1]);
 
 	child_info = g_new0(ChildInfo, 1);
 
@@ -981,13 +989,13 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		ret_str = g_locale_from_utf8(msg_str, strlen(msg_str),
 					     &by_read, &by_written, NULL);
 		if (ret_str && by_written) {
-			write(chld_in[1], ret_str, strlen(ret_str));
+			(void)write(chld_in[1], ret_str, strlen(ret_str));
 			g_free(ret_str);
 		} else
-			write(chld_in[1], msg_str, strlen(msg_str));
+			(void)write(chld_in[1], msg_str, strlen(msg_str));
 		if (!(children->action_type &
 		      (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
-			close(chld_in[1]);
+			(void)close(chld_in[1]);
 		child_info->chld_in = -1; /* No more input */
 	}
 
@@ -1104,13 +1112,13 @@ static void childinfo_close_pipes(ChildInfo *child_info)
 		gdk_input_remove(child_info->tag_err);
 
 	if (child_info->chld_in >= 0)
-		close(child_info->chld_in);
+		(void)close(child_info->chld_in);
 	if (child_info->chld_out >= 0)
-		close(child_info->chld_out);
+		(void)close(child_info->chld_out);
 	if (child_info->chld_err >= 0)
-		close(child_info->chld_err);
+		(void)close(child_info->chld_err);
 
-	close(child_info->chld_status);
+	(void)close(child_info->chld_status);
 }
 
 static void free_children(Children *children)
@@ -1280,8 +1288,14 @@ static void create_io_dialog(Children *children)
 		g_signal_connect(G_OBJECT(entry), "activate",
 				 G_CALLBACK(send_input), children);
 		gtk_box_pack_start(GTK_BOX(input_hbox), entry, TRUE, TRUE, 0);
-		if (children->action_type & ACTION_USER_HIDDEN_IN)
+		if (children->action_type & ACTION_USER_HIDDEN_IN) {
 			gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+#ifdef MAEMO
+			hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry), 
+				HILDON_GTK_INPUT_MODE_FULL | 
+				HILDON_GTK_INPUT_MODE_INVISIBLE);
+#endif
+		}
 		gtk_widget_show(entry);
 
 		send_button = gtk_button_new_from_stock(GTK_STOCK_EXECUTE);
@@ -1437,11 +1451,11 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 	} while (c >= 0 && count < len);
 
 	if (c >= 0)
-		write(child_info->chld_in, "\n", 2);
+		(void)write(child_info->chld_in, "\n", 2);
 
 	g_free(input);
 
-	close(child_info->chld_in);
+	(void)close(child_info->chld_in);
 	child_info->chld_in = -1;
 	debug_print("Input to grand child sent.\n");
 }
@@ -1516,12 +1530,12 @@ static void catch_output(gpointer data, gint source, GdkInputCondition cond)
 		if (source == child_info->chld_out) {
 			gdk_input_remove(child_info->tag_out);
 			child_info->tag_out = -1;
-			close(child_info->chld_out);
+			(void)close(child_info->chld_out);
 			child_info->chld_out = -1;
 		} else {
 			gdk_input_remove(child_info->tag_err);
 			child_info->tag_err = -1;
-			close(child_info->chld_err);
+			(void)close(child_info->chld_err);
 			child_info->chld_err = -1;
 		}
 	}
