@@ -929,7 +929,7 @@ static void addrcompl_add_entry( CompletionWindow *cw, gchar *address ) {
 		/* Select first row for now */
 		gtk_tree_selection_select_iter(selection, &iter);
 	}
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	else if( cw->listCount == 2 ) {
 		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
 		/* Move off first row */
@@ -1290,7 +1290,7 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 		new = get_next_complete_address();
 		g_free( new );
 	}
-#if (!defined(USE_LDAP) && !defined(MAEMO))
+#if (!defined(USE_LDAP) && !defined(GENERIC_UMPC))
 	/* Select the address if there is only one match */
 	if (ncount == 2) {
 		/* Display selected address in entry field */		
