@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* General functions for accessing address book files */
@@ -500,8 +500,10 @@ static void addrbook_parse_person(AddressBookFile *book, XMLFile *file)
 		value = ((XMLAttr *)attr->data)->value;
 		if (!person) 
 			person = addritem_create_item_person();
-		if (strcmp(name, AB_ATTAG_UID) == 0)
+		if (strcmp(name, AB_ATTAG_UID) == 0) {
 			ADDRITEM_ID(person) = g_strdup(value);
+			person->picture = g_strdup(value);
+		}
 		else if (strcmp(name, AB_ATTAG_FIRST_NAME) == 0)
 			person->firstName = g_strdup(value);
 		else if (strcmp(name, AB_ATTAG_LAST_NAME) == 0)
@@ -920,7 +922,7 @@ gint addrbook_read_data(AddressBookFile *book)
 	g_return_val_if_fail(book != NULL, -1);
 
 	/*
-	printf( "...addrbook_read_data :%s:\t:%s:\n", book->fileName,
+	g_print( "...addrbook_read_data :%s:\t:%s:\n", book->fileName,
 		addrcache_get_name( book->addressCache ) );
 	*/
 
@@ -957,13 +959,18 @@ gint addrbook_read_data(AddressBookFile *book)
  * \param lvl  Indent level.
  * \param name Element name.
  */
-static void addrbook_write_elem_s(FILE *fp, gint lvl, gchar *name)
+static int addrbook_write_elem_s(FILE *fp, gint lvl, gchar *name)
 {
 	gint i;
 	for (i = 0; i < lvl; i++) 
-		fputs("  ", fp);
-	fputs("<", fp);
-	fputs(name, fp);
+		if (fputs("  ", fp) == EOF)
+			return -1;
+	if (fputs("<", fp) == EOF)
+		return -1;
+	if (fputs(name, fp) == EOF)
+		return -1;
+		
+	return 0;
 }
 
 /**
@@ -972,14 +979,20 @@ static void addrbook_write_elem_s(FILE *fp, gint lvl, gchar *name)
  * \param lvl  Indent level.
  * \param name Element name.
  */
-static void addrbook_write_elem_e(FILE *fp, gint lvl, gchar *name)
+static int addrbook_write_elem_e(FILE *fp, gint lvl, gchar *name)
 {
 	gint i;
 	for(i = 0; i < lvl; i++)
-		fputs("  ", fp);
-	fputs("</", fp);
-	fputs(name, fp);
-	fputs(">\n", fp);
+		if (fputs("  ", fp) == EOF)
+			return -1;
+	if (fputs("</", fp) == EOF)
+		return -1;
+	if (fputs(name, fp) == EOF)
+		return -1;
+	if (fputs(">\n", fp) == EOF)
+		return -1;
+		
+	return 0;
 }
 
 /**
@@ -988,15 +1001,27 @@ static void addrbook_write_elem_e(FILE *fp, gint lvl, gchar *name)
  * \param name  Attribute name.
  * \param value Attribute value.
  */
-static void addrbook_write_attr(FILE *fp, gchar *name, gchar *value)
+static int addrbook_write_attr(FILE *fp, gchar *name, gchar *value)
 {
-	fputs(" ", fp);
-	fputs(name, fp);
-	fputs("=\"", fp);
-	xml_file_put_escape_str(fp, value);
-	fputs("\"", fp);
+	if (fputs(" ", fp) == EOF)
+		return -1;
+	if (fputs(name, fp) == EOF)
+		return -1;
+	if (fputs("=\"", fp) == EOF)
+		return -1;
+	if (xml_file_put_escape_str(fp, value) < 0)
+		return -1;
+	if (fputs("\"", fp) == EOF)
+		return -1;
+	
+	return 0;
 }
 
+typedef struct _HashLoopData {
+	FILE *fp;
+	gboolean error;
+} HashLoopData;
+
 /**
  * Write person and associated addresses and attributes to file.
  * file hash table visitor function.
@@ -1005,10 +1030,11 @@ static void addrbook_write_attr(FILE *fp, gchar *name, gchar *value)
  * \param data  File pointer.
  */
 static void addrbook_write_item_person_vis(gpointer key, gpointer value, 
-					   gpointer data)
+					   gpointer d)
 {
 	AddrItemObject *obj = (AddrItemObject *) value;
-	FILE *fp = (FILE *) data;
+	HashLoopData *data = (HashLoopData *)d;
+	FILE *fp = data->fp;
 	GList *node;
 
 	if (!obj)
@@ -1016,46 +1042,72 @@ static void addrbook_write_item_person_vis(gpointer key, gpointer value,
 	if (ADDRITEM_TYPE(obj) == ITEMTYPE_PERSON) {
 		ItemPerson *person = (ItemPerson *) value;
 		if (person) {
-			addrbook_write_elem_s(fp, 1, AB_ELTAG_PERSON);
-			addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(person));
-			addrbook_write_attr(fp, AB_ATTAG_FIRST_NAME, person->firstName);
-			addrbook_write_attr(fp, AB_ATTAG_LAST_NAME, person->lastName);
-			addrbook_write_attr(fp, AB_ATTAG_NICK_NAME, person->nickName);
-			addrbook_write_attr(fp, AB_ATTAG_COMMON_NAME, ADDRITEM_NAME(person));
-			fputs(" >\n", fp);
+			if (addrbook_write_elem_s(fp, 1, AB_ELTAG_PERSON) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(person)) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_FIRST_NAME, person->firstName) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_LAST_NAME, person->lastName) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_NICK_NAME, person->nickName) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_COMMON_NAME, ADDRITEM_NAME(person)) < 0)
+				data->error = TRUE;
+			if (fputs(" >\n", fp) == EOF)
+				data->error = TRUE;
 
 			/* Output email addresses */
-			addrbook_write_elem_s(fp, 2, AB_ELTAG_ADDRESS_LIST);
-			fputs(">\n", fp);
+			if (addrbook_write_elem_s(fp, 2, AB_ELTAG_ADDRESS_LIST) < 0)
+				data->error = TRUE;
+			if (fputs(">\n", fp) == EOF)
+				data->error = TRUE;
 			node = person->listEMail;
 			while (node) {
 				ItemEMail *email = node->data;
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_ADDRESS);
-				addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(email));
-				addrbook_write_attr(fp, AB_ATTAG_ALIAS, ADDRITEM_NAME(email));
-				addrbook_write_attr(fp, AB_ATTAG_EMAIL, email->address);
-				addrbook_write_attr(fp, AB_ATTAG_REMARKS, email->remarks);
-				fputs(" />\n", fp);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_ADDRESS) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(email)) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_ALIAS, ADDRITEM_NAME(email)) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_EMAIL, email->address) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_REMARKS, email->remarks) < 0)
+					data->error = TRUE;
+				if (fputs(" />\n", fp) == EOF)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
-			addrbook_write_elem_e(fp, 2, AB_ELTAG_ADDRESS_LIST);
+			if (addrbook_write_elem_e(fp, 2, AB_ELTAG_ADDRESS_LIST) < 0)
+				data->error = TRUE;
 
 			/* Output user attributes */
-			addrbook_write_elem_s(fp, 2, AB_ELTAG_ATTRIBUTE_LIST);
-			fputs(">\n", fp);
+			if (addrbook_write_elem_s(fp, 2, AB_ELTAG_ATTRIBUTE_LIST) < 0)
+				data->error = TRUE;
+			if (fputs(">\n", fp) == EOF)
+				data->error = TRUE;
 			node = person->listAttrib;
 			while (node) {
 				UserAttribute *attrib = node->data;
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_ATTRIBUTE);
-				addrbook_write_attr(fp, AB_ATTAG_UID, attrib->uid);
-				addrbook_write_attr(fp, AB_ATTAG_NAME, attrib->name);
-				fputs(" >", fp);
-				xml_file_put_escape_str(fp, attrib->value);
-				addrbook_write_elem_e(fp, 0, AB_ELTAG_ATTRIBUTE);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_ATTRIBUTE) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_UID, attrib->uid) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_NAME, attrib->name) < 0)
+					data->error = TRUE;
+				if (fputs(" >", fp) == EOF)
+					data->error = TRUE;
+				if (xml_file_put_escape_str(fp, attrib->value) < 0)
+					data->error = TRUE;
+				if (addrbook_write_elem_e(fp, 0, AB_ELTAG_ATTRIBUTE) < 0)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
-			addrbook_write_elem_e(fp, 2, AB_ELTAG_ATTRIBUTE_LIST);
-			addrbook_write_elem_e(fp, 1, AB_ELTAG_PERSON);
+			if (addrbook_write_elem_e(fp, 2, AB_ELTAG_ATTRIBUTE_LIST) < 0)
+				data->error = TRUE;
+			if (addrbook_write_elem_e(fp, 1, AB_ELTAG_PERSON) < 0)
+				data->error = TRUE;
 		}
 	}
 }
@@ -1068,10 +1120,12 @@ static void addrbook_write_item_person_vis(gpointer key, gpointer value,
  * \param data  File pointer.
  */
 static void addrbook_write_item_group_vis(gpointer key, gpointer value, 
-					  gpointer data)
+					  gpointer d)
 {
 	AddrItemObject *obj = (AddrItemObject *) value;
-	FILE *fp = (FILE *) data;
+	HashLoopData *data = (HashLoopData *)d;
+	FILE *fp = data->fp;
+
 	GList *node;
 
 	if (!obj)
@@ -1079,27 +1133,40 @@ static void addrbook_write_item_group_vis(gpointer key, gpointer value,
 	if (ADDRITEM_TYPE(obj) == ITEMTYPE_GROUP) {
 		ItemGroup *group = (ItemGroup *) value;
 		if (group) {
-			addrbook_write_elem_s(fp, 1, AB_ELTAG_GROUP);
-			addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(group));
-			addrbook_write_attr(fp, AB_ATTAG_NAME, ADDRITEM_NAME(group));
-			addrbook_write_attr(fp, AB_ATTAG_REMARKS, group->remarks);
-			fputs(" >\n", fp);
+			if (addrbook_write_elem_s(fp, 1, AB_ELTAG_GROUP) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(group)) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_NAME, ADDRITEM_NAME(group)) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_REMARKS, group->remarks) < 0)
+				data->error = TRUE;
+			if (fputs(" >\n", fp) == EOF)
+				data->error = TRUE;
 
 			/* Output email address links */
-			addrbook_write_elem_s(fp, 2, AB_ELTAG_MEMBER_LIST);
-			fputs(">\n", fp);
+			if (addrbook_write_elem_s(fp, 2, AB_ELTAG_MEMBER_LIST) < 0)
+				data->error = TRUE;
+			if (fputs(">\n", fp) == EOF)
+				data->error = TRUE;
 			node = group->listEMail;
 			while (node) {
 				ItemEMail *email = node->data;
 				ItemPerson *person = (ItemPerson *) ADDRITEM_PARENT(email);
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_MEMBER);
-				addrbook_write_attr(fp, AB_ATTAG_PID, ADDRITEM_ID(person));
-				addrbook_write_attr(fp, AB_ATTAG_EID, ADDRITEM_ID(email));
-				fputs(" />\n", fp);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_MEMBER) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_PID, ADDRITEM_ID(person)) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_EID, ADDRITEM_ID(email)) < 0)
+					data->error = TRUE;
+				if (fputs(" />\n", fp) == EOF)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
-			addrbook_write_elem_e(fp, 2, AB_ELTAG_MEMBER_LIST);
-			addrbook_write_elem_e(fp, 1, AB_ELTAG_GROUP);
+			if (addrbook_write_elem_e(fp, 2, AB_ELTAG_MEMBER_LIST) < 0)
+				data->error = TRUE;
+			if (addrbook_write_elem_e(fp, 1, AB_ELTAG_GROUP) < 0)
+				data->error = TRUE;
 		}
 	}
 }
@@ -1112,10 +1179,11 @@ static void addrbook_write_item_group_vis(gpointer key, gpointer value,
  * \param data  File pointer.
  */
 static void addrbook_write_item_folder_vis(gpointer key, gpointer value, 
-					   gpointer data)
+					   gpointer d)
 {
 	AddrItemObject *obj = (AddrItemObject *) value;
-	FILE *fp = (FILE *) data;
+	HashLoopData *data = (HashLoopData *)d;
+	FILE *fp = data->fp;
 	GList *node;
 
 	if (!obj)
@@ -1123,22 +1191,33 @@ static void addrbook_write_item_folder_vis(gpointer key, gpointer value,
 	if (ADDRITEM_TYPE(obj) == ITEMTYPE_FOLDER) {
 		ItemFolder *folder = (ItemFolder *) value;
 		if (folder) {
-			addrbook_write_elem_s(fp, 1, AB_ELTAG_FOLDER);
-			addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(folder));
-			addrbook_write_attr(fp, AB_ATTAG_NAME, ADDRITEM_NAME(folder));
-			addrbook_write_attr(fp, AB_ATTAG_REMARKS, folder->remarks);
-			fputs(" >\n", fp);
-			addrbook_write_elem_s(fp, 2, AB_ELTAG_ITEM_LIST);
-			fputs(">\n", fp);
+			if (addrbook_write_elem_s(fp, 1, AB_ELTAG_FOLDER) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(folder)) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_NAME, ADDRITEM_NAME(folder)) < 0)
+				data->error = TRUE;
+			if (addrbook_write_attr(fp, AB_ATTAG_REMARKS, folder->remarks) < 0)
+				data->error = TRUE;
+			if (fputs(" >\n", fp) == EOF)
+				data->error = TRUE;
+			if (addrbook_write_elem_s(fp, 2, AB_ELTAG_ITEM_LIST) < 0)
+				data->error = TRUE;
+			if (fputs(">\n", fp) == EOF)
+				data->error = TRUE;
 
 			/* Output persons */
 			node = folder->listPerson;
 			while (node) {
 				ItemPerson *item = node->data;
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM);
-				addrbook_write_attr(fp, AB_ATTAG_TYPE,  AB_ATTAG_VAL_PERSON);
-				addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item));
-				fputs(" />\n", fp);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_TYPE,  AB_ATTAG_VAL_PERSON) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item)) < 0)
+					data->error = TRUE;
+				if (fputs(" />\n", fp) == EOF)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
 
@@ -1146,10 +1225,14 @@ static void addrbook_write_item_folder_vis(gpointer key, gpointer value,
 			node = folder->listGroup;
 			while (node) {
 				ItemGroup *item = node->data;
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM);
-				addrbook_write_attr(fp, AB_ATTAG_TYPE, AB_ATTAG_VAL_GROUP);
-				addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item));
-				fputs(" />\n", fp);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_TYPE, AB_ATTAG_VAL_GROUP) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item)) < 0)
+					data->error = TRUE;
+				if (fputs(" />\n", fp) == EOF)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
 
@@ -1157,14 +1240,20 @@ static void addrbook_write_item_folder_vis(gpointer key, gpointer value,
 			node = folder->listFolder;
 			while (node) {
 				ItemFolder *item = node->data;
-				addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM);
-				addrbook_write_attr(fp, AB_ATTAG_TYPE, AB_ATTAG_VAL_FOLDER);
-				addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item));
-				fputs(" />\n", fp);
+				if (addrbook_write_elem_s(fp, 3, AB_ELTAG_ITEM) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_TYPE, AB_ATTAG_VAL_FOLDER) < 0)
+					data->error = TRUE;
+				if (addrbook_write_attr(fp, AB_ATTAG_UID, ADDRITEM_ID(item)) < 0)
+					data->error = TRUE;
+				if (fputs(" />\n", fp) == EOF)
+					data->error = TRUE;
 				node = g_list_next(node);
 			}
-			addrbook_write_elem_e(fp, 2, AB_ELTAG_ITEM_LIST);
-			addrbook_write_elem_e(fp, 1, AB_ELTAG_FOLDER);
+			if (addrbook_write_elem_e(fp, 2, AB_ELTAG_ITEM_LIST) < 0)
+				data->error = TRUE;
+			if (addrbook_write_elem_e(fp, 1, AB_ELTAG_FOLDER) < 0)
+				data->error = TRUE;
 		}
 	}
 }
@@ -1179,6 +1268,7 @@ static gint addrbook_write_to(AddressBookFile *book, gchar *newFile)
 {
 	FILE *fp;
 	gchar *fileSpec;
+	HashLoopData data;
 #ifndef DEV_STANDALONE
 	PrefFile *pfile;
 #endif
@@ -1193,32 +1283,52 @@ static gint addrbook_write_to(AddressBookFile *book, gchar *newFile)
 	fp = g_fopen(fileSpec, "wb");
 	g_free(fileSpec);
 	if (fp) {
-		fputs("<?xml version=\"1.0\" ?>\n", fp);
+		if (fputs("<?xml version=\"1.0\" ?>\n", fp) == EOF) {
+			book->retVal = MGU_ERROR_WRITE;
+			return book->retVal;
+		}
 #else
 	pfile = prefs_write_open(fileSpec);
 	g_free(fileSpec);
 	if (pfile) {
 		fp = pfile->fp;
-		fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL );
+		if (fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL ) < 0)
+			goto fail;
 #endif
-		addrbook_write_elem_s(fp, 0, AB_ELTAG_ADDRESS_BOOK);
-		addrbook_write_attr(fp, AB_ATTAG_NAME,
-				    addrcache_get_name(book->addressCache));
-		fputs(" >\n", fp);
+		if (addrbook_write_elem_s(fp, 0, AB_ELTAG_ADDRESS_BOOK) < 0)
+			goto fail;
+		if (addrbook_write_attr(fp, AB_ATTAG_NAME,
+				    addrcache_get_name(book->addressCache)) < 0)
+			goto fail;
+		if (fputs(" >\n", fp) == EOF)
+			goto fail;
 
 		/* Output all persons */
+		data.fp = fp;
+		data.error = FALSE;
+
 		g_hash_table_foreach(book->addressCache->itemHash, 
-				     addrbook_write_item_person_vis, fp);
+				     addrbook_write_item_person_vis, &data);
+		if (data.error)
+			goto fail;
 
 		/* Output all groups */
 		g_hash_table_foreach(book->addressCache->itemHash, 
-				     addrbook_write_item_group_vis, fp);
+				     addrbook_write_item_group_vis, &data);
+
+		if (data.error)
+			goto fail;
 
 		/* Output all folders */
 		g_hash_table_foreach(book->addressCache->itemHash, 
-				     addrbook_write_item_folder_vis, fp);
+				     addrbook_write_item_folder_vis, &data);
+
+		if (data.error)
+			goto fail;
+
+		if (addrbook_write_elem_e(fp, 0, AB_ELTAG_ADDRESS_BOOK) < 0)
+			goto fail;
 
-		addrbook_write_elem_e(fp, 0, AB_ELTAG_ADDRESS_BOOK);
 		book->retVal = MGU_SUCCESS;
 #ifdef DEV_STANDALONE
 		fclose(fp);
@@ -1230,6 +1340,12 @@ static gint addrbook_write_to(AddressBookFile *book, gchar *newFile)
 
 	fileSpec = NULL;
 	return book->retVal;
+fail:
+	g_warning("error writing AB\n");
+	book->retVal = MGU_ERROR_WRITE;
+	if (pfile)
+		prefs_file_close_revert( pfile );
+	return book->retVal;
 }
 
 /**
@@ -1839,7 +1955,7 @@ static void addrbook_chkparse_attribute(AddressBookFile *book, XMLFile *file)
 	attr = xml_get_current_tag_attr(file);
 	/* addrbook_show_attribs( attr ); */
 	element = xml_get_element(file);
-	/* printf( "\t\tattrib value : %s\n", element ); */
+	/* g_print( "\t\tattrib value : %s\n", element ); */
 }
 
 /**
@@ -2045,7 +2161,7 @@ gint addrbook_test_read_file(AddressBookFile *book, gchar *fileName)
 	if (file) {
 		book->retVal = MGU_BAD_FORMAT;
 		if (setjmp(book->jumper)) {
-			/* printf( "Caught Ya!!!\n" ); */
+			/* g_print( "Caught Ya!!!\n" ); */
 			xml_close_file(file);
 			return book->retVal;
 		}
