@@ -445,6 +445,7 @@ static GtkWidget* create_email_view(GtkListStore *store)
 	GtkCellRenderer *renderer;
 
 	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), prefs_common.use_stripes_everywhere);
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
 	        -1,
@@ -464,6 +465,7 @@ static GtkWidget* create_detail_view(GtkListStore *store)
 	GList *walk;
 
 	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), prefs_common.use_stripes_everywhere);
 	renderer = gtk_cell_renderer_text_new();
 
 	/* col 1 */
@@ -871,7 +873,7 @@ gboolean addrduplicates_delete_item_person(ItemPerson *item, AddressDataSource *
 	if(item) {
 		gchar *filename = addritem_person_get_picture(item);
 		if (filename && is_file_exist(filename))
-			g_unlink(filename);
+			claws_unlink(filename);
 		g_free(filename);
 		addritem_free_item_person(item);
 	}
