@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -405,7 +405,7 @@ static void addrgather_page_fields( gint pageNum, gchar *pageLbl ) {
 
 	/* Recurse folders */
 	top += 4;
-	checkRecurse = gtk_check_button_new_with_label( _("Include sub-folders" ) );
+	checkRecurse = gtk_check_button_new_with_label( _("Include subfolders" ) );
 	gtk_table_attach( GTK_TABLE(table), checkRecurse, 0, 2, top, (top + 1),
 			GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0 );
 
