@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -927,11 +927,16 @@ static AddressInterface *addrindex_tag_get_datasource(
  * \param lvl  Indentation level.
  * \param name Element name.
  */
-static void addrindex_write_elem_s( FILE *fp, const gint lvl, const gchar *name ) {
+static int addrindex_write_elem_s( FILE *fp, const gint lvl, const gchar *name ) {
 	gint i;
-	for( i = 0; i < lvl; i++ ) fputs( "  ", fp );
-	fputs( "<", fp );
-	fputs( name, fp );
+	for( i = 0; i < lvl; i++ ) 
+		if (fputs( "  ", fp ) == EOF)
+			return -1;
+	if (fputs( "<", fp ) == EOF)
+		return -1;
+	if (fputs( name, fp ) == EOF)
+		return -1;
+	return 0;
 }
 
 /**
@@ -940,12 +945,18 @@ static void addrindex_write_elem_s( FILE *fp, const gint lvl, const gchar *name
  * \param lvl  Indentation level.
  * \param name Element name.
  */
-static void addrindex_write_elem_e( FILE *fp, const gint lvl, const gchar *name ) {
+static int addrindex_write_elem_e( FILE *fp, const gint lvl, const gchar *name ) {
 	gint i;
-	for( i = 0; i < lvl; i++ ) fputs( "  ", fp );
-	fputs( "</", fp );
-	fputs( name, fp );
-	fputs( ">\n", fp );
+	for( i = 0; i < lvl; i++ ) 
+		if (fputs( "  ", fp ) == EOF)
+			return -1;
+	if (fputs( "</", fp ) == EOF)
+		return -1;
+	if (fputs( name, fp ) == EOF)
+		return -1;
+	if (fputs( ">\n", fp ) == EOF)
+		return -1;
+	return 0;
 }
 
 /**
@@ -954,12 +965,18 @@ static void addrindex_write_elem_e( FILE *fp, const gint lvl, const gchar *name
  * \param name  Attribute name.
  * \param value Attribute value.
  */
-static void addrindex_write_attr( FILE *fp, const gchar *name, const gchar *value ) {
-	fputs( " ", fp );
-	fputs( name, fp );
-	fputs( "=\"", fp );
-	xml_file_put_escape_str( fp, value );
-	fputs( "\"", fp );
+static int addrindex_write_attr( FILE *fp, const gchar *name, const gchar *value ) {
+	if (fputs( " ", fp ) == EOF)
+		return -1;
+	if (fputs( name, fp ) == EOF)
+		return -1;
+	if (fputs( "=\"", fp ) == EOF)
+		return -1;
+	if (xml_file_put_escape_str( fp, value ) < 0)
+		return -1;
+	if (fputs( "\"", fp ) == EOF)
+		return -1;
+	return 0;
 }
 
 /**
@@ -979,7 +996,7 @@ static AddressIfFragment *addrindex_read_fragment( XMLFile *file ) {
 	guint prevLevel;
 	gint rc;
 
-	/* printf( "addrindex_read_fragment\n" ); */
+	/* g_print( "addrindex_read_fragment\n" ); */
 
 	prevLevel = file->level;
 
@@ -1030,37 +1047,45 @@ static AddressIfFragment *addrindex_read_fragment( XMLFile *file ) {
  * \param fragment DOM fragment for configuration element.
  * \param lvl      Indent level.
  */
-static void addrindex_write_fragment(
+static int addrindex_write_fragment(
 		FILE *fp, const AddressIfFragment *fragment, const gint lvl )
 {
 	GList *node;
 
 	if( fragment ) {
-		addrindex_write_elem_s( fp, lvl, fragment->name );
+		if (addrindex_write_elem_s( fp, lvl, fragment->name ) < 0)
+			return -1;
 		node = fragment->attributes;
 		while( node ) {
 			AddressIfAttrib *nv = node->data;
-			addrindex_write_attr( fp, nv->name, nv->value );
+			if (addrindex_write_attr( fp, nv->name, nv->value ) < 0)
+				return -1;
 			node = g_list_next( node );
 		}
 		if( fragment->children ) {
-			fputs(" >\n", fp);
+			if (fputs(" >\n", fp) == EOF)
+				return -1;
 
 			/* Output children */
 			node = fragment->children;
 			while( node ) {
 				AddressIfFragment *child = node->data;
-				addrindex_write_fragment( fp, child, 1+lvl );
+				if (addrindex_write_fragment( fp, child, 1+lvl ) < 0)
+					return -1;
 				node = g_list_next( node );
 			}
 
 			/* Output closing tag */
-			addrindex_write_elem_e( fp, lvl, fragment->name );
+			if (addrindex_write_elem_e( fp, lvl, fragment->name ) < 0)
+				return -1;
 		}
 		else {
-			fputs(" />\n", fp);
+			if (fputs(" />\n", fp) == EOF)
+				return -1;
 		}
 	}
+	
+	return 0;
 }
 
 /**
@@ -1092,14 +1117,19 @@ static AddressDataSource *addrindex_parse_book( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_book( FILE *fp, AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_book( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	AddressBookFile *abf = ds->rawDataSource;
 	if( abf ) {
-		addrindex_write_elem_s( fp, lvl, TAG_DS_ADDRESS_BOOK );
-		addrindex_write_attr( fp, ATTAG_BOOK_NAME, addrbook_get_name( abf ) );
-		addrindex_write_attr( fp, ATTAG_BOOK_FILE, abf->fileName );
-		fputs( " />\n", fp );
+		if (addrindex_write_elem_s( fp, lvl, TAG_DS_ADDRESS_BOOK ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_BOOK_NAME, addrbook_get_name( abf ) ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_BOOK_FILE, abf->fileName ) < 0)
+			return -1;
+		if (fputs( " />\n", fp ) == EOF)
+			return -1;
 	}
+	return 0;
 }
 
 static AddressDataSource *addrindex_parse_vcard( XMLFile *file ) {
@@ -1125,14 +1155,19 @@ static AddressDataSource *addrindex_parse_vcard( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_vcard( FILE *fp, AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_vcard( FILE *fp, AddressDataSource *ds, gint lvl ) {
      	VCardFile *vcf = ds->rawDataSource;
 	if( vcf ) {
-		addrindex_write_elem_s( fp, lvl, TAG_DS_VCARD );
-		addrindex_write_attr( fp, ATTAG_VCARD_NAME, vcard_get_name( vcf ) );
-		addrindex_write_attr( fp, ATTAG_VCARD_FILE, vcf->path );
-		fputs( " />\n", fp );
+		if (addrindex_write_elem_s( fp, lvl, TAG_DS_VCARD ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_VCARD_NAME, vcard_get_name( vcf ) ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_VCARD_FILE, vcf->path ) < 0)
+			return -1;
+		if (fputs( " />\n", fp ) == EOF)
+			return -1;
 	}
+	return 0;
 }
 
 #ifdef USE_JPILOT
@@ -1171,27 +1206,33 @@ static AddressDataSource *addrindex_parse_jpilot( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_jpilot( FILE *fp,AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_jpilot( FILE *fp,AddressDataSource *ds, gint lvl ) {
 	JPilotFile *jpf = ds->rawDataSource;
 	if( jpf ) {
 		gint ind;
 		GList *node;
 		GList *customLbl = jpilot_get_custom_labels( jpf );
-		addrindex_write_elem_s( fp, lvl, TAG_DS_JPILOT );
-		addrindex_write_attr( fp, ATTAG_JPILOT_NAME, jpilot_get_name( jpf ) );
-		addrindex_write_attr( fp, ATTAG_JPILOT_FILE, jpf->path );
+		if (addrindex_write_elem_s( fp, lvl, TAG_DS_JPILOT ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_JPILOT_NAME, jpilot_get_name( jpf ) ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_JPILOT_FILE, jpf->path ) < 0)
+			return -1;
 		node = customLbl;
 		ind = 1;
 		while( node ) {
 			gchar name[256];
 			g_snprintf( name, sizeof(name), "%s%d",
 				    ATTAG_JPILOT_CUSTOM, ind );
-			addrindex_write_attr( fp, name, node->data );
+			if (addrindex_write_attr( fp, name, node->data ) < 0)
+				return -1;
 			ind++;
 			node = g_list_next( node );
 		}
-		fputs( " />\n", fp );
+		if (fputs( " />\n", fp ) == EOF)
+			return -1;
 	}
+	return 0;
 }
 
 #else
@@ -1206,11 +1247,13 @@ static AddressDataSource *addrindex_parse_jpilot( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_jpilot( FILE *fp, AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_jpilot( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	AddressIfFragment *fragment = ds->rawDataSource;
 	if( fragment ) {
-		addrindex_write_fragment( fp, fragment, lvl );
+		if (addrindex_write_fragment( fp, fragment, lvl ) < 0)
+			return -1;
 	}
+	return 0;
 }
 #endif
 
@@ -1306,7 +1349,7 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 	gboolean bTLS, bSSL;
 	gint iMatch;
 
-	/* printf( "addrindex_parse_ldap\n" ); */
+	/* g_print( "addrindex_parse_ldap\n" ); */
 	/* Set up some defaults */
 	bDynSearch = FALSE;
 	bTLS = FALSE;
@@ -1343,7 +1386,7 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 		else if( strcmp( name, ATTAG_LDAP_CRITERIA ) == 0 ) {
 			g_free( criteria );
 			criteria = g_strdup( value );
-			printf("criteria %s\n", criteria);
+			g_print("criteria %s\n", criteria);
 		}
 		else if( strcmp( name, ATTAG_LDAP_MAX_ENTRY ) == 0 ) {
 			ldapctl_set_max_entries( ctl, ivalue );
@@ -1410,7 +1453,7 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	LdapServer *server = ds->rawDataSource;
 	LdapControl *ctl = NULL;
 	GList *node;
@@ -1419,55 +1462,76 @@ static void addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	if( server ) {
 		ctl = server->control;
 	}
-	if( ctl == NULL ) return;
+	if( ctl == NULL ) return 0;
 
 	/* Output start element with attributes */
-	addrindex_write_elem_s( fp, lvl, TAG_DS_LDAP );
-	addrindex_write_attr( fp, ATTAG_LDAP_NAME, ldapsvr_get_name( server ) );
-	addrindex_write_attr( fp, ATTAG_LDAP_HOST, ctl->hostName );
+	if (addrindex_write_elem_s( fp, lvl, TAG_DS_LDAP ) < 0)
+		return -1;
+	if (addrindex_write_attr( fp, ATTAG_LDAP_NAME, ldapsvr_get_name( server ) ) < 0)
+		return -1;
+	if (addrindex_write_attr( fp, ATTAG_LDAP_HOST, ctl->hostName ) < 0)
+		return -1;
 
 	sprintf( value, "%d", ctl->port );	
-	addrindex_write_attr( fp, ATTAG_LDAP_PORT, value );
+	if (addrindex_write_attr( fp, ATTAG_LDAP_PORT, value ) < 0)
+		return -1;
 
-	addrindex_write_attr( fp, ATTAG_LDAP_BASE_DN, ctl->baseDN );
-	addrindex_write_attr( fp, ATTAG_LDAP_BIND_DN, ctl->bindDN );
-	addrindex_write_attr( fp, ATTAG_LDAP_BIND_PASS, ctl->bindPass );
+	if (addrindex_write_attr( fp, ATTAG_LDAP_BASE_DN, ctl->baseDN ) < 0)
+		return -1;
+	if (addrindex_write_attr( fp, ATTAG_LDAP_BIND_DN, ctl->bindDN ) < 0)
+		return -1;
+	if (addrindex_write_attr( fp, ATTAG_LDAP_BIND_PASS, ctl->bindPass ) < 0)
+		return -1;
 
 	sprintf( value, "%d", ctl->maxEntries );
-	addrindex_write_attr( fp, ATTAG_LDAP_MAX_ENTRY, value );
+	if (addrindex_write_attr( fp, ATTAG_LDAP_MAX_ENTRY, value ) < 0)
+		return -1;
 	sprintf( value, "%d", ctl->timeOut );
-	addrindex_write_attr( fp, ATTAG_LDAP_TIMEOUT, value );
+	if (addrindex_write_attr( fp, ATTAG_LDAP_TIMEOUT, value ) < 0)
+		return -1;
 	sprintf( value, "%d", ctl->maxQueryAge );
-	addrindex_write_attr( fp, ATTAG_LDAP_MAX_AGE, value );
+	if (addrindex_write_attr( fp, ATTAG_LDAP_MAX_AGE, value ) < 0)
+		return -1;
 
-	addrindex_write_attr( fp, ATTAG_LDAP_DYN_SEARCH,
+	if (addrindex_write_attr( fp, ATTAG_LDAP_DYN_SEARCH,
 			server->searchFlag ?
-			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO );
+			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO ) < 0)
+		return -1;
 
-	addrindex_write_attr( fp, ATTAG_LDAP_MATCH_OPT,
+	if (addrindex_write_attr( fp, ATTAG_LDAP_MATCH_OPT,
 		( ctl->matchingOption == LDAPCTL_MATCH_CONTAINS ) ?
-		ATVAL_LDAP_MATCH_CONTAINS : ATVAL_LDAP_MATCH_BEGIN );
+		ATVAL_LDAP_MATCH_CONTAINS : ATVAL_LDAP_MATCH_BEGIN ) < 0)
+		return -1;
 
-	addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_TLS,
+	if (addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_TLS,
 			ctl->enableTLS ?
-			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO );
-	addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_SSL,
+			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO ) < 0)
+		return -1;
+	if (addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_SSL,
 			ctl->enableSSL ?
-			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO );
+			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO ) < 0)
+		return -1;
 
-	fputs(" >\n", fp);
+	if (fputs(" >\n", fp) == EOF)
+		return -1;
 
 	/* Output attributes */
 	node = ldapctl_get_criteria_list( ctl );
 	while( node ) {
-		addrindex_write_elem_s( fp, 1+lvl, ELTAG_LDAP_ATTR_SRCH );
-		addrindex_write_attr( fp, ATTAG_LDAP_ATTR_NAME, node->data );
-		fputs(" />\n", fp);
+		if (addrindex_write_elem_s( fp, 1+lvl, ELTAG_LDAP_ATTR_SRCH ) < 0)
+			return -1;
+		if (addrindex_write_attr( fp, ATTAG_LDAP_ATTR_NAME, node->data ) < 0)
+			return -1;
+		if (fputs(" />\n", fp) == EOF)
+			return -1;
 		node = g_list_next( node );
 	}
 
 	/* End of element */	
-	addrindex_write_elem_e( fp, lvl, TAG_DS_LDAP );
+	if (addrindex_write_elem_e( fp, lvl, TAG_DS_LDAP ) < 0)
+		return -1;
+	
+	return 0;
 }
 
 #else
@@ -1482,11 +1546,13 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 	return ds;
 }
 
-static void addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
+static int addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	AddressIfFragment *fragment = ds->rawDataSource;
 	if( fragment ) {
-		addrindex_write_fragment( fp, fragment, lvl );
+		if (addrindex_write_fragment( fp, fragment, lvl ) < 0)
+			return -1;
 	}
+	return 0;
 }
 #endif
 
@@ -1604,7 +1670,7 @@ static gint addrindex_read_file( AddressIndex *addrIndex ) {
 
 	if( file == NULL ) {
 		/*
-		fprintf( stdout, " file '%s' does not exist.\n", addrIndex->fileName );
+		g_print( " file '%s' does not exist.\n", addrIndex->fileName );
 		*/
 		return addrIndex->retVal;
 	}
@@ -1625,7 +1691,7 @@ static gint addrindex_read_file( AddressIndex *addrIndex ) {
 	return addrIndex->retVal;
 }
 
-static void addrindex_write_index( AddressIndex *addrIndex, FILE *fp ) {
+static int addrindex_write_index( AddressIndex *addrIndex, FILE *fp ) {
 	GList *nodeIF, *nodeDS;
 	gint lvlList = 1;
 	gint lvlItem = 1 + lvlList;
@@ -1635,30 +1701,38 @@ static void addrindex_write_index( AddressIndex *addrIndex, FILE *fp ) {
 		AddressInterface *iface = nodeIF->data;
 		if( ! iface->legacyFlag ) {
 			nodeDS = iface->listSource;
-			addrindex_write_elem_s( fp, lvlList, iface->listTag );
-			fputs( ">\n", fp );
+			if (addrindex_write_elem_s( fp, lvlList, iface->listTag ) < 0)
+				return -1;
+			if (fputs( ">\n", fp ) == EOF)
+				return -1;
 			while( nodeDS ) {
 				AddressDataSource *ds = nodeDS->data;
 				if( ds ) {
 					if( iface->type == ADDR_IF_BOOK ) {
-						addrindex_write_book( fp, ds, lvlItem );
+						if (addrindex_write_book( fp, ds, lvlItem ) < 0)
+							return -1;
 					}
 					if( iface->type == ADDR_IF_VCARD ) {
-						addrindex_write_vcard( fp, ds, lvlItem );
+						if (addrindex_write_vcard( fp, ds, lvlItem ) < 0)
+							return -1;
 					}
 					if( iface->type == ADDR_IF_JPILOT ) {
-						addrindex_write_jpilot( fp, ds, lvlItem );
+						if (addrindex_write_jpilot( fp, ds, lvlItem ) < 0)
+							return -1;
 					}
 					if( iface->type == ADDR_IF_LDAP ) {
-						addrindex_write_ldap( fp, ds, lvlItem );
+						if (addrindex_write_ldap( fp, ds, lvlItem ) < 0)
+							return -1;
 					}
 				}
 				nodeDS = g_list_next( nodeDS );
 			}
-			addrindex_write_elem_e( fp, lvlList, iface->listTag );
+			if (addrindex_write_elem_e( fp, lvlList, iface->listTag ) < 0)
+				return -1;
 		}
 		nodeIF = g_list_next( nodeIF );
 	}
+	return 0;
 }
 
 /*
@@ -1689,13 +1763,18 @@ static gint addrindex_write_to( AddressIndex *addrIndex, const gchar *newFile )
 	g_free( fileSpec );
 	if( pfile ) {
 		fp = pfile->fp;
-		fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL );
+		if (fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL ) < 0)
+			goto fail;
 #endif
-		addrindex_write_elem_s( fp, 0, TAG_ADDRESS_INDEX );
-		fputs( ">\n", fp );
+		if (addrindex_write_elem_s( fp, 0, TAG_ADDRESS_INDEX ) < 0)
+			goto fail;
+		if (fputs( ">\n", fp ) == EOF)
+			goto fail;
 
-		addrindex_write_index( addrIndex, fp );
-		addrindex_write_elem_e( fp, 0, TAG_ADDRESS_INDEX );
+		if (addrindex_write_index( addrIndex, fp ) < 0)
+			goto fail;
+		if (addrindex_write_elem_e( fp, 0, TAG_ADDRESS_INDEX ) < 0)
+			goto fail;
 
 		addrIndex->retVal = MGU_SUCCESS;
 #ifdef DEV_STANDALONE
@@ -1709,6 +1788,12 @@ static gint addrindex_write_to( AddressIndex *addrIndex, const gchar *newFile )
 
 	fileSpec = NULL;
 	return addrIndex->retVal;
+fail:
+	g_warning("error writing AB index\n");
+	addrIndex->retVal = MGU_ERROR_WRITE;
+	if (pfile)
+		prefs_file_close_revert( pfile );
+	return addrIndex->retVal;
 }
 
 /*
@@ -1716,12 +1801,15 @@ static gint addrindex_write_to( AddressIndex *addrIndex, const gchar *newFile )
 * return: Status code, from addrIndex->retVal.
 */
 gint addrindex_save_data( AddressIndex *addrIndex ) {
-	GList *nodeIf, *nodeDS;
-
+#ifdef USE_LDAP
+	GList *nodeIf;
+	GList *nodeDS;
+#endif
+	
 	g_return_val_if_fail( addrIndex != NULL, -1 );
-	nodeIf = addrIndex->interfaceList;
 
 #ifdef USE_LDAP
+	nodeIf = addrIndex->interfaceList;
 	/* save LDAP interfaces */
 	while ( nodeIf ) {
 		AddressInterface *iface = nodeIf->data;
@@ -1914,7 +2002,7 @@ static void addrindex_add_obj( XMLFile *file, AddressCvtNode *node ) {
 			node->list = g_list_append( node->list, newNode );
 		}
 		else {
-			/* printf( "invalid: !!! \n" ); */
+			/* g_print( "invalid: !!! \n" ); */
 			attr = xml_get_current_tag_attr( file );
 		}
 	}
@@ -1935,11 +2023,11 @@ static void addrindex_consume_tree( XMLFile *file ) {
 		if (file->level < prev_level) return;
 
 		xtag = xml_get_current_tag( file );
-		/* printf( "tag : %s\n", xtag->tag ); */
+		/* g_print( "tag : %s\n", xtag->tag ); */
 		element = xml_get_element( file );
 		attr = xml_get_current_tag_attr( file );
 		/* show_attribs( attr ); */
-		/* printf( "\ttag  value : %s :\n", element ); */
+		/* g_print( "\ttag  value : %s :\n", element ); */
 		addrindex_consume_tree( file );
 	}
 }
@@ -2140,7 +2228,7 @@ static void addrindex_convert_tree( AddressIndex *addrIndex, XMLFile *file ) {
 		if (file->level < prev_level) return;
 
 		xtag = xml_get_current_tag( file );
-		/* printf( "tag : %d : %s\n", prev_level, xtag->tag ); */
+		/* g_print( "tag : %d : %s\n", prev_level, xtag->tag ); */
 		if( strcmp( xtag->tag, TAG_IF_OLD_COMMON ) == 0 ) {
 			if( addrindex_process_book( addrIndex, file, DISP_OLD_COMMON ) ) {
 				addrIndex->needsConversion = FALSE;
@@ -2160,7 +2248,7 @@ static void addrindex_convert_tree( AddressIndex *addrIndex, XMLFile *file ) {
 		element = xml_get_element( file );
 		attr = xml_get_current_tag_attr( file );
 		/* show_attribs( attr ); */
-		/* printf( "\ttag  value : %s :\n", element ); */
+		/* g_print( "\ttag  value : %s :\n", element ); */
 		addrindex_consume_tree( file );
 	}
 }
@@ -2175,7 +2263,7 @@ static gint addrindex_convert_data( AddressIndex *addrIndex ) {
 	g_free( fileSpec );
 
 	if( file == NULL ) {
-		/* fprintf( stdout, " file '%s' does not exist.\n", addrIndex->fileName ); */
+		/* g_print( " file '%s' does not exist.\n", addrIndex->fileName ); */
 		return addrIndex->retVal;
 	}
 
@@ -2371,7 +2459,7 @@ gint addrindex_ds_read_data( AddressDataSource *ds ) {
 	if( iface->getReadData ) {
 		/*
 		gchar *name = ( iface->getName ) ( ds->rawDataSource );
-		printf( "addrindex_ds_read_data...reading:::%s:::\n", name );
+		g_print( "addrindex_ds_read_data...reading:::%s:::\n", name );
 		*/
 		retVal = ( iface->getReadData ) ( ds->rawDataSource );
 	}
@@ -2494,7 +2582,7 @@ gint addrindex_setup_search(
 	queryID = req->queryID;
 	qryreq_set_search_type( req, ADDRSEARCH_DYNAMIC );
 
-	/* printf( "***> query ID ::%d::\n", queryID ); */
+	/* g_print( "***> query ID ::%d::\n", queryID ); */
 	return queryID;
 }
 
@@ -2526,7 +2614,7 @@ static gboolean addrindex_start_dynamic( QueryRequest *req ) {
 	GList *nodeDS;
 	gint type;
 
-	/* printf( "addrindex_start_dynamic::%d::\n", req->queryID ); */
+	/* g_print( "addrindex_start_dynamic::%d::\n", req->queryID ); */
 	nodeIf = _addressIndex_->searchOrder;
 	while( nodeIf ) {
 		iface = nodeIf->data;
@@ -2579,7 +2667,7 @@ void addrindex_stop_search( const gint queryID ){
 	AddrQueryObject *aqo;
 	GList *node;
 
-	/* printf( "addrindex_stop_search/queryID=%d\n", queryID ); */
+	/* g_print( "addrindex_stop_search/queryID=%d\n", queryID ); */
 	/* If query ID does not match, search has not been setup */
 	req = qrymgr_find_request( queryID );
 	if( req == NULL ) {
@@ -2701,7 +2789,7 @@ gboolean addrindex_start_search( const gint queryID ) {
 	AddrSearchType searchType;
 
 	retVal = FALSE;
-	/* printf( "addrindex_start_search/queryID=%d\n", queryID ); */
+	/* g_print( "addrindex_start_search/queryID=%d\n", queryID ); */
 	req = qrymgr_find_request( queryID );
 	if( req == NULL ) {
 		return retVal;
@@ -2728,11 +2816,11 @@ void addrindex_remove_results( AddressDataSource *ds, ItemFolder *folder ) {
 	AddressCache *cache;
 	gint queryID = 0;
 
-	/* printf( "addrindex_remove_results/start\n" ); */
+	/* g_print( "addrindex_remove_results/start\n" ); */
 
 	/* Test for folder */
 	if( folder->folderType != ADDRFOLDER_QUERY_RESULTS ) return;
-	/* printf( "folder name ::%s::\n", ADDRITEM_NAME(folder) ); */
+	/* g_print( "folder name ::%s::\n", ADDRITEM_NAME(folder) ); */
 	adbase = ( AddrBookBase * ) ds->rawDataSource;
 	if( adbase == NULL ) return;
 	cache = adbase->addressCache;
@@ -2747,23 +2835,23 @@ void addrindex_remove_results( AddressDataSource *ds, ItemFolder *folder ) {
 
 		qry = ( LdapQuery * ) folder->folderData;
 		queryID = ADDRQUERY_ID(qry);
-		/* printf( "calling ldapquery_remove_results...queryID=%d\n", queryID ); */
+		/* g_print( "calling ldapquery_remove_results...queryID=%d\n", queryID ); */
 		delFlag = ldapquery_remove_results( qry );
 		if (delFlag) {
 			ldapqry_free( qry );
 		}
-		/* printf( "calling ldapquery_remove_results...done\n" ); */
+		/* g_print( "calling ldapquery_remove_results...done\n" ); */
 		/*
 		if( delFlag ) {
-			printf( "delFlag IS-TRUE\n" );
+			g_print( "delFlag IS-TRUE\n" );
 		}
 		else {
-			printf( "delFlag IS-FALSE\n" );
+			g_print( "delFlag IS-FALSE\n" );
 		}
 		*/
 #endif
 	}
-	/* printf( "addrindex_remove_results/end\n" ); */
+	/* g_print( "addrindex_remove_results/end\n" ); */
 
 	/* Delete query request */
 	if( queryID > 0 ) {
@@ -2916,7 +3004,7 @@ gboolean addrindex_load_completion(
 
 				AddressBookFile *abf = book->rawDataSource;
 
-				debug_print("addrindex_load_completion: book %p '%s'\n", book, abf->fileName);
+				debug_print("addrindex_load_completion: book %p '%s'\n", book, abf?abf->fileName:"(null)");
 
 				addrindex_load_completion_load_persons( callBackFunc, book );
 
@@ -3028,6 +3116,127 @@ gboolean addrindex_load_person_attribute(
 	return TRUE;
 }
 
+/**
+ * This function can be used to collect information about
+ * addressbook entries
+ *
+ * \param callBackFunc Function to be called for each ItemPerson
+ * \return <i>TRUE</i>
+ */
+gboolean addrindex_load_person_ds( gint (*callBackFunc)
+			( ItemPerson *, AddressDataSource * ) )
+{
+	AddressDataSource *ds;
+	GList *nodeIf, *nodeDS;
+	GList *listP, *nodeP;
+
+	nodeIf = addrindex_get_interface_list( _addressIndex_ );
+	while( nodeIf ) {
+		AddressInterface *iface = nodeIf->data;
+
+		nodeIf = g_list_next( nodeIf );
+
+		if( ! iface->useInterface || iface->externalQuery )
+			continue;
+
+		nodeDS = iface->listSource;
+		while( nodeDS ) {
+			ds = nodeDS->data;
+
+			/* Read address book */
+			if( addrindex_ds_get_modify_flag( ds ) ) {
+				addrindex_ds_read_data( ds );
+			}
+
+			if( ! addrindex_ds_get_read_flag( ds ) ) {
+				addrindex_ds_read_data( ds );
+			}
+
+			/* Get all persons */
+			listP = addrindex_ds_get_all_persons( ds );
+			nodeP = listP;
+			while( nodeP ) {
+				ItemPerson *person = nodeP->data;
+
+				callBackFunc(person, ds);
+				nodeP = g_list_next( nodeP );
+			}
+			/* Free up the list */
+			g_list_free( listP );
+
+			nodeDS = g_list_next( nodeDS );
+		}
+	}
+	return TRUE;
+}
+
+gchar *addrindex_get_picture_file(const gchar *emailaddr)
+{
+	AddressDataSource *ds;
+	GList *nodeIf, *nodeDS;
+	GList *listP, *nodeP;
+	gboolean found = FALSE;
+	gchar *filename = NULL;
+	gchar *raw_addr = NULL;
+	
+	if (!emailaddr)
+		return NULL;
+
+	Xstrdup_a(raw_addr, emailaddr, return NULL);
+	extract_address(raw_addr);
+
+	nodeIf = addrindex_get_interface_list( _addressIndex_ );
+	while( nodeIf ) {
+		AddressInterface *iface = nodeIf->data;
+
+		nodeIf = g_list_next( nodeIf );
+
+		if( ! iface->useInterface || iface->externalQuery )
+			continue;
+
+		nodeDS = iface->listSource;
+		while( nodeDS && !found) {
+			ds = nodeDS->data;
+
+			/* Read address book */
+			if( addrindex_ds_get_modify_flag( ds ) ) {
+				addrindex_ds_read_data( ds );
+			}
+
+			if( ! addrindex_ds_get_read_flag( ds ) ) {
+				addrindex_ds_read_data( ds );
+			}
+
+			/* Get all persons */
+			listP = addrindex_ds_get_all_persons( ds );
+			nodeP = listP;
+			while( nodeP ) {
+				GList *nodeM;
+				ItemPerson *person = nodeP->data;
+				nodeM = person->listEMail;
+				while(nodeM) {
+					ItemEMail *email = nodeM->data;
+					if (email->address && !strcasecmp(raw_addr, email->address)) {
+						found = TRUE;
+						filename = g_strconcat( get_rc_dir(), G_DIR_SEPARATOR_S, 
+							ADDRBOOK_DIR, G_DIR_SEPARATOR_S, 
+							person->picture, ".png", NULL );
+						break;
+					}
+					nodeM = nodeM->next;
+				}
+				nodeP = g_list_next( nodeP );
+			}
+			/* Free up the list */
+			g_list_free( listP );
+
+			nodeDS = g_list_next( nodeDS );
+		}
+	}
+
+	return filename;
+}
+
 /*
  * End of Source.
  */
