@@ -348,7 +348,7 @@ gint session_start_tls(Session *session)
 	if (!ssl_init_socket_with_method(session->sock, SSL_METHOD_TLSv1)) {
 		g_warning("couldn't start TLS session.\n");
 		if (nb_mode)
-			sock_set_nonblocking_mode(session->sock, TRUE);
+			sock_set_nonblocking_mode(session->sock, session->nonblocking);
 		return -1;
 	}
 
