@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,7 +62,7 @@
 #include "socket.h"
 #include "utils.h"
 #include "log.h"
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 #  include "ssl.h"
 #endif
 
@@ -409,6 +409,8 @@ static gboolean sock_check(GSource *source)
 				condition |= G_IO_IN;
 		}
 	}
+#elif USE_GNUTLS
+/* ?? */
 #endif
 
 	FD_ZERO(&fds);
@@ -454,7 +456,7 @@ guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
 	sock->condition = condition;
 	sock->data = data;
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	if (sock->ssl)
 	{
 		GSource *source = g_source_new(&sock_watch_funcs,
@@ -551,7 +553,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 	if (sigsetjmp(jmpenv, 1)) {
 		alarm(0);
 		signal(SIGALRM, prev_handler);
-		fprintf(stderr, "%s: host lookup timed out.\n", hostname);
+		g_printerr("%s: host lookup timed out.\n", hostname);
 		errno = 0;
 		return NULL;
 	}
@@ -563,7 +565,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 		alarm(0);
 		signal(SIGALRM, prev_handler);
 #endif
-		fprintf(stderr, "%s: unknown host.\n", hostname);
+		g_printerr("%s: unknown host.\n", hostname);
 		errno = 0;
 		return NULL;
 	}
@@ -611,13 +613,13 @@ static gint sock_connect_by_hostname(gint sock, const gchar *hostname,
 
 	if (!my_inet_aton(hostname, &ad.sin_addr)) {
 		if ((hp = my_gethostbyname(hostname)) == NULL) {
-			fprintf(stderr, "%s: unknown host.\n", hostname);
+			g_printerr("%s: unknown host.\n", hostname);
 			errno = 0;
 			return -1;
 		}
 
 		if (hp->h_length != 4 && hp->h_length != 8) {
-			fprintf(stderr, "illegal address length received for host %s\n", hostname);
+			g_printerr("illegal address length received for host %s\n", hostname);
 			errno = 0;
 			return -1;
 		}
@@ -648,7 +650,7 @@ static gint sock_connect_by_getaddrinfo(const gchar *hostname, gushort	port)
 	g_snprintf(port_str, sizeof(port_str), "%d", port);
 
 	if ((gai_error = getaddrinfo(hostname, port_str, &hints, &res)) != 0) {
-		fprintf(stderr, "getaddrinfo for %s:%s failed: %s\n",
+		g_printerr("getaddrinfo for %s:%s failed: %s\n",
 			hostname, port_str, gai_strerror(gai_error));
 		return -1;
 	}
@@ -675,40 +677,6 @@ static gint sock_connect_by_getaddrinfo(const gchar *hostname, gushort	port)
 }
 #endif /* !INET6 */
 
-
-/* Open a connection using an external program.  May be useful when
- * you need to tunnel through a SOCKS or other firewall, or to
- * establish an IMAP-over-SSH connection. */
-/* TODO: Recreate this for sock_connect_thread() */
-SockInfo *sock_connect_cmd(const gchar *hostname, const gchar *tunnelcmd)
-{
-#ifdef G_OS_UNIX
-	gint fd[2];
-	int r;
-		     
-	if ((r = socketpair(AF_UNIX, SOCK_STREAM, 0, fd)) == -1) {
-		perror("socketpair");
-		return NULL;
-	}
-	log_message(LOG_PROTOCOL, "launching tunnel command \"%s\"\n", tunnelcmd);
-	if (fork() == 0) {
-		close(fd[0]);
-		close(0);
-		close(1);
-		dup(fd[1]);	/* set onto stdin */
-		dup(fd[1]);
-		execlp("/bin/sh", "/bin/sh", "-c", tunnelcmd, NULL);
-	}
-
-	close(fd[1]);
-	return sockinfo_from_fd(hostname, 0, fd[0]);
-#else
-        /* We would need a special implementation for W32. */
-        return NULL;
-#endif
-}
-
-
 SockInfo *sock_connect(const gchar *hostname, gushort port)
 {
 #ifdef G_OS_WIN32
@@ -1318,6 +1286,37 @@ static gint ssl_read(SSL *ssl, gchar *buf, gint len)
 		return -1;
 	}
 }
+#elif USE_GNUTLS
+static gint ssl_read(gnutls_session ssl, gchar *buf, gint len)
+{
+	gint r;
+
+	if (gnutls_record_check_pending(ssl) == 0) {
+		if (fd_check_io(GPOINTER_TO_INT(gnutls_transport_get_ptr(ssl)), G_IO_IN) < 0)
+			return -1;
+	}
+
+	while (1) {
+		r = gnutls_record_recv(ssl, buf, len);
+		if (r > 0)
+			return r;
+
+		switch (r) {
+		case 0: /* closed connection */
+			return -1;
+
+		case GNUTLS_E_AGAIN:
+		case GNUTLS_E_INTERRUPTED:
+			errno = EAGAIN;
+			return -1;
+		break;
+
+		default:
+			return -1;
+		}
+	}
+
+}
 #endif
 
 gint sock_read(SockInfo *sock, gchar *buf, gint len)
@@ -1326,7 +1325,7 @@ gint sock_read(SockInfo *sock, gchar *buf, gint len)
 
 	g_return_val_if_fail(sock != NULL, -1);
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	if (sock->ssl)
 		ret = ssl_read(sock->ssl, buf, len);
 	else
@@ -1366,6 +1365,28 @@ static gint ssl_write(SSL *ssl, const gchar *buf, gint len)
 		return -1;
 	}
 }
+#elif USE_GNUTLS
+static gint ssl_write(gnutls_session ssl, const gchar *buf, gint len)
+{
+	gint ret;
+
+	if (fd_check_io(GPOINTER_TO_INT(gnutls_transport_get_ptr(ssl)), G_IO_OUT) < 0)
+		return -1;
+
+	ret = gnutls_record_send(ssl, buf, len);
+
+	switch (ret) {
+	case 0:
+		return -1;
+	case GNUTLS_E_AGAIN:
+	case GNUTLS_E_INTERRUPTED:
+		return 0;
+
+	default:
+		return ret;
+	}
+}
+
 #endif
 
 gint sock_write(SockInfo *sock, const gchar *buf, gint len)
@@ -1374,7 +1395,7 @@ gint sock_write(SockInfo *sock, const gchar *buf, gint len)
 
 	g_return_val_if_fail(sock != NULL, -1);
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	if (sock->ssl)
 		ret = ssl_write(sock->ssl, buf, len);
 	else
@@ -1413,8 +1434,12 @@ gint fd_write_all(gint fd, const gchar *buf, gint len)
 	return wrlen;
 }
 
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 #if USE_OPENSSL
 static gint ssl_write_all(SSL *ssl, const gchar *buf, gint len)
+#else
+static gint ssl_write_all(gnutls_session ssl, const gchar *buf, gint len)
+#endif
 {
 	gint n, wrlen = 0;
 
@@ -1437,7 +1462,7 @@ gint sock_write_all(SockInfo *sock, const gchar *buf, gint len)
 
 	g_return_val_if_fail(sock != NULL, -1);
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	if (sock->ssl)
 		ret = ssl_write_all(sock->ssl, buf, len);
 	else
@@ -1504,84 +1529,6 @@ Single-byte send() and recv().
 	return bp - buf;
 }
 
-#if USE_OPENSSL
-static gint ssl_peek		(SSL *ssl, gchar *buf, gint len);
-
-static gint ssl_gets(SSL *ssl, gchar *buf, gint len)
-{
-	gchar *newline, *bp = buf;
-	gint n;
-
-	if (--len < 1)
-		return -1;
-	do {
-		if ((n = ssl_peek(ssl, bp, len)) <= 0)
-			return -1;
-		if ((newline = memchr(bp, '\n', n)) != NULL)
-			n = newline - bp + 1;
-		if ((n = ssl_read(ssl, bp, n)) < 0)
-			return -1;
-		bp += n;
-		len -= n;
-	} while (!newline && len);
-
-	*bp = '\0';
-	return bp - buf;
-}
-#endif
-
-gint sock_gets(SockInfo *sock, gchar *buf, gint len)
-{
-	gint ret;
-
-	g_return_val_if_fail(sock != NULL, -1);
-
-#if USE_OPENSSL
-	if (sock->ssl)
-		return ssl_gets(sock->ssl, buf, len);
-	else
-#endif
-		return fd_gets(sock->sock, buf, len);
-
-	if (ret < 0)
-		sock->state = CONN_DISCONNECTED;
-	return ret;
-}
-
-/* peek at the socket data without actually reading it */
-#if USE_OPENSSL
-static gint ssl_peek(SSL *ssl, gchar *buf, gint len)
-{
-	gint err, ret;
-
-	if (SSL_pending(ssl) == 0) {
-		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
-			return -1;
-	}
-
-	ret = SSL_peek(ssl, buf, len);
-
-	switch ((err = SSL_get_error(ssl, ret))) {
-	case SSL_ERROR_NONE:
-		return ret;
-	case SSL_ERROR_WANT_READ:
-	case SSL_ERROR_WANT_WRITE:
-		errno = EAGAIN;
-		return -1;
-	case SSL_ERROR_ZERO_RETURN:
-		return 0;
-	case SSL_ERROR_SYSCALL:
-		g_warning("SSL_peek() returned syscall error. errno=%d\n", errno);
-		return -1;
-	default:
-		g_warning("SSL_peek() returned error %d, ret = %d\n", err, ret);
-		if (ret == 0)
-			return 0;
-		return -1;
-	}
-}
-#endif
-
 gint sock_close(SockInfo *sock)
 {
 	gint ret;
@@ -1592,7 +1539,7 @@ gint sock_close(SockInfo *sock)
 	if (sock->sock_ch)
 		g_io_channel_unref(sock->sock_ch);
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	if (sock->ssl)
 		ssl_done_socket(sock);
 	if (sock->g_source != 0)
