@@ -123,14 +123,14 @@ static gint SSL_connect_nb(SSL *ssl)
 static gint SSL_connect_nb(gnutls_session ssl)
 #endif
 {
+#ifdef USE_GNUTLS
+	int result;
+#endif
 #ifdef USE_PTHREAD
 	thread_data *td = g_new0(thread_data, 1);
 	pthread_t pt;
 	pthread_attr_t pta;
 	void *res = NULL;
-#ifdef USE_GNUTLS
-	int result;
-#endif
 	time_t start_time = time(NULL);
 	gboolean killed = FALSE;
 	
@@ -174,12 +174,12 @@ static gint SSL_connect_nb(gnutls_session ssl)
 			GPOINTER_TO_INT(res));
 	
 	return GPOINTER_TO_INT(res);
-#else
+#else /* USE_PTHREAD */
 #ifdef USE_OPENSSL
 	return SSL_connect(ssl);
 #else
 	do {
-		result = gnutls_handshake(td->ssl);
+		result = gnutls_handshake(ssl);
 	} while (result == GNUTLS_E_AGAIN || result == GNUTLS_E_INTERRUPTED);
 #endif
 #endif
