@@ -33,16 +33,16 @@
 #include <string.h>
 #endif
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <stdio.h>
-#include <netdb.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-
 #ifdef G_OS_WIN32
-#include "winsock2.h"
-#endif
+#  include <winsock2.h>
+#else
+#  include <sys/socket.h>
+#  include <netinet/in.h>
+#  include <netdb.h>
+#endif /* G_OS_WIN32 */
 #include "ssl_certificate.h"
 #include "utils.h"
 #include "log.h"
@@ -361,7 +361,7 @@ void ssl_certificate_delete_from_disk(SSLCertificate *cert)
 	gchar *file;
 	buf = g_strdup_printf("%d", cert->port);
 	file = get_certificate_path(cert->host, buf, cert->fingerprint);
-	g_unlink (file);
+	claws_unlink (file);
 	g_free(file);
 	g_free(buf);
 }
