@@ -71,19 +71,11 @@ gint g_chmod	(const gchar	*path,
 
 #endif /* !GLIB_CHECK_VERSION */
 
-/* The AC_CHECK_SIZEOF() in configure fails for some machines.
- * we provide some fallback values here */
-#if !SIZEOF_UNSIGNED_SHORT
-  #undef SIZEOF_UNSIGNED_SHORT
-  #define SIZEOF_UNSIGNED_SHORT 2
-#endif
-#if !SIZEOF_UNSIGNED_INT
-  #undef SIZEOF_UNSIGNED_INT
-  #define SIZEOF_UNSIGNED_INT 4
-#endif
-#if !SIZEOF_UNSIGNED_LONG
-  #undef SIZEOF_UNSIGNED_LONG
-  #define SIZEOF_UNSIGNED_LONG 4
+/* why is this sometimes undefined !? */
+#ifndef G_MAXOFFSET
+typedef gint64 goffset;
+#define G_MINOFFSET	G_MININT64
+#define G_MAXOFFSET	G_MAXINT64
 #endif
 
 #ifndef HAVE_U32_TYPEDEF
@@ -243,7 +235,7 @@ gint to_number			(const gchar *nstr);
 gchar *itos_buf			(gchar	     *nstr,
 				 gint	      n);
 gchar *itos			(gint	      n);
-gchar *to_human_readable	(off_t	      size);
+gchar *to_human_readable	(goffset      size);
 
 /* alternative string functions */
 gint strcmp2		(const gchar	*s1,
@@ -527,6 +519,8 @@ void mailcap_update_default	   (const gchar *type,
 gboolean file_is_email(const gchar *filename);
 gboolean sc_g_list_bigger(GList *list, gint max);
 gboolean sc_g_slist_bigger(GSList *list, gint max);
+
+int claws_unlink(const gchar *filename);
 #ifdef __cplusplus
 }
 #endif
