@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -33,6 +33,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#ifdef G_OS_WIN32
+#  include <w32lib.h>
+#endif
+
 #include "mgutils.h"
 #include "utils.h"
 #include "exporthtml.h"
@@ -51,7 +55,7 @@
 #define BORDER_SIZE         2
 #define CELL_PADDING        2
 #define CELL_SPACING        2
-#define CHAR_ENCODING       "ISO-8859-1"
+#define CHAR_ENCODING       "UTF-8"
 
 /* Stylesheet names */
 #define FILENAME_NONE       ""
@@ -117,7 +121,7 @@ ExportHtmlCtl *exporthtml_create( void ) {
 	ctl->path = NULL;
 	ctl->dirOutput = NULL;
 	ctl->fileHtml = NULL;
-	ctl->encoding = NULL;
+	ctl->encoding = g_strconcat(CHAR_ENCODING, NULL);
 	ctl->stylesheet = EXPORT_HTML_ID_NONE;
 	ctl->nameFormat = EXPORT_HTML_FIRST_LAST;
 	ctl->banding = FALSE;
@@ -910,7 +914,9 @@ static void exporthtml_fmt_header(
 	entry = exporthtml_find_stylesheet( ctl );
 
 	fprintf( stream,
-		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n" );
+		"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n" );
+	fprintf( stream,
+                "\"http://www.w3.org/TR/html4/loose.dtd\">\n" );
 	fprintf( stream, "<html>\n" );
 	fprintf( stream, "<head>\n" );
 
@@ -931,7 +937,6 @@ static void exporthtml_fmt_header(
 			fprintf( stream, "href=\"%s\" >\n", entry->fileName );
 		}
 	}
-	fprintf( stream, "<body>\n" );
 	fprintf( stream, "</head>\n" );
 }
 
@@ -956,7 +961,7 @@ void exporthtml_process(
 	htmlFile = g_fopen( ctl->path, "wb" );
 	if( ! htmlFile ) {
 		/* Cannot open file */
-		printf( "Cannot open file for write\n" );
+		g_print( "Cannot open file for write\n" );
 		ctl->retVal = MGU_OPEN_FILE;
 		return;
 	}
@@ -979,7 +984,7 @@ void exporthtml_process(
 
 	tt = time( NULL );
 	fprintf( htmlFile, "<p>%s</p>\n", ctime_r( &tt, buf ) );
-	fprintf( htmlFile, "<hr width=\"100%%\"></hr>\n" );
+	fprintf( htmlFile, "<hr width=\"100%%\">\n" );
 
 	fprintf( htmlFile, "</body>\n" );
 	fprintf( htmlFile, "</html>\n" );
@@ -1177,7 +1182,8 @@ void exporthtml_save_settings( ExportHtmlCtl *ctl ) {
 	xmlprops_set_property_b( props, EXMLPROP_BANDING, ctl->banding );
 	xmlprops_set_property_b( props, EXMLPROP_FMT_EMAIL, ctl->linkEMail );
 	xmlprops_set_property_b( props, EXMLPROP_FMT_ATTRIB, ctl->showAttribs );
-	xmlprops_save_file( props );
+	if (xmlprops_save_file( props ) != MGU_SUCCESS)
+		g_warning("can't save settings");
 	xmlprops_free( props );
 }
 
