@@ -81,6 +81,7 @@ static GtkWidget *entry;
 static GtkWidget *ok_button;
 static GtkWidget *cancel_button;
 static GtkWidget *new_button;
+static gboolean   root_selectable;
 
 static FolderItem *folder_item;
 static FolderItem *selected_item;
@@ -140,9 +141,10 @@ static gboolean tree_view_folder_item_func	(GtkTreeModel	  *model,
 						 FolderItemSearch *data);
 
 FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
-				 const gchar *default_folder)
+				 const gchar *default_folder, gboolean can_sel_mailbox)
 {
 	selected_item = NULL;
+	root_selectable = can_sel_mailbox;
 
 	if (!window) {
 		foldersel_create();
@@ -198,7 +200,7 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	gtk_tree_store_clear(tree_store);
 
 	if (!cancelled &&
-	    selected_item && selected_item->path) {
+	    selected_item && (selected_item->path || root_selectable)) {
 		folder_item = selected_item;
 		return folder_item;
 	} else
@@ -267,6 +269,10 @@ static void foldersel_create(void)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview),
 	                             prefs_common.use_stripes_everywhere);
+#if GTK_CHECK_VERSION(2,10,0)
+	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(treeview),
+							prefs_common.enable_dotted_lines);
+#endif
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview),
 					FOLDERSEL_FOLDERNAME);
 
@@ -477,6 +483,7 @@ static void foldersel_set_tree(Folder *cur_folder, FolderSelectionType type)
 	gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
 }
 
+#include "localfolder.h"
 static gboolean foldersel_selected(GtkTreeSelection *selection,
 				   GtkTreeModel *model, GtkTreePath *path,
 				   gboolean currently_selected, gpointer data)
@@ -499,6 +506,14 @@ static gboolean foldersel_selected(GtkTreeSelection *selection,
 		id = folder_item_get_identifier(selected_item);
 		gtk_entry_set_text(GTK_ENTRY(entry), id);
 		g_free(id);
+	} else
+	if (root_selectable && selected_item && selected_item->folder &&
+			(FOLDER_TYPE(selected_item->folder) == F_MH ||
+			 FOLDER_TYPE(selected_item->folder) == F_MBOX ||
+			 FOLDER_TYPE(selected_item->folder) == F_IMAP)) {
+		gchar *id = folder_get_identifier(selected_item->folder);
+		gtk_entry_set_text(GTK_ENTRY(entry), id);
+		g_free(id);
 	} else
 		gtk_entry_set_text(GTK_ENTRY(entry), "");
 
