@@ -488,7 +488,7 @@ static gboolean grouplist_recv_func(SockInfo *sock, gint count, gint read_bytes,
 
 	g_snprintf(buf, sizeof(buf),
 		   _("%d newsgroups received (%s read)"),
-		   count, to_human_readable(read_bytes));
+		   count, to_human_readable((goffset)read_bytes));
 	gtk_label_set_text(GTK_LABEL(status_label), buf);
 	GTK_EVENTS_FLUSH();
 	if (ack == FALSE)
