@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,8 @@
 #include <stdarg.h>
 #include <sys/stat.h>
 
+#include "combobox.h"
+
 #if HAVE_LIBCOMPFACE
 #  include <compface.h>
 #endif
@@ -61,6 +63,7 @@
 #include "manage_window.h"
 #include "base64.h"
 #include "manual.h"
+#include "combobox.h"
 
 gboolean gtkut_get_font_size(GtkWidget *widget,
 			     gint *width, gint *height)
@@ -88,11 +91,17 @@ void gtkut_widget_set_small_font_size(GtkWidget *widget)
 	g_return_if_fail(widget != NULL);
 	g_return_if_fail(widget->style != NULL);
 
-	font_desc = pango_font_description_from_string(NORMAL_FONT);
-	size = pango_font_description_get_size(font_desc);
-	pango_font_description_set_size(font_desc, size * PANGO_SCALE_SMALL);
-	gtk_widget_modify_font(widget, font_desc);
-	pango_font_description_free(font_desc);
+	if (prefs_common.derive_from_normal_font || !SMALL_FONT) {
+		font_desc = pango_font_description_from_string(NORMAL_FONT);
+		size = pango_font_description_get_size(font_desc);
+		pango_font_description_set_size(font_desc, size * PANGO_SCALE_SMALL);
+		gtk_widget_modify_font(widget, font_desc);
+		pango_font_description_free(font_desc);
+	} else {
+		font_desc = pango_font_description_from_string(SMALL_FONT);
+		gtk_widget_modify_font(widget, font_desc);
+		pango_font_description_free(font_desc);
+	}
 }
 
 void gtkut_convert_int_to_gdk_color(gint rgbvalue, GdkColor *color)
@@ -116,7 +125,7 @@ void gtkut_stock_button_add_help(GtkWidget *bbox, GtkWidget **help_btn)
 	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX (bbox),
 			*help_btn, TRUE);
 	gtk_widget_set_sensitive(*help_btn,
-			manual_available(MANUAL_MANUAL_LOCAL));
+			manual_available(MANUAL_MANUAL_CLAWS));
 	gtk_widget_show(*help_btn);
 }
 
@@ -188,122 +197,23 @@ void gtkut_stock_with_text_button_set_create(GtkWidget **bbox,
 
 	if (button2) {
 		*button2 = gtk_button_new_with_mnemonic(text2);
-	gtk_button_set_image(GTK_BUTTON(*button2),
-		gtk_image_new_from_stock(label2, GTK_ICON_SIZE_BUTTON));
+		gtk_button_set_image(GTK_BUTTON(*button2),
+			gtk_image_new_from_stock(label2, GTK_ICON_SIZE_BUTTON));
+		GTK_WIDGET_SET_FLAGS(*button2, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(*bbox), *button2, TRUE, TRUE, 0);
 		gtk_widget_show(*button2);
 	}
 
 	if (button3) {
 		*button3 = gtk_button_new_with_mnemonic(text3);
-	gtk_button_set_image(GTK_BUTTON(*button3),
-		gtk_image_new_from_stock(label3, GTK_ICON_SIZE_BUTTON));
+		gtk_button_set_image(GTK_BUTTON(*button3),
+			gtk_image_new_from_stock(label3, GTK_ICON_SIZE_BUTTON));
+		GTK_WIDGET_SET_FLAGS(*button3, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(*bbox), *button3, TRUE, TRUE, 0);
 		gtk_widget_show(*button3);
 	}
 }
 
-static void combo_button_size_request(GtkWidget *widget,
-				      GtkRequisition *requisition,
-				      gpointer data)
-{
-	ComboButton *combo = (ComboButton *)data;
-
-	if (combo->arrow->allocation.height != requisition->height)
-		gtk_widget_set_size_request(combo->arrow,
-					    -1, requisition->height);
-}
-
-static void combo_button_enter(GtkWidget *widget, gpointer data)
-{
-	ComboButton *combo = (ComboButton *)data;
-
-	if (GTK_WIDGET_STATE(combo->arrow) != GTK_STATE_PRELIGHT) {
-		gtk_widget_set_state(combo->arrow, GTK_STATE_PRELIGHT);
-		gtk_widget_queue_draw(combo->arrow);
-	}
-	if (GTK_WIDGET_STATE(combo->button) != GTK_STATE_PRELIGHT) {
-		gtk_widget_set_state(combo->button, GTK_STATE_PRELIGHT);
-		gtk_widget_queue_draw(combo->button);
-	}
-}
-
-static void combo_button_leave(GtkWidget *widget, gpointer data)
-{
-	ComboButton *combo = (ComboButton *)data;
-
-	if (GTK_WIDGET_STATE(combo->arrow) != GTK_STATE_NORMAL) {
-		gtk_widget_set_state(combo->arrow, GTK_STATE_NORMAL);
-		gtk_widget_queue_draw(combo->arrow);
-	}
-	if (GTK_WIDGET_STATE(combo->button) != GTK_STATE_NORMAL) {
-		gtk_widget_set_state(combo->button, GTK_STATE_NORMAL);
-		gtk_widget_queue_draw(combo->button);
-	}
-}
-
-static gint combo_button_arrow_pressed(GtkWidget *widget, GdkEventButton *event,
-				       gpointer data)
-{
-	ComboButton *combo = (ComboButton *)data;
-
-	if (!event) return FALSE;
-
-	gtk_menu_popup(GTK_MENU(combo->menu), NULL, NULL,
-		       menu_button_position, combo->button,
-		       event->button, event->time);
-
-	return FALSE;
-}
-
-static void combo_button_destroy(GtkWidget *widget, gpointer data)
-{
-	ComboButton *combo = (ComboButton *)data;
-
-	gtk_object_destroy(GTK_OBJECT(combo->factory));
-	g_free(combo);
-}
-
-ComboButton *gtkut_combo_button_create(GtkWidget *button,
-				       GtkItemFactoryEntry *entries,
-				       gint n_entries, const gchar *path,
-				       gpointer data)
-{
-	ComboButton *combo;
-	GtkWidget *arrow;
-
-	combo = g_new0(ComboButton, 1);
-
-	combo->arrow = gtk_button_new();
-	arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-	gtk_widget_set_size_request(arrow, 7, -1);
-	gtk_container_add(GTK_CONTAINER(combo->arrow), arrow);
-	GTK_WIDGET_UNSET_FLAGS(combo->arrow, GTK_CAN_FOCUS);
-	gtk_widget_show_all(combo->arrow);
-
-	combo->button = button;
-	combo->menu = menu_create_items(entries, n_entries, path,
-					&combo->factory, data);
-	combo->data = data;
-
-	g_signal_connect(G_OBJECT(combo->button), "size_request",
-			 G_CALLBACK(combo_button_size_request), combo);
-	g_signal_connect(G_OBJECT(combo->button), "enter",
-			 G_CALLBACK(combo_button_enter), combo);
-	g_signal_connect(G_OBJECT(combo->button), "leave",
-			 G_CALLBACK(combo_button_leave), combo);
-	g_signal_connect(G_OBJECT(combo->arrow), "enter",
-			 G_CALLBACK(combo_button_enter), combo);
-	g_signal_connect(G_OBJECT(combo->arrow), "leave",
-			 G_CALLBACK(combo_button_leave), combo);
-	g_signal_connect(G_OBJECT(combo->arrow), "button_press_event",
-			 G_CALLBACK(combo_button_arrow_pressed), combo);
-	g_signal_connect(G_OBJECT(combo->arrow), "destroy",
-			 G_CALLBACK(combo_button_destroy), combo);
-
-	return combo;
-}
-
 #define CELL_SPACING 1
 #define ROW_TOP_YPIXEL(clist, row) (((clist)->row_height * (row)) + \
 				    (((row) + 1) * CELL_SPACING) + \
@@ -467,65 +377,12 @@ void gtkut_clist_set_focus_row(GtkCList *clist, gint row)
 	GTKUT_CTREE_REFRESH(clist);
 }
 
-void gtkut_combo_set_items(GtkCombo *combo, const gchar *str1, ...)
-{
-	va_list args;
-	gchar *s;
-	GList *combo_items = NULL;
-
-	g_return_if_fail(str1 != NULL);
-
-	combo_items = g_list_append(combo_items, (gpointer)str1);
-	va_start(args, str1);
-	s = va_arg(args, gchar*);
-	while (s) {
-		combo_items = g_list_append(combo_items, (gpointer)s);
-		s = va_arg(args, gchar*);
-	}
-	va_end(args);
-
-	gtk_combo_set_popdown_strings(combo, combo_items);
-
-	g_list_free(combo_items);
-}
-
-gchar *gtkut_editable_get_selection(GtkEditable *editable)
-{
-	guint start_pos, end_pos;
-	gboolean found;
-
-	g_return_val_if_fail(GTK_IS_EDITABLE(editable), NULL);
-
-	found = gtk_editable_get_selection_bounds(editable,
-						  &start_pos, &end_pos);
-	if (found)
-		return gtk_editable_get_chars(editable, start_pos, end_pos);
-	else
-		return NULL;
-}
-
-void gtkut_editable_disable_im(GtkEditable *editable)
-{
-	g_return_if_fail(editable != NULL);
-
-#if USE_XIM
-	if (editable->ic) {
-		gdk_ic_destroy(editable->ic);
-		editable->ic = NULL;
-	}
-	if (editable->ic_attr) {
-		gdk_ic_attr_destroy(editable->ic_attr);
-		editable->ic_attr = NULL;
-	}
-#endif
-}
-
 void gtkut_container_remove(GtkContainer *container, GtkWidget *widget)
 {
 	gtk_container_remove(container, widget);
 }
 
-gboolean gtkut_text_buffer_match_string(GtkTextBuffer *textbuf,
+static gboolean gtkut_text_buffer_match_string(GtkTextBuffer *textbuf,
 					const GtkTextIter *iter,
 					gunichar *wcs, gint len,
 					gboolean case_sens)
@@ -571,7 +428,7 @@ gboolean gtkut_text_buffer_match_string(GtkTextBuffer *textbuf,
 		return FALSE;
 }
 
-gboolean gtkut_text_buffer_find(GtkTextBuffer *buffer, const GtkTextIter *iter,
+static gboolean gtkut_text_buffer_find(GtkTextBuffer *buffer, const GtkTextIter *iter,
 				const gchar *str, gboolean case_sens,
 				GtkTextIter *match_pos)
 {
@@ -606,7 +463,7 @@ gboolean gtkut_text_buffer_find(GtkTextBuffer *buffer, const GtkTextIter *iter,
 	return found;
 }
 
-gboolean gtkut_text_buffer_find_backward(GtkTextBuffer *buffer,
+static gboolean gtkut_text_buffer_find_backward(GtkTextBuffer *buffer,
 					 const GtkTextIter *iter,
 					 const gchar *str, gboolean case_sens,
 					 GtkTextIter *match_pos)
@@ -701,7 +558,7 @@ gboolean gtkut_text_view_search_string(GtkTextView *text, const gchar *str,
 		gtk_text_iter_forward_chars(&end, len);
 		/* place "insert" at the last character */
 		gtk_text_buffer_select_range(buffer, &end, &match_pos);
-		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
+		gtk_text_view_scroll_to_mark(text, mark, 0.0, TRUE, 0.0, 0.5);
 		return TRUE;
 	}
 
@@ -733,7 +590,7 @@ gboolean gtkut_text_view_search_string_backward(GtkTextView *text, const gchar *
 
 		gtk_text_iter_forward_chars(&end, len);
 		gtk_text_buffer_select_range(buffer, &match_pos, &end);
-		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
+		gtk_text_view_scroll_to_mark(text, mark, 0.0, TRUE, 0.0, 0.5);
 		return TRUE;
 	}
 
@@ -757,7 +614,11 @@ void gtkut_window_popup(GtkWidget *window)
 		gdk_window_move(window->window, new_x, new_y);
 
 	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), FALSE);
+#if GTK_CHECK_VERSION(2,8,0)
+	gtk_window_present_with_time(GTK_WINDOW(window), time(NULL));
+#else
 	gtk_window_present(GTK_WINDOW(window));
+#endif
 }
 
 void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
@@ -782,7 +643,7 @@ void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
 
 void gtkut_widget_draw_now(GtkWidget *widget)
 {
-	if (GTK_WIDGET_VISIBLE(widget) && GTK_WIDGET_DRAWABLE(widget))
+	if (widget && GTK_WIDGET_VISIBLE(widget) && GTK_WIDGET_DRAWABLE(widget))
 		gdk_window_process_updates(widget->window, FALSE);
 }
 
@@ -924,21 +785,22 @@ void label_window_destroy(GtkWidget *window)
 }
 
 GtkWidget *gtkut_account_menu_new(GList			*ac_list,
-				  GCallback		 callback,
-				  gpointer		 data)
+					GCallback		callback,
+				  gpointer		data)
 {
 	GList *cur_ac;
-	GtkWidget *menu;
+	GtkWidget *optmenu;
+	GtkListStore *menu;
+	GtkTreeIter iter;
+	PrefsAccount *account;
+	gchar *name;
 	
 	g_return_val_if_fail(ac_list != NULL, NULL);
 
-	menu = gtk_menu_new();
+	optmenu = gtkut_sc_combobox_create(NULL, FALSE);
+	menu = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(optmenu)));
 
 	for (cur_ac = ac_list; cur_ac != NULL; cur_ac = cur_ac->next) {
-		gchar *name;
-		GtkWidget *menuitem;
-		PrefsAccount *account;
-		
 		account = (PrefsAccount *) cur_ac->data;
 		if (account->name)
 			name = g_strdup_printf("%s: %s <%s>",
@@ -949,13 +811,15 @@ GtkWidget *gtkut_account_menu_new(GList			*ac_list,
 			name = g_strdup_printf("%s: %s",
 					       account->account_name,
 					       account->address);
-		MENUITEM_ADD(menu, menuitem, name, account->account_id);
+		COMBOBOX_ADD_ESCAPED(menu, name, account->account_id);
 		g_free(name);
-		if (callback != NULL)
-			g_signal_connect(G_OBJECT(menuitem), "activate",
-					 callback, data);
 	}
-	return menu;
+	gtk_combo_box_set_active(GTK_COMBO_BOX(optmenu), 0);
+
+	if( callback != NULL )
+		g_signal_connect(G_OBJECT(optmenu), "changed", callback, data);
+
+	return optmenu;
 }
 
 void gtkut_set_widget_bgcolor_rgb(GtkWidget *widget, guint rgbvalue)
@@ -1223,7 +1087,8 @@ GtkWidget *xface_get_from_header(const gchar *o_xface, GdkColor *background,
 	pixmap = gdk_pixmap_create_from_xpm_d
 		(window, &mask, 
 		 background, xpm_xface);
-	return gtk_image_new_from_pixmap(pixmap, mask);
+	return gtk_image_new_from_pixbuf(
+		gdk_pixbuf_new_from_xpm_data((const char **)xpm_xface));
 }
 #endif
 
@@ -1313,7 +1178,7 @@ static void link_btn_clicked(GtkButton *button, gpointer data)
 	gchar *url = (gchar *)data;
 	gtk_button_set_relief(button, GTK_RELIEF_NONE);
 	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
-	open_uri(url, prefs_common.uri_cmd);
+	open_uri(url, prefs_common_get_uri_cmd());
 }
 
 static void link_btn_unrealize(GtkButton *button, gpointer data)
@@ -1374,20 +1239,35 @@ GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *
 	return btn;
 }
 
+static gboolean _combobox_separator_func(GtkTreeModel *model,
+		GtkTreeIter *iter, gpointer data)
+{
+	gchar *txt = NULL;
+
+	g_return_val_if_fail(model != NULL, FALSE);
+
+	gtk_tree_model_get(model, iter, COMBOBOX_TEXT, &txt, -1);
+
+	if( txt == NULL )
+		return TRUE;
+	return FALSE;
+}
+
 GtkWidget *gtkut_sc_combobox_create(GtkWidget *eventbox, gboolean focus_on_click)
 {
 	GtkWidget *combobox;
 	GtkListStore *menu;
 	GtkCellRenderer *rend;
 
-	menu = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+	menu = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
 
 	combobox = gtk_combo_box_new_with_model(GTK_TREE_MODEL(menu));
 
 	rend = gtk_cell_renderer_text_new();
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), rend, TRUE);
 	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), rend,
-			"markup", 0,
+			"markup", COMBOBOX_TEXT,
+			"sensitive", COMBOBOX_SENS,
 			NULL);
 
 	if( eventbox != NULL )
@@ -1395,6 +1275,10 @@ GtkWidget *gtkut_sc_combobox_create(GtkWidget *eventbox, gboolean focus_on_click
 #if GTK_CHECK_VERSION(2,6,0)
 	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(combobox), focus_on_click);
 #endif
+
+	gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combobox),
+			(GtkTreeViewRowSeparatorFunc)_combobox_separator_func, NULL, NULL);
+
 	return combobox;
 }
 
@@ -1612,26 +1496,20 @@ gboolean gtkut_tree_model_get_iter_last(GtkTreeModel *model,
 	return gtk_tree_model_iter_nth_child(model, iter, NULL, count - 1);
 }
 
-#ifdef MAEMO
-HildonWindow *gtkut_window_new		(GtkWindowType	 type,
-					 const gchar	*class)
-{
-	HildonWindow *window = HILDON_WINDOW(hildon_window_new());
-	gtk_window_set_role(GTK_WINDOW(window), class);
-	hildon_program_add_window(hildon_program(), window);
-	return window;
-}
-#else
 GtkWidget *gtkut_window_new		(GtkWindowType	 type,
 					 const gchar	*class)
 {
+#ifndef MAEMO
 	GtkWidget *window = gtk_window_new(type);
+#else
+	GtkWidget *window = hildon_window_new();
+	hildon_program_add_window(hildon_program, HILDON_WINDOW(window));
+#endif
 	gtk_window_set_role(GTK_WINDOW(window), class);
 	return window;
 }
-#endif
 
-gboolean gtkut_tree_iter_comp(GtkTreeModel *model, 
+static gboolean gtkut_tree_iter_comp(GtkTreeModel *model, 
 				     GtkTreeIter *iter1, 
 				     GtkTreeIter *iter2)
 {
