@@ -64,7 +64,8 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	
 	char *issuer_commonname, *issuer_location, *issuer_organization;
 	char *subject_commonname, *subject_location, *subject_organization;
-	char *fingerprint, *sig_status, *exp_date;
+	char *sig_status, *exp_date;
+	char *md5_fingerprint, *sha1_fingerprint, *fingerprint;
 	unsigned int n;
 	char buf[100];
 	unsigned char md[128];	
@@ -197,19 +198,24 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 
 	memset(buf, 0, sizeof(buf));
 	strftime(buf, sizeof(buf)-1, prefs_common.date_format, localtime_r(&exp_time_t, &lt));
-	exp_date = buf? g_strdup(buf):g_strdup("?");
+	exp_date = (*buf) ? g_strdup(buf):g_strdup("?");
 
 	/* fingerprint */
 #if USE_OPENSSL
 	X509_digest(cert->x509_cert, EVP_md5(), md, &n);
-	fingerprint = readable_fingerprint(md, (int)n);
+	md5_fingerprint = readable_fingerprint(md, (int)n);
+	X509_digest(cert->x509_cert, EVP_sha1(), md, &n);
+	sha1_fingerprint = readable_fingerprint(md, (int)n);
 
 	/* signature */
 	sig_status = ssl_certificate_check_signer(cert->x509_cert);
 #else
 	n = 128;
 	gnutls_x509_crt_get_fingerprint(cert->x509_cert, GNUTLS_DIG_MD5, md, &n);
-	fingerprint = readable_fingerprint(md, (int)n);
+	md5_fingerprint = readable_fingerprint(md, (int)n);
+	n = 128;
+	gnutls_x509_crt_get_fingerprint(cert->x509_cert, GNUTLS_DIG_SHA1, md, &n);
+	sha1_fingerprint = readable_fingerprint(md, (int)n);
 
 	/* signature */
 	sig_status = ssl_certificate_check_signer(cert->x509_cert, cert->status);
@@ -233,6 +239,7 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(owner_table, label, 0, 1, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(subject_commonname);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(owner_table, label, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -240,6 +247,7 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(owner_table, label, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(subject_organization);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(owner_table, label, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -247,6 +255,7 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(owner_table, label, 0, 1, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(subject_location);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(owner_table, label, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
@@ -254,6 +263,7 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(signer_table, label, 0, 1, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(issuer_commonname);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(signer_table, label, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -261,6 +271,7 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(signer_table, label, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(issuer_organization);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(signer_table, label, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -268,25 +279,32 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(signer_table, label, 0, 1, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(issuer_location);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(signer_table, label, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-	label = gtk_label_new(_("Fingerprint: "));
+	label = gtk_label_new(_("Fingerprint: \n"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(status_table, label, 0, 1, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	fingerprint = g_strdup_printf("MD5: %s\nSHA1: %s", 
+			md5_fingerprint, sha1_fingerprint);
 	label = gtk_label_new(fingerprint);
+	g_free(fingerprint);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(status_table, label, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(_("Signature status: "));
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(status_table, label, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(sig_status);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(status_table, label, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(_("Expires on: "));
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 	gtk_table_attach(status_table, label, 0, 1, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(exp_date);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach(status_table, label, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -307,7 +325,8 @@ static GtkWidget *cert_presenter(SSLCertificate *cert)
 	g_free(subject_commonname);
 	g_free(subject_location);
 	g_free(subject_organization);
-	g_free(fingerprint);
+	g_free(md5_fingerprint);
+	g_free(sha1_fingerprint);
 	g_free(sig_status);
 	g_free(exp_date);
 	return vbox;
@@ -376,6 +395,7 @@ static gboolean sslcertwindow_ask_new_cert(SSLCertificate *cert)
 
 	buf = g_strdup_printf(_("Signature status: %s"), sig_status);
 	label = gtk_label_new(buf);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 	g_free(buf);
@@ -420,6 +440,7 @@ static gboolean sslcertwindow_ask_expired_cert(SSLCertificate *cert)
 
 	buf = g_strdup_printf(_("Signature status: %s"), sig_status);
 	label = gtk_label_new(buf);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 	g_free(buf);
@@ -478,6 +499,7 @@ static gboolean sslcertwindow_ask_changed_cert(SSLCertificate *old_cert, SSLCert
 
 	buf = g_strdup_printf(_("Signature status: %s"), sig_status);
 	label = gtk_label_new(buf);
+	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, 0);
 	g_free(buf);
