@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -61,9 +61,28 @@
 #include "log.h"
 #include "hooks.h"
 
+#ifdef MAEMO
+#ifdef CHINOOK
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-sound.h>
+#else
+#include <hildon-widgets/hildon-banner.h>
+#include <hildon-widgets/hildon-system-sound.h>
+#endif
+#include <libosso.h>
+
+#ifdef CONIC
+#include <conicconnection.h>
+#include <conicconnectionevent.h>
+
+static ConIcConnection *maemo_connection = NULL;
+static gboolean maemo_warned_offline = FALSE;
+#endif
+#endif
+
 static GList *inc_dialog_list = NULL;
 
-static guint inc_lock_count = 0;
+guint inc_lock_count = 0;
 
 static GdkPixbuf *currentpix;
 static GdkPixbuf *errorpix;
@@ -186,6 +205,7 @@ void inc_mail(MainWindow *mainwin, gboolean notify)
 		if (execute_command_line(prefs_common.extinc_cmd, FALSE) < 0) {
 			main_window_unlock(mainwin);
 			inc_autocheck_timer_set();
+			inc_unlock();
 			return;
 		}
 	} else {
@@ -322,6 +342,17 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck,
 		return;
 	}
 
+	if (prefs_common.use_extinc && prefs_common.extinc_cmd) {
+		/* external incorporating program */
+		if (execute_command_line(prefs_common.extinc_cmd, FALSE) < 0) {
+			log_error(LOG_PROTOCOL, _("%s failed\n"), prefs_common.extinc_cmd);
+			
+			main_window_unlock(mainwin);
+			inc_autocheck_timer_set();
+			return;
+		}
+	}
+	
 	/* check local folders */
 	account_new_msgs = inc_all_spool();
 	if (account_new_msgs > 0)
@@ -596,7 +627,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 						 "Done (%d messages (%s) received)",
 					 pop3_session->cur_total_num),
 					 pop3_session->cur_total_num,
-					 to_human_readable(pop3_session->cur_total_recv_bytes));
+					 to_human_readable((goffset)pop3_session->cur_total_recv_bytes));
 			else
 				msg = g_strdup_printf(_("Done (no new messages)"));
 			SET_PIXMAP_AND_TEXT(okpix, msg);
@@ -760,7 +791,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 	g_free(buf);
 
 	server = pop3_session->ac_prefs->recv_server;
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	port = pop3_session->ac_prefs->set_popport ?
 		pop3_session->ac_prefs->popport :
 		pop3_session->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
@@ -787,7 +818,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 #endif
 
 	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."), server);
-	statusbar_print_all("%s", buf);
+	statuswindow_print_all("%s", buf);
 	log_message(LOG_PROTOCOL, "%s\n", buf);
 
 	progress_dialog_set_label(inc_dialog->dialog, buf);
@@ -811,7 +842,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 			    server, port);
 		}
 		session->inc_state = INC_CONNECT_ERROR;
-		statusbar_pop_all();
+		statuswindow_pop_all();
 		return INC_CONNECT_ERROR;
 	}
 
@@ -888,7 +919,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 	case POP3_GETAUTH_PASS:
 	case POP3_GETAUTH_APOP:
 		progress_dialog_set_label(dialog, _("Authenticating..."));
-		statusbar_pop_all();
+		statuswindow_pop_all();
 		statusbar_print_all(_("Retrieving messages from %s (%s) ..."),
 				    SESSION(session)->server,
 				    session->ac_prefs->account_name);
@@ -937,11 +968,11 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	if (pop3_session->state == POP3_RETR ||
 	    pop3_session->state == POP3_RETR_RECV ||
 	    pop3_session->state == POP3_DELETE) {
-		Xstrdup_a(total_size_str, to_human_readable(total), return);
+		Xstrdup_a(total_size_str, to_human_readable((goffset)total), return);
 		g_snprintf(buf, sizeof(buf),
 			   _("Retrieving message (%d / %d) (%s / %s)"),
 			   pop3_session->cur_msg, pop3_session->count,
-			   to_human_readable(cur_total), total_size_str);
+			   to_human_readable((goffset)cur_total), total_size_str);
 		progress_dialog_set_label(inc_dialog->dialog, buf);
 	}
 
@@ -963,7 +994,7 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 				    pop3_session->cur_total_num),
 			   pop3_session->cur_total_num,
 			   to_human_readable
-			   (pop3_session->cur_total_recv_bytes));
+			   ((goffset)pop3_session->cur_total_recv_bytes));
 		progress_dialog_list_set_status(inc_dialog->dialog,
 						inc_dialog->cur_row,
 						buf);
@@ -1089,7 +1120,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	} else
 		inbox = folder_get_default_inbox();
 	if (!inbox) {
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 
@@ -1099,7 +1130,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	/* add msg file to drop folder */
 	if ((msgnum = folder_item_add_msg(
 			dropfolder, file, NULL, TRUE)) < 0) {
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 
@@ -1182,7 +1213,9 @@ static void inc_put_error(IncState istate, Pop3Session *session)
 			err_msg = g_strdup(log_msg);
 		break;
 	case INC_TIMEOUT:
-		log_msg = _("Session timed out.");
+		log_msg = _("Session timed out. You may be able to "
+			    "recover by increasing the timeout value in "
+			    "Preferences/Other.");
 		if (prefs_common.no_recv_err_panel)
 			break;
 		err_msg = g_strdup_printf(_("Connection to %s:%d timed out."), 
@@ -1203,7 +1236,7 @@ static void inc_put_error(IncState istate, Pop3Session *session)
 		mainwindow_show_error();
 
 	if (err_msg) {
-		alertpanel_error_log(err_msg);
+		alertpanel_error_log("%s", err_msg);
 		g_free(err_msg);
 	}
 }
@@ -1268,17 +1301,22 @@ static gint inc_spool_account(PrefsAccount *account)
 	} else
 		inbox = folder_get_default_inbox();
 
-	if (is_file_exist(account->local_mbox))
-		mbox = g_strdup(account->local_mbox);
-	else if (is_dir_exist(account->local_mbox)) 
-		mbox = g_strconcat(account->local_mbox, G_DIR_SEPARATOR_S,
-				   g_get_user_name(), NULL);
-	else {
-		debug_print("%s: local mailbox not found.\n", 
-			    account->local_mbox);
+	if (account->local_mbox) {
+		if (is_file_exist(account->local_mbox))
+			mbox = g_strdup(account->local_mbox);
+		else if (is_dir_exist(account->local_mbox)) 
+			mbox = g_strconcat(account->local_mbox, G_DIR_SEPARATOR_S,
+					   g_get_user_name(), NULL);
+		else {
+			debug_print("%s: local mailbox not found.\n", 
+				    account->local_mbox);
+			return -1;
+		}
+	} else {
+		debug_print("local mailbox not set in account info.\n");
 		return -1;
-	}
-	
+	}	
+
 	result = get_spool(inbox, mbox, account);
 	g_free(mbox);
 	
@@ -1342,19 +1380,19 @@ static gint get_spool(FolderItem *dest, const gchar *mbox, PrefsAccount *account
 
 	msgs = proc_mbox(dest, tmp_mbox, account->filter_on_recv, account);
 
-	g_unlink(tmp_mbox);
+	claws_unlink(tmp_mbox);
 	if (msgs >= 0) empty_mbox(mbox);
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
 	return msgs;
 }
 
-void inc_lock(void)
+void inc_lock_real(void)
 {
 	inc_lock_count++;
 }
 
-void inc_unlock(void)
+void inc_unlock_real(void)
 {
 	if (inc_lock_count > 0)
 		inc_lock_count--;
@@ -1363,15 +1401,21 @@ void inc_unlock(void)
 static guint autocheck_timer = 0;
 static gpointer autocheck_data = NULL;
 
+#ifdef MAEMO
+osso_context_t *get_osso_context(void);
+#endif
+
 static void inc_notify_cmd(gint new_msgs, gboolean notify)
 {
 
+#ifndef MAEMO
 	gchar *buf, *numpos, *ret_str;
 	gssize by_read = 0, by_written = 0;
 
 	if (!(new_msgs && notify && prefs_common.newmail_notify_cmd &&
 	    *prefs_common.newmail_notify_cmd))
 		     return;
+
 	buf = g_strdup(prefs_common.newmail_notify_cmd);
 	if ((numpos = strstr(buf, "%d")) != NULL) {
 		gchar *buf2;
@@ -1392,12 +1436,65 @@ static void inc_notify_cmd(gint new_msgs, gboolean notify)
 	execute_command_line(buf, TRUE);
 
 	g_free(buf);
+
+#else
+	if (new_msgs) {
+		if (prefs_common.maemo_play_sound)
+			hildon_play_system_sound("/usr/share/sounds/ui-new_email.wav");
+		if (prefs_common.maemo_show_banner) {
+			gchar *info = g_strdup_printf(ngettext("Claws Mail: %d new message",
+					  	   "Claws Mail: %d new messages",
+					  	   new_msgs), new_msgs);
+			osso_system_note_infoprint(get_osso_context(), info, NULL);
+			g_free(info);
+		}
+	}
+#endif
 }
- 
+
+#if (defined(MAEMO) && defined(CONIC))
+static void maemo_connection_event(ConIcConnection *connection, 
+                     		   ConIcConnectionEvent *event,
+				   gpointer user_data)
+{
+	ConIcConnectionStatus status =
+		con_ic_connection_event_get_status(event);
+	MainWindow *mainwin = (MainWindow *)user_data;
+	switch(status) {
+	case CON_IC_STATUS_CONNECTED:
+		debug_print("we're connected\n");
+		main_window_toggle_work_offline(mainwin, FALSE, FALSE);
+		break;
+	default:
+		debug_print("we're disconnected\n");
+		main_window_toggle_work_offline(mainwin, TRUE, FALSE);
+		maemo_warned_offline = FALSE;
+		break;
+	}
+}
+
+#endif
+
 void inc_autocheck_timer_init(MainWindow *mainwin)
 {
+#if (defined(MAEMO) && defined(CONIC))
+	GValue *val = g_new0(GValue, 1);
+	maemo_connection = con_ic_connection_new();
+
+	g_value_init(val, G_TYPE_BOOLEAN);
+	g_value_set_boolean(val, TRUE);
+	g_object_set_property(G_OBJECT(maemo_connection),
+			"automatic-connection-events", val);
+	g_free(val);	
+	g_signal_connect (maemo_connection, "connection-event",
+			  G_CALLBACK(maemo_connection_event), mainwin);	
+	con_ic_connection_connect (maemo_connection,
+		CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED);
+	autocheck_data = mainwin;
+#else
 	autocheck_data = mainwin;
 	inc_autocheck_timer_set();
+#endif
 }
 
 static void inc_autocheck_timer_set_interval(guint interval)
@@ -1416,6 +1513,10 @@ static void inc_autocheck_timer_set_interval(guint interval)
 void inc_autocheck_timer_set(void)
 {
 	inc_autocheck_timer_set_interval(prefs_common.autochk_itv * 60000);
+#if (defined(MAEMO) && defined(CONIC))
+	con_ic_connection_connect (maemo_connection,
+		CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED);
+#endif
 }
 
 void inc_autocheck_timer_remove(void)
@@ -1448,7 +1549,29 @@ gboolean inc_offline_should_override(gboolean force_ask, const gchar *msg)
 	static time_t overridden_no  = 0;
 	int length = 10; /* minutes */
 	gint answer = G_ALERTDEFAULT;
-	
+
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	/* If no network connection is available, override is not possible */
+	if(!networkmanager_is_online(NULL))
+		return FALSE;
+#endif
+
+#if (defined(MAEMO) && defined(CONIC))
+	if (prefs_common.work_offline) {
+		if (force_ask && !maemo_warned_offline) {
+			if (mainwindow_get_mainwindow())
+				hildon_banner_show_information(
+					mainwindow_get_mainwindow()->window, 
+					NULL,
+					_("Unable to connect: you are offline."));
+			maemo_warned_offline = TRUE;
+		}
+		return FALSE;
+	} else {
+		return TRUE;
+	}
+#endif	
+
 	if (prefs_common.autochk_newmail)
 		length = prefs_common.autochk_itv; /* minutes */
 
