@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -32,6 +32,7 @@
 #include <string.h>
 #include <lber.h>
 
+#include "defs.h"
 #include "ldaputil.h"
 #include "ldapquery.h"
 #include "ldapctrl.h"
@@ -39,6 +40,7 @@
 
 #include "addritem.h"
 #include "addrcache.h"
+#include "common/utils.h"
 
 /*
  * Key for thread specific data.
@@ -113,8 +115,12 @@ void ldapqry_set_control( LdapQuery *qry, LdapControl *ctl ) {
  * \param value Name.
  */
 void ldapqry_set_name( LdapQuery* qry, const gchar *value ) {
+	g_return_if_fail( qry != NULL );
 	ADDRQUERY_NAME(qry) = mgu_replace_string( ADDRQUERY_NAME(qry), value );
+	if (ADDRQUERY_NAME(qry) == NULL)
+		return;
 	g_strstrip( ADDRQUERY_NAME(qry) );
+	debug_print("set name: %s\n", ADDRQUERY_NAME(qry));
 }
 
 /**
@@ -123,8 +129,12 @@ void ldapqry_set_name( LdapQuery* qry, const gchar *value ) {
  * \param value 
  */
 void ldapqry_set_search_value( LdapQuery *qry, const gchar *value ) {
+	g_return_if_fail( qry != NULL );
 	ADDRQUERY_SEARCHVALUE(qry) = mgu_replace_string( ADDRQUERY_SEARCHVALUE(qry), value );
+	if (ADDRQUERY_SEARCHVALUE(qry) == NULL)
+		return;
 	g_strstrip( ADDRQUERY_SEARCHVALUE(qry) );
+	debug_print("search value: %s\n", ADDRQUERY_SEARCHVALUE(qry));
 }
 
 /**
@@ -159,6 +169,7 @@ void ldapqry_set_search_type( LdapQuery *qry, const AddrSearchType value ) {
  * \param value ID for the query.
  */
 void ldapqry_set_query_id( LdapQuery* qry, const gint value ) {
+	g_return_if_fail( qry != NULL );
 	ADDRQUERY_ID(qry) = value;
 }
 
@@ -382,7 +393,8 @@ static GSList *ldapqry_add_list_values(
 
 	if( ( vals = ldap_get_values_len( ld, entry, attr ) ) != NULL ) {
 		for( i = 0; vals[i] != NULL; i++ ) {
-			/* printf( "lv\t%s: %s\n", attr, vals[i] ); */
+			/*debug_print("lv\t%s: %s\n", attr?attr:"null",
+					vals[i]->bv_val?vals[i]->bv_val:"null");*/
 			list = g_slist_append( list, g_strndup( vals[i]->bv_val, vals[i]->bv_len) );
 		}
 	}
@@ -403,8 +415,19 @@ static GSList *ldapqry_add_single_value( LDAP *ld, LDAPMessage *entry, char *att
 
 	if( ( vals = ldap_get_values_len( ld, entry, attr ) ) != NULL ) {
 		if( vals[0] != NULL ) {
-			/* printf( "sv\t%s: %s\n", attr, vals[0] ); */
-			list = g_slist_append( list, g_strndup( vals[0]->bv_val, vals[0]->bv_len ));
+			if (strcmp(attr, "jpegPhoto")) {
+				debug_print("sv\t%s: %s\n", attr?attr:"null",
+						vals[0]->bv_val?vals[0]->bv_val:"null");
+				list = g_slist_append( list, g_strndup( vals[0]->bv_val, vals[0]->bv_len ));
+			} else {
+				char *file = get_tmp_file();
+				FILE *fp = fopen(file, "wb");
+				if (fp) {
+					fwrite(vals[0]->bv_val, 1, vals[0]->bv_len, fp);
+					fclose(fp);
+				}
+				list = g_slist_append( list, file);
+			}
 		}
 	}
 	ldap_value_free_len( vals );
@@ -443,6 +466,7 @@ static GList *ldapqry_build_items_fl(
 	ItemPerson *person;
 	ItemEMail *email;
 	ItemFolder *folder;
+	gchar *picfile = NULL;
 	GList *listReturn = NULL;
 
 	folder = ADDRQUERY_FOLDER(qry);
@@ -499,7 +523,24 @@ static GList *ldapqry_build_items_fl(
 	
 	for (cur = attributes; cur; cur = cur->next) {
 		UserAttribute *attrib = addritem_copy_attribute((UserAttribute *)cur->data);
-		addritem_person_add_attribute( person, attrib );
+		if (attrib->name && strcmp(attrib->name, "jpegPhoto")) {
+			addritem_person_add_attribute( person, attrib );
+		} else {
+			if (qry && qry->server && qry->server->control) {
+				gchar *dir = g_strconcat( get_rc_dir(), G_DIR_SEPARATOR_S, 
+							ADDRBOOK_DIR, G_DIR_SEPARATOR_S, NULL );
+				gchar *filename = g_strdup_printf("%s-%s-%s",
+					qry->server->control->hostName?qry->server->control->hostName:"nohost", 
+					qry->server->control->baseDN?qry->server->control->baseDN:"nobase", 
+					dn);
+				picfile = g_strdup_printf("%s%s.png", dir, filename);
+				addritem_person_set_picture( person, filename );
+				rename_force(attrib->value, picfile);
+				g_free(filename);
+				g_free(picfile);
+				g_free(dir);
+			}
+		}
 	}
 	
 	addrcache_folder_add_person( cache, ADDRQUERY_FOLDER(qry), person );
@@ -514,7 +555,9 @@ static GList *ldapqry_build_items_fl(
 		addrcache_id_email( cache, email );
 		addrcache_person_add_email( cache, person, email );
 		addritem_person_add_email( person, email );
-		/*addritem_print_item_email(email, stdout);*/
+		/*if (debug_get_mode()) {
+			addritem_print_item_email(email, stdout);
+		}*/
 		listReturn = g_list_append( listReturn, email );
 		nodeAddress = g_slist_next( nodeAddress );
 	}
@@ -552,7 +595,7 @@ static GList *ldapqry_process_single_entry(
 	listReturn = NULL;
 	ctl = qry->control;
 	dnEntry = ldap_get_dn( ld, e );
-	/* printf( "DN: %s\n", dnEntry ); */
+	debug_print( "DN: %s\n", dnEntry?dnEntry:"null" );
 
 	/* Process all attributes */
 	for( attribute = ldap_first_attribute( ld, e, &ber ); attribute != NULL;
@@ -650,16 +693,22 @@ static gint ldapqry_connect( LdapQuery *qry ) {
 	gchar *uri = NULL;
 
 	/* Initialize connection */
-	/* printf( "===ldapqry_connect===\n" ); */
-	/* ldapqry_print( qry, stdout ); */
+	if (debug_get_mode()) {
+		debug_print("===ldapqry_connect===\n");
+		/*ldapqry_print(qry, stdout);*/
+	}
 	ctl = qry->control;
-	/* ldapctl_print( ctl, stdout ); */
-	/* printf( "======\n" ); */
+	/*if (debug_get_mode()) {
+		ldapctl_print(ctl, stdout);
+		debug_print("======\n");
+	}*/
 	ldapqry_touch( qry );
 	qry->startTime = qry->touchTime;
 	qry->elapsedTime = -1;
 	ADDRQUERY_RETVAL(qry) = LDAPRC_INIT;
 
+	ldapsrv_set_options (ctl->timeOut, NULL);
+
 	uri = g_strdup_printf("ldap%s://%s:%d",
 				ctl->enableSSL?"s":"",
 				ctl->hostName, ctl->port);
@@ -676,9 +725,8 @@ static gint ldapqry_connect( LdapQuery *qry ) {
 	}
 	ldapqry_touch( qry );
 
-	/*
-	printf( "connected to LDAP host %s on port %d\n", ctl->hostName, ctl->port );
-	*/
+	debug_print("connected to LDAP host %s on port %d\n",
+			ctl->hostName?ctl->hostName:"null", ctl->port);
 
 #ifdef USE_LDAP_TLS
 	/* Handle TLS */
@@ -693,9 +741,9 @@ static gint ldapqry_connect( LdapQuery *qry ) {
 			ADDRQUERY_RETVAL(qry) = LDAPRC_TLS;
 			rc = ldap_start_tls_s( ld, NULL, NULL );
 			
-			/* printf( "rc=%d\n", rc );
-			printf( "LDAP Status: set_option: %s\n", ldap_err2string( rc ) ); */
-			
+			debug_print("rc=%d\n", rc);
+			debug_print("LDAP Status: set_option: %s\n", ldap_err2string(rc));
+
 			if( rc != LDAP_SUCCESS ) {
 				return ADDRQUERY_RETVAL(qry);
 			}
@@ -707,14 +755,11 @@ static gint ldapqry_connect( LdapQuery *qry ) {
 	ADDRQUERY_RETVAL(qry) = LDAPRC_BIND;
 	if( ctl->bindDN ) {
 		if( * ctl->bindDN != '\0' ) {
-			/* printf( "binding...\n" ); */
+			debug_print("binding...\n");
 			rc = claws_ldap_simple_bind_s( ld, ctl->bindDN, ctl->bindPass );
-			/* printf( "rc=%d\n", rc ); */
+			debug_print("rc=%d\n", rc);
 			if( rc != LDAP_SUCCESS ) {
-				/*
-				printf( "LDAP Error: ldap_simple_bind_s: %s\n",
-					ldap_err2string( rc ) );
-				*/
+				debug_print("LDAP Error: ldap_simple_bind_s: %s\n",	ldap_err2string(rc));
 				return ADDRQUERY_RETVAL(qry);
 			}
 		}
@@ -780,11 +825,11 @@ static gint ldapqry_search_retrieve( LdapQuery *qry ) {
 	ADDRQUERY_RETVAL(qry) = LDAPRC_SUCCESS;
 
 	/* Define all attributes we are interested in. */
-	attribs = ldapctl_attribute_array( ctl );
+	attribs = ldapctl_full_attribute_array( ctl );
 
 	/* Create LDAP search string */
 	criteria = ldapctl_format_criteria( ctl, ADDRQUERY_SEARCHVALUE(qry) );
-	/* printf( "Search criteria ::%s::\n", criteria ); */
+	debug_print("Search criteria ::%s::\n", criteria?criteria:"null");
 
 	/*
 	 * Execute the search - this step may take some time to complete
@@ -793,6 +838,8 @@ static gint ldapqry_search_retrieve( LdapQuery *qry ) {
 	ADDRQUERY_RETVAL(qry) = LDAPRC_TIMEOUT;
 	rc = ldap_search_ext_s( ld, ctl->baseDN, LDAP_SCOPE_SUBTREE, criteria,
 		attribs, 0, NULL, NULL, &timeout, 0, &result );
+	debug_print("LDAP Error: ldap_search_st: %d\n", rc);
+	debug_print("LDAP Error: ldap_search_st: %s\n", ldap_err2string(rc));
 	ldapctl_free_attribute_array( attribs );
 	g_free( criteria );
 	criteria = NULL;
@@ -813,17 +860,13 @@ static gint ldapqry_search_retrieve( LdapQuery *qry ) {
 		searchFlag = TRUE;
 	}
 	else {
-		/*
-		printf( "LDAP Error: ldap_search_st: %d\n", rc );
-		printf( "LDAP Error: ldap_search_st: %s\n", ldap_err2string( rc ) );
-		*/
+		debug_print("LDAP Error: ldap_search_st: %d\n", rc);
+		debug_print("LDAP Error: ldap_search_st: %s\n", ldap_err2string(rc));
 		return ADDRQUERY_RETVAL(qry);
 	}
 	ADDRQUERY_RETVAL(qry) = LDAPRC_STOP_FLAG;
 
-	/*
-	printf( "Total results are: %d\n", ldap_count_entries( ld, result ) );
-	*/
+	debug_print("Total results are: %d\n", ldap_count_entries(ld, result));
 
 	/* Process results */
 	first = TRUE;
@@ -858,10 +901,12 @@ static gint ldapqry_search_retrieve( LdapQuery *qry ) {
 			qry->callBackEntry( qry, ADDRQUERY_ID(qry), listEMail, qry->data );
 		}
 		else {
-			/*GList *node = listEMail;
-			while (node) {
-				addritem_print_item_email(node->data, stdout);
-				node = g_list_next(node);
+			/*if (debug_get_mode()) {
+				GList *node = listEMail;
+				while (node) {
+					addritem_print_item_email(node->data, stdout);
+					node = g_list_next(node);
+				}
 			}*/
 			g_list_free( listEMail );
 		}
@@ -944,16 +989,12 @@ static gint ldapqry_search( LdapQuery *qry ) {
 		qry->server->addressCache->dataRead = TRUE;
 		qry->server->addressCache->accessFlag = FALSE;
 		if( ldapqry_get_stop_flag( qry ) ) {
-			/*
-			printf( "Search was terminated prematurely\n" );
-			*/
+			debug_print("Search was terminated prematurely\n");
 		}
 		else {
 			ldapqry_touch( qry );
 			qry->completed = TRUE;
-			/*
-			printf( "Search ran to completion\n" );
-			*/
+			debug_print("Search ran to completion\n");
 		}
 	}
 	ldapqry_set_stop_flag( qry, TRUE );
@@ -981,9 +1022,7 @@ gint ldapqry_read_data_th( LdapQuery *qry ) {
 	ldapqry_touch( qry );
 	if( ldapqry_check_search( qry ) ) {
 		if( ADDRQUERY_RETVAL(qry) == LDAPRC_SUCCESS ) {
-			/*
-			printf( "Starting LDAP search thread\n");
-			*/
+			debug_print("Starting LDAP search thread\n");
 			ldapqry_set_busy_flag( qry, TRUE );
 			qry->thread = g_malloc0( sizeof( pthread_t ) );
 
@@ -1004,9 +1043,10 @@ static void ldapqry_destroyer( void * ptr ) {
 	LdapQuery *qry;
 
 	qry = ( LdapQuery * ) ptr;
-	/*
-	printf( "ldapqry_destroyer::%d::%s\n", (int) pthread_self(), ADDRQUERY_NAME(qry) );
-	*/
+	g_return_if_fail( qry != NULL );
+
+	debug_print("ldapqry_destroyer::%d::%s\n", (int) pthread_self(),
+			ADDRQUERY_NAME(qry)?ADDRQUERY_NAME(qry):"null");
 
 	/* Perform any destruction here */
 	if( qry->control != NULL ) {
@@ -1024,12 +1064,11 @@ static void ldapqry_destroyer( void * ptr ) {
 void ldapqry_cancel( LdapQuery *qry ) {
 	g_return_if_fail( qry != NULL );
 
-	/*
-	printf( "cancelling::%d::%s\n", (int) pthread_self(), ADDRQUERY_NAME(qry) );
-	*/
+	debug_print("cancelling::%d::%s\n", (int) pthread_self(),
+			ADDRQUERY_NAME(qry)?ADDRQUERY_NAME(qry):"null");
 	if( ldapqry_get_busy_flag( qry ) ) {
 		if( qry->thread ) {
-			/* printf( "calling pthread_cancel\n" ); */
+			debug_print("calling pthread_cancel\n");
 			pthread_cancel( * qry->thread );
 		}
 	}
@@ -1040,15 +1079,13 @@ void ldapqry_cancel( LdapQuery *qry ) {
  * any LDAP queries to initialize thread specific data.
  */
 void ldapqry_initialize( void ) {
-	/* printf( "ldapqry_initialize...\n" ); */
+	debug_print("ldapqry_initialize...\n");
 	if( ! _queryThreadInit_ ) {
-		/*
-		printf( "ldapqry_initialize::creating thread specific area\n" );
-		*/
+		debug_print("ldapqry_initialize::creating thread specific area\n");
 		pthread_key_create( &_queryThreadKey_, ldapqry_destroyer );
 		_queryThreadInit_ = TRUE;
 	}
-	/* printf( "ldapqry_initialize... done!\n" ); */
+	debug_print("ldapqry_initialize... done!\n");
 }
 
 /**
@@ -1231,15 +1268,11 @@ static gint ldapqry_locate_retrieve( LdapQuery *qry ) {
 	}
 	ADDRQUERY_RETVAL(qry) = LDAPRC_SEARCH;
 	if( rc != LDAP_SUCCESS ) {
-		/*
-		printf( "LDAP Error: ldap_search_st: %s\n", ldap_err2string( rc ) );
-		*/
+		debug_print("LDAP Error: ldap_search_st: %s\n", ldap_err2string(rc));
 		return ADDRQUERY_RETVAL(qry);
 	}
 
-	/*
-	printf( "Total results are: %d\n", ldap_count_entries( ld, result ) );
-	*/
+	debug_print("Total results are: %d\n", ldap_count_entries(ld, result));
 
 	/* Process results */
 	ADDRQUERY_RETVAL(qry) = LDAPRC_STOP_FLAG;
@@ -1343,6 +1376,22 @@ gboolean ldapquery_remove_results( LdapQuery *qry ) {
 	return retVal;
 }
 
+void ldapqry_print(LdapQuery *qry, FILE *stream) {
+	g_return_if_fail( qry != NULL );
+
+	ldapsvr_print_data(qry->server, stream);
+	ldapctl_print(qry->control, stream);
+	fprintf(stream, "entriesRead: %d\n", qry->entriesRead);
+	fprintf(stream, "elapsedTime: %d\n", qry->elapsedTime);
+	fprintf(stream, "stopFlag: %d\n", qry->stopFlag);
+	fprintf(stream, "busyFlag: %d\n", qry->busyFlag);
+	fprintf(stream, "agedFlag: %d\n", qry->agedFlag);
+	fprintf(stream, "completed: %d\n", qry->completed);
+	fprintf(stream, "startTime: %d\n", (int) qry->startTime);
+	fprintf(stream, "touchTime: %d\n", (int) qry->touchTime);
+	fprintf(stream, "data: %s\n", qry->data?(gchar *)qry->data:"null");
+}
+
 #endif	/* USE_LDAP */
 
 /*
