@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -47,7 +47,6 @@ LdifFile *ldif_create() {
 	ldifFile = g_new0( LdifFile, 1 );
 	ldifFile->path = NULL;
 	ldifFile->file = NULL;
-	ldifFile->bufptr = ldifFile->buffer;
 	ldifFile->hashFields = g_hash_table_new( g_str_hash, g_str_equal );
 	ldifFile->tempList = NULL;
 	ldifFile->dirtyFlag = TRUE;
@@ -210,24 +209,22 @@ void ldif_free( LdifFile *ldifFile ) {
  * \return <i>TRUE</i> if file opened successfully.
  */
 static gint ldif_open_file( LdifFile* ldifFile ) {
-	/* printf( "Opening file\n" ); */
+	/* g_print( "Opening file\n" ); */
 	if( ldifFile->path ) {
 		ldifFile->file = g_fopen( ldifFile->path, "rb" );
 		if( ! ldifFile->file ) {
-			/* printf( "can't open %s\n", ldifFile->path ); */
+			/* g_print( "can't open %s\n", ldifFile->path ); */
 			ldifFile->retVal = MGU_OPEN_FILE;
 			return ldifFile->retVal;
 		}
 	}
 	else {
-		/* printf( "file not specified\n" ); */
+		/* g_print( "file not specified\n" ); */
 		ldifFile->retVal = MGU_NO_FILE;
 		return ldifFile->retVal;
 	}
 
 	/* Setup a buffer area */
-	ldifFile->buffer[0] = '\0';
-	ldifFile->bufptr = ldifFile->buffer;
 	ldifFile->retVal = MGU_SUCCESS;
 	return ldifFile->retVal;
 }
@@ -248,14 +245,15 @@ static void ldif_close_file( LdifFile *ldifFile ) {
  * \return ptr to buffer where line starts.
  */
 static gchar *ldif_get_line( LdifFile *ldifFile ) {
-	gchar buf[ LDIFBUFSIZE ];
+	gchar *buf = g_malloc(LDIFBUFSIZE);
 	gint ch;
 	int i = 0;
+	int cur_alloc = LDIFBUFSIZE;
 
 	if( feof( ldifFile->file ) ) 
 		return NULL;
 
-	while( i < LDIFBUFSIZE-1 ) {
+	while( i < cur_alloc-1 ) {
 		ch = fgetc( ldifFile->file );
 		if (ferror( ldifFile->file ))
 			ldifFile->retVal = MGU_ERROR_READ;
@@ -272,6 +270,10 @@ static gchar *ldif_get_line( LdifFile *ldifFile ) {
 			break;
 		buf[i] = ch;
 		i++;
+		if (i == cur_alloc-1 && cur_alloc < LDIFBUFSIZE * 32) {
+			cur_alloc += LDIFBUFSIZE;
+			buf = g_realloc(buf, cur_alloc);
+		}
 	}
 	buf[i] = '\0';
 
@@ -394,7 +396,7 @@ static void ldif_build_items(
 	ItemEMail *email;
 
 	nodeAddress = rec->listAddress;
-	if( nodeAddress == NULL ) return;
+//	if( nodeAddress == NULL ) return;
 
 	/* Find longest first name in list */
 	nodeFirst = rec->listFName;
