@@ -35,7 +35,7 @@ typedef struct _MainWindow  MainWindow;
 #define ACCOUNT_LIST_CHANGED_HOOKLIST "account_list_changed"
 #define MAIN_WINDOW_CLOSE "mainwindow_close"
 #define MAIN_WINDOW_GOT_ICONIFIED "mainwindow_iconified"
-
+#define THEME_CHANGED_HOOKLIST "theme_changed"
 typedef enum
 {
 	M_UNLOCKED            = 1 << 0,
@@ -59,7 +59,9 @@ typedef enum
 	M_HAVE_QUEUED_MAILS   = 1 << 18,
 	M_WANT_SYNC	      = 1 << 19,
 	M_TAGS_EXIST	      = 1 << 20,
-	M_HAVE_PROCESSING     = 1 << 21
+	M_HAVE_PROCESSING     = 1 << 21,
+	M_SUMMARY_ISLIST      = 1 << 22,
+	M_IN_MSGLIST	      = 1 << 23
 } SensitiveCond;
 
 typedef enum
@@ -170,8 +172,6 @@ void main_window_get_position		(MainWindow	*mainwin);
 
 void main_window_progress_on		(MainWindow	*mainwin);
 void main_window_progress_off		(MainWindow	*mainwin);
-void main_window_empty_trash		(MainWindow	*mainwin,
-					 gboolean	 confirm);
 
 void main_window_set_menu_sensitive	(MainWindow	*mainwin);
 
