@@ -71,6 +71,7 @@
 #include "printing.h"
 #include "quoted-printable.h"
 #include "version.h"
+#include "statusbar.h"
 
 static GList *messageview_list = NULL;
 
@@ -465,6 +466,8 @@ MessageView *messageview_create(MainWindow *mainwin)
 	messageview->statusbar     = NULL;
 	messageview->statusbar_cid = 0;
 
+	messageview->show_full_text= FALSE;
+
 	messageview->msginfo_update_callback_id =
 		hooks_register_hook(MSGINFO_UPDATE_HOOKLIST, messageview_update_msg, (gpointer) messageview);
 
@@ -492,7 +495,7 @@ static void messageview_add_toolbar(MessageView *msgview, GtkWidget *window)
  	GtkWidget *handlebox;
 	GtkWidget *vbox;
 	GtkWidget *menubar;
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	GtkWidget *statusbar = NULL;
 #endif
 	guint n_menu_entries;
@@ -522,12 +525,17 @@ static void messageview_add_toolbar(MessageView *msgview, GtkWidget *window)
 #else
 	msgview->toolbar = toolbar_create(TOOLBAR_MSGVIEW, handlebox,
 					  (gpointer)msgview);
+#ifndef GENERIC_UMPC
 	statusbar = gtk_statusbar_new();
 	gtk_widget_show(statusbar);
 	gtk_box_pack_end(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);
 	msgview->statusbar = statusbar;
 	msgview->statusbar_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR(statusbar), "Message View");
+#else
+	msgview->statusbar = NULL;
+	msgview->statusbar_cid = 0;
+#endif
 #endif
 
 
@@ -767,7 +775,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		    addrp) < 0) {
 		g_free(addrp);
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -782,7 +790,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		if (fprintf(fp, "SCF:%s\n", path) < 0) {
 			g_free(path);
 			fclose(fp);
-			g_unlink(tmp);
+			claws_unlink(tmp);
 			return -1;
 		}
 		g_free(path);
@@ -790,7 +798,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 
 	if (fprintf(fp, "X-Claws-End-Special-Headers: 1\n") < 0) {
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -798,7 +806,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 	get_rfc822_date(buf, sizeof(buf));
 	if (fprintf(fp, "Date: %s\n", buf) < 0) {
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -809,20 +817,20 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 			 strlen("From: "));
 		if (fprintf(fp, "From: %s <%s>\n", buf, account->address) < 0) {
 			fclose(fp);
-			g_unlink(tmp);
+			claws_unlink(tmp);
 			return -1;
 		}
 	} else
 		if (fprintf(fp, "From: %s\n", account->address) < 0) {
 			fclose(fp);
-			g_unlink(tmp);
+			claws_unlink(tmp);
 			return -1;
 		}
 
 
 	if (fprintf(fp, "To: %s\n", to) < 0) {
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -831,7 +839,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 				    strlen("Subject: "));
 	if (fprintf(fp, "Subject: Disposition notification: %s\n", buf) < 0) {
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -852,7 +860,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 
 		if (fprintf(fp, "Message-ID: <%s>\n", buf) < 0) {
 			fclose(fp);
-			g_unlink(tmp);
+			claws_unlink(tmp);
 			return -1;
 		}
 	}
@@ -864,7 +872,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		orig_to = g_strdup(msginfo->to);
 		extract_address(orig_to);
 	}
-	if (msginfo->subject) {
+	if (msginfo->subject && *(msginfo->subject)) {
 		enc_sub = g_malloc0(strlen(msginfo->subject)*8);
 		qp_encode_line(enc_sub, (const guchar *)msginfo->subject);
 		g_strstrip(enc_sub);
@@ -920,7 +928,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 			msginfo->msgid?msginfo->msgid:"NO MESSAGE ID",
 			boundary) < 0) {
 		fclose(fp);
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		g_free(boundary);
 		return -1;
 	}
@@ -932,7 +940,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 
 	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(tmp, "fclose");
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 
@@ -941,13 +949,13 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 	if (!queue) queue = folder_get_default_queue();
 	if (!queue) {
 		g_warning("can't find queue folder\n");
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 	folder_item_scan(queue);
 	if ((num = folder_item_add_msg(queue, tmp, NULL, TRUE)) < 0) {
 		g_warning("can't queue the message\n");
-		g_unlink(tmp);
+		claws_unlink(tmp);
 		return -1;
 	}
 		
@@ -999,6 +1007,9 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	gchar *subject = NULL;
 	g_return_val_if_fail(msginfo != NULL, -1);
 
+	if (msginfo != messageview->msginfo)
+		messageview->show_full_text = FALSE;
+
 	if (messageview->mimeview->textview &&
 	    messageview->mimeview->textview->loading) {
 		messageview->mimeview->textview->stop_loading = TRUE;
@@ -1015,17 +1026,33 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 			 MSG_IS_SPAM(msginfo->flags)?LEARN_HAM:LEARN_SPAM);
 
 	if (messageview->toolbar) {
-		if (messageview->toolbar->learn_spam_btn)
+		if (messageview->toolbar->learn_spam_btn) {
+			gboolean can_learn = FALSE;
+			if (procmsg_spam_can_learn() &&
+			    (msginfo->folder &&
+			     msginfo->folder->folder->klass->type != F_UNKNOWN &&
+			     msginfo->folder->folder->klass->type != F_NEWS))
+				can_learn = TRUE;
+
 			gtk_widget_set_sensitive(
 				messageview->toolbar->learn_spam_btn, 
-				procmsg_spam_can_learn());
+				can_learn);
+		}
 	}
 	
 	noticeview_hide(messageview->noticeview);
 	mimeview_clear(messageview->mimeview);
 	messageview->updating = TRUE;
 
+	if (msginfo->size > 1024*1024)
+		statuswindow_print_all(_("Fetching message (%s)..."),
+			to_human_readable(msginfo->size));
+	
 	file = procmsg_get_message_file_path(msginfo);
+
+	if (msginfo->size > 1024*1024)
+		statuswindow_pop_all();
+
 	if (!file) {
 		g_warning("can't get message file path.\n");
 		textview_show_error(messageview->mimeview->textview);
@@ -1090,7 +1117,7 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	}
 	mimeview_show_message(messageview->mimeview, mimeinfo, file);
 	
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	messageview_set_position(messageview, 0);
 #endif
 
@@ -1115,14 +1142,16 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	main_create_mailing_list_menu(messageview->mainwin, messageview->msginfo);
 
 	if (messageview->msginfo && messageview->msginfo->extradata
-	    && messageview->msginfo->extradata->partial_recv)
+	    && messageview->msginfo->extradata->partial_recv
+	    && !noticeview_is_visible(messageview->noticeview))
 		partial_recv_show(messageview->noticeview, 
 				  messageview->msginfo);
 	else if (messageview->msginfo && messageview->msginfo->extradata &&
 	    (messageview->msginfo->extradata->dispositionnotificationto || 
 	     messageview->msginfo->extradata->returnreceiptto) &&
 	    !MSG_IS_RETRCPT_SENT(messageview->msginfo->flags) &&
-	    !prefs_common.never_send_retrcpt)
+	    !prefs_common.never_send_retrcpt &&
+	    !noticeview_is_visible(messageview->noticeview))
 		return_receipt_show(messageview->noticeview, 
 				    messageview->msginfo);
 
@@ -1133,6 +1162,54 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 			&& (mimeinfo->type != MIMETYPE_MULTIPART || 
 	    strcasecmp(mimeinfo->subtype, "signed"))) {
 	    	if (strcasecmp(mimeinfo->subtype, "html")) {
+		    	MimeInfo *saved_mimeinfo = mimeinfo;
+			MimeInfo *alt_parent = mimeinfo;
+
+			/* if multipart/{related,mixed} part, look inside for a multipart/alternative child */
+			if (prefs_common.promote_html_part &&
+			    mimeinfo->type == MIMETYPE_MULTIPART &&
+			    (!strcasecmp(mimeinfo->subtype, "related") ||
+			     !strcasecmp(mimeinfo->subtype, "mixed"))) {
+				for (; mimeinfo; mimeinfo = procmime_mimeinfo_next(mimeinfo)) {
+					if (mimeinfo->node->parent != saved_mimeinfo->node) {
+						/* only consider children of the 
+						 * multipart/{related,mixed} part */
+						continue;
+					}
+					if (mimeinfo->type == MIMETYPE_MULTIPART && 
+					    !strcasecmp(mimeinfo->subtype, "alternative")) {
+					    	/* we got an alternative part */
+					    	alt_parent = mimeinfo;
+						break;
+					}
+				}
+			}
+
+			/* if we now have a multipart/alternative part (possibly inside a
+			 * multipart/{related,mixed} part, look for an HTML part inside */
+			if (prefs_common.promote_html_part && mimeinfo && 
+			    mimeinfo->type == MIMETYPE_MULTIPART &&
+			    !strcasecmp(mimeinfo->subtype, "alternative")) {
+				for (; mimeinfo; mimeinfo = procmime_mimeinfo_next(mimeinfo)) {
+					if (mimeinfo->node->parent != alt_parent->node) {
+						/* only consider children of the 
+						 * multipart/alternative part, so as
+						 * not to show html attachments */
+						continue;
+					}
+					if (mimeinfo->type == MIMETYPE_TEXT && 
+					    !strcasecmp(mimeinfo->subtype, "html")) {
+					    	/* we got it */
+						mimeview_select_mimepart_icon(messageview->mimeview, mimeinfo);
+						goto done;
+					}
+				}
+			}
+			
+			/* if we didn't find anything, go back to start */
+			if (!mimeinfo) 
+				mimeinfo = saved_mimeinfo;
+
 			mimeview_show_part(messageview->mimeview,mimeinfo);
 			goto done;
 		} else if (prefs_common.invoke_plugin_on_html) {
@@ -1278,8 +1355,8 @@ void messageview_delete(MessageView *msgview)
 			/* NOTE: does not update to next message in summaryview */
 		}
 	}
-#ifdef MAEMO
-	if (msgview->window) {
+#ifdef GENERIC_UMPC
+	if (msgview->window && !prefs_common.always_show_msg) {
 		messageview_destroy(msgview);
 	}
 #endif
@@ -1480,6 +1557,30 @@ static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 }
 #endif
+
+static void messageview_show_partial_display_cb(NoticeView *noticeview, MessageView *messageview)
+{
+	messageview->show_full_text = TRUE;
+	main_window_cursor_wait(mainwindow_get_mainwindow());
+	GTK_EVENTS_FLUSH();
+	messageview_show(messageview, messageview->msginfo, messageview->all_headers);
+	main_window_cursor_normal(mainwindow_get_mainwindow());
+}
+
+void messageview_show_partial_display(MessageView *messageview, MsgInfo *msginfo,
+					     size_t length)
+{
+	gchar *msg = g_strdup_printf(_("Show all %s."), to_human_readable((goffset)length));
+	noticeview_set_icon(messageview->noticeview, STOCK_PIXMAP_NOTICE_WARN);
+	noticeview_set_text(messageview->noticeview, _("Only the first megabyte of text is shown."));
+	noticeview_set_button_text(messageview->noticeview, msg);
+	g_free(msg);
+	noticeview_set_button_press_callback(messageview->noticeview,
+					     G_CALLBACK(messageview_show_partial_display_cb),
+					     (gpointer) messageview);
+	noticeview_show(messageview->noticeview);
+}
+
 static void return_receipt_show(NoticeView *noticeview, MsgInfo *msginfo)
 {
 	gchar *addr = NULL;
@@ -1563,7 +1664,7 @@ static void partial_recv_show(NoticeView *noticeview, MsgInfo *msginfo)
 			text = g_strdup_printf(_("This message has been "
 					"partially retrieved;\nit is %s."),
 					to_human_readable(
-						(off_t)(msginfo->total_size)));
+						(goffset)(msginfo->total_size)));
 			button1 = _("Mark for download");
 			button2 = _("Mark for deletion");
 			button1_cb = partial_recv_dload_clicked;
@@ -1574,7 +1675,7 @@ static void partial_recv_show(NoticeView *noticeview, MsgInfo *msginfo)
 					"partially retrieved;\nit is %s and "
 					"will be downloaded."),
 					to_human_readable(
-						(off_t)(msginfo->total_size)));
+						(goffset)(msginfo->total_size)));
 			button1 = _("Unmark");
 			button1_cb = partial_recv_unmark_clicked;
 			button2 = _("Mark for deletion");
@@ -1585,7 +1686,7 @@ static void partial_recv_show(NoticeView *noticeview, MsgInfo *msginfo)
 					"partially retrieved;\nit is %s and "
 					"will be deleted."),
 					to_human_readable(
-						(off_t)(msginfo->total_size)));
+						(goffset)(msginfo->total_size)));
 			button1 = _("Mark for download");
 			button1_cb = partial_recv_dload_clicked;
 			button2 = _("Unmark");
@@ -1876,7 +1977,7 @@ static void prev_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -1901,7 +2002,7 @@ static void next_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -1926,7 +2027,7 @@ static void prev_unread_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -1951,7 +2052,7 @@ static void next_unread_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -1976,7 +2077,7 @@ static void prev_new_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2001,7 +2102,7 @@ static void next_new_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2026,7 +2127,7 @@ static void prev_marked_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2051,7 +2152,7 @@ static void next_marked_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2076,7 +2177,7 @@ static void prev_labeled_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2101,7 +2202,7 @@ static void next_labeled_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2126,7 +2227,7 @@ static void last_read_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2151,7 +2252,7 @@ static void parent_cb(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2176,7 +2277,7 @@ static void goto_unread_folder_cb(gpointer data, guint action, GtkWidget *widget
 		return;
 	}
 	if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 		if (msginfo)
@@ -2195,7 +2296,7 @@ static void goto_folder_cb(gpointer data, guint action, GtkWidget *widget)
 	FolderItem *to_folder;
 	messageview->updating = FALSE;
 
-	to_folder = foldersel_folder_sel(NULL, FOLDER_SEL_ALL, NULL);
+	to_folder = foldersel_folder_sel(NULL, FOLDER_SEL_ALL, NULL, FALSE);
 
 	if (to_folder) {
 		folderview_select(messageview->mainwin->folderview, to_folder);
@@ -2206,7 +2307,7 @@ static void goto_folder_cb(gpointer data, guint action, GtkWidget *widget)
 			return;
 		}
 		if (messageview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			MsgInfo * msginfo = summary_get_selected_msg(messageview->mainwin->summaryview);
 		       
 			if (msginfo)
