@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef __NEWS_H__
@@ -47,4 +47,8 @@ gint news_cancel_article		(Folder 	*folder,
 					 MsgInfo 	*msginfo);
 int news_folder_locked			(Folder 	*folder);
 
+guint nntp_folder_get_refcnt(Folder *folder);
+void nntp_folder_ref(Folder *folder);
+void nntp_folder_unref(Folder *folder);
+
 #endif /* __NEWS_H__ */
