@@ -317,7 +317,7 @@ static int partial_uidl_mark_mail(MsgInfo *msginfo, int download)
 	}
 
 	fclose(fp);
-	g_unlink(filename);
+	claws_unlink(filename);
 	rename(pathnew, filename);
 	g_free(pathnew);
 	msginfo->planned_download = download;
