@@ -121,8 +121,10 @@ SignatureStatus sgpgme_sigstat_gpgme_to_privacy(gpgme_ctx_t ctx, gpgme_verify_re
 			return SIGNATURE_CHECK_FAILED;
 		}
 	case GPG_ERR_SIG_EXPIRED:
-	case GPG_ERR_KEY_EXPIRED:
+	case GPG_ERR_CERT_REVOKED:
 		return SIGNATURE_WARN;
+	case GPG_ERR_KEY_EXPIRED:
+		return SIGNATURE_KEY_EXPIRED;
 	case GPG_ERR_BAD_SIGNATURE:
 		return SIGNATURE_INVALID;
 	case GPG_ERR_NO_PUBKEY:
@@ -256,11 +258,18 @@ gchar *sgpgme_sigstat_info_full(gpgme_ctx_t ctx, gpgme_verify_result_t status)
 	while (sig) {
 		gpgme_user_id_t user = NULL;
 		gpgme_key_t key;
-
+		gpgme_error_t err;
 		const gchar *keytype, *keyid, *uid;
 		
-		gpgme_get_key(ctx, sig->fpr, &key, 0);
+		err = gpgme_get_key(ctx, sig->fpr, &key, 0);
 
+		if (err != GPG_ERR_NO_ERROR) {
+			key = NULL;
+			g_string_append_printf(siginfo, 
+				_("Error checking signature: %s\n"),
+				gpgme_strerror(err));
+			goto bail;
+		}
 		if (key) {
 			user = key->uids;
 			keytype = gpgme_pubkey_algo_name(
@@ -330,7 +339,7 @@ gchar *sgpgme_sigstat_info_full(gpgme_ctx_t ctx, gpgme_verify_result_t status)
 		i++;
 		sig = sig->next;
 	}
-
+bail:
 	ret = siginfo->str;
 	g_string_free(siginfo, FALSE);
 	return ret;
@@ -353,7 +362,7 @@ gpgme_data_t sgpgme_data_from_mimeinfo(MimeInfo *mimeinfo)
 	debug_print("tmp file %s\n", tmp_file);
 	
 	err = gpgme_data_new_from_file(&data, tmp_file, 1);
-	g_unlink(tmp_file);
+	claws_unlink(tmp_file);
 	g_free(tmp_file);
 
 	debug_print("data %p (%d %d)\n", (void *)&data, mimeinfo->offset, mimeinfo->length);
