@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,6 +39,8 @@
 #include "spamassassin.h"
 #include "statusbar.h"
 #include "menu.h"
+#include "addressbook.h"
+#include "combobox.h"
 
 struct SpamAssassinPage
 {
@@ -59,6 +61,8 @@ struct SpamAssassinPage
 	GtkWidget *max_size;
 	GtkWidget *timeout;
 	GtkWidget *mark_as_read;
+	GtkWidget *whitelist_ab;
+	GtkWidget *whitelist_ab_folder_combo;
 
 	SpamAssassinTransport	trans;
 };
@@ -88,6 +92,20 @@ struct Transport transports[] = {
 	{ N_("Unix Socket"),	SPAMASSASSIN_TRANSPORT_UNIX,		PAGE_UNIX,    0 },
 };
 
+static void spamassassin_whitelist_ab_select_cb(GtkWidget *widget, gpointer data)
+{
+	struct SpamAssassinPage *page = (struct SpamAssassinPage *) data;
+	const gchar *folderpath = NULL;
+	gchar *new_path = NULL;
+
+	folderpath = gtk_entry_get_text(GTK_ENTRY(GTK_BIN(page->whitelist_ab_folder_combo)->child));
+	new_path = addressbook_folder_selection(folderpath);
+	if (new_path) {
+		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(page->whitelist_ab_folder_combo)->child), new_path);
+		g_free(new_path);
+	} 
+}
+
 static void foldersel_cb(GtkWidget *widget, gpointer data)
 {
 	struct SpamAssassinPage *page = (struct SpamAssassinPage *) data;
@@ -95,7 +113,7 @@ static void foldersel_cb(GtkWidget *widget, gpointer data)
 	gchar *item_id;
 	gint newpos = 0;
 	
-	item = foldersel_folder_sel(NULL, FOLDER_SEL_MOVE, NULL);
+	item = foldersel_folder_sel(NULL, FOLDER_SEL_MOVE, NULL, FALSE);
 	if (item && (item_id = folder_item_get_identifier(item)) != NULL) {
 		gtk_editable_delete_text(GTK_EDITABLE(page->save_folder), 0, -1);
 		gtk_editable_insert_text(GTK_EDITABLE(page->save_folder), item_id, strlen(item_id), &newpos);
@@ -171,8 +189,15 @@ static void transport_sel_cb(GtkMenuItem *menuitem, gpointer data)
 {
 	struct SpamAssassinPage *page = (struct SpamAssassinPage *) data;
 	struct Transport *transport;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	g_return_if_fail(gtk_combo_box_get_active_iter(
+			GTK_COMBO_BOX(page->transport_optmenu), &iter));
+	
+	model = gtk_combo_box_get_model(GTK_COMBO_BOX(page->transport_optmenu));			
+	gtk_tree_model_get(model, &iter, 1, &transport, -1);
 
-	transport = (struct Transport *) g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID);
 	show_transport(page, transport);
 }
 
@@ -188,13 +213,15 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	GtkWidget *frame_transport, *table_transport, *vbox_transport;
 	GtkWidget *hbox_spamd, *hbox_max_size, *hbox_timeout;
 	GtkWidget *hbox_process_emails, *hbox_save_spam;
-	GtkWidget *hbox_mark_as_read;
+	GtkWidget *hbox_mark_as_read, *hbox_whitelist;
+	GtkWidget *whitelist_ab_checkbtn;
+	GtkWidget *whitelist_ab_folder_combo;
+	GtkWidget *whitelist_ab_select_btn;
 
 	GtkWidget *enable_sa_checkbtn;
 
 	GtkWidget *transport_label;
 	GtkWidget *transport_optmenu;
-	GtkWidget *transport_menu;
 
 	GtkWidget *user_label;
 	GtkWidget *user_entry;
@@ -225,6 +252,10 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	GtkWidget *mark_as_read_checkbtn;
 
 	GtkTooltips *tooltips;
+	
+	GtkListStore *store;
+	GtkCellRenderer *renderer;
+	GtkTreeIter iter;
 
 	tooltips = gtk_tooltips_new();
 
@@ -256,13 +287,18 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	gtk_label_set_justify(GTK_LABEL(transport_label), GTK_JUSTIFY_RIGHT);
 	gtk_misc_set_alignment(GTK_MISC(transport_label), 1, 0.5);
 
-	transport_optmenu = gtk_option_menu_new();
+	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
+	transport_optmenu = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+	renderer = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(transport_optmenu),
+				   renderer, TRUE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(transport_optmenu),
+				       renderer, "text", 0, NULL);
 	gtk_widget_show(transport_optmenu);
 
 	gtk_table_attach (GTK_TABLE (table_transport), transport_optmenu, 1, 2, 0, 1,
 			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			(GtkAttachOptions) (0), 0, 0);
-	transport_menu = gtk_menu_new();
 
 	user_label = gtk_label_new(_("User"));
 	gtk_widget_show(user_label);
@@ -334,7 +370,7 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 			_("Messages larger than this will not be checked"), NULL);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(max_size_spinbtn), TRUE);
 
-	max_size_kb_label = gtk_label_new(_("kB"));
+	max_size_kb_label = gtk_label_new(_("KB"));
 	gtk_widget_show(max_size_kb_label);
 	gtk_box_pack_start(GTK_BOX(hbox_max_size), max_size_kb_label, FALSE, FALSE, 0);
 
@@ -400,6 +436,27 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	gtk_widget_show(mark_as_read_checkbtn);
 	gtk_box_pack_start(GTK_BOX(hbox_mark_as_read), mark_as_read_checkbtn, TRUE, TRUE, 0);
 
+	hbox_whitelist = gtk_hbox_new(FALSE, 8);
+	gtk_widget_show(hbox_whitelist);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox_whitelist, TRUE, TRUE, 0);
+
+	whitelist_ab_checkbtn = gtk_check_button_new_with_label(_("Whitelist senders found in address book/folder"));
+	gtk_widget_show(whitelist_ab_checkbtn);
+	gtk_box_pack_start(GTK_BOX(hbox_whitelist), whitelist_ab_checkbtn, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip(tooltips, whitelist_ab_checkbtn,
+			_("Messages coming from your address book contacts will be received in the normal folder even if detected as spam"), NULL);
+
+	whitelist_ab_folder_combo = combobox_text_new(TRUE, _("Any"), NULL);
+	gtk_widget_set_size_request(whitelist_ab_folder_combo, 100, -1);
+	gtk_box_pack_start (GTK_BOX (hbox_whitelist), whitelist_ab_folder_combo, TRUE, TRUE, 0);
+
+	whitelist_ab_select_btn = gtk_button_new_with_label(_("Select ..."));
+	gtk_widget_show (whitelist_ab_select_btn);
+	gtk_box_pack_start (GTK_BOX (hbox_whitelist), whitelist_ab_select_btn, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip(tooltips, whitelist_ab_select_btn,
+			_("Click this button to select a book or folder in the address book"),
+			NULL);
+
 	SET_TOGGLE_SENSITIVITY(enable_sa_checkbtn, frame_transport);
 	SET_TOGGLE_SENSITIVITY(enable_sa_checkbtn, hbox_max_size);
 	SET_TOGGLE_SENSITIVITY(enable_sa_checkbtn, hbox_timeout);
@@ -408,11 +465,15 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	SET_TOGGLE_SENSITIVITY(save_spam_checkbtn, save_spam_folder_select);
 	SET_TOGGLE_SENSITIVITY(enable_sa_checkbtn, hbox_process_emails);
 	SET_TOGGLE_SENSITIVITY(save_spam_checkbtn, mark_as_read_checkbtn);
+	SET_TOGGLE_SENSITIVITY(whitelist_ab_checkbtn, whitelist_ab_folder_combo);
+	SET_TOGGLE_SENSITIVITY(whitelist_ab_checkbtn, whitelist_ab_select_btn);
 
 	config = spamassassin_get_config();
 
 	g_signal_connect(G_OBJECT(save_spam_folder_select), "clicked",
 			G_CALLBACK(foldersel_cb), page);
+	g_signal_connect(G_OBJECT (whitelist_ab_select_btn), "clicked",
+			 G_CALLBACK(spamassassin_whitelist_ab_select_cb), page);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_sa_checkbtn), config->enable);
 	if (config->username != NULL)
@@ -421,11 +482,26 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 		gtk_entry_set_text(GTK_ENTRY(spamd_hostname_entry), config->hostname);
 	if (config->socket != NULL)
 		gtk_entry_set_text(GTK_ENTRY(spamd_socket_entry), config->socket);
+	if (config->whitelist_ab_folder != NULL) {
+		/* translate "Any" (stored UNtranslated) */
+		if (strcasecmp(config->whitelist_ab_folder, "Any") == 0)
+			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(whitelist_ab_folder_combo)->child),
+					config->whitelist_ab_folder);
+		else
+		/* backward compatibility (when translated "Any" was stored) */
+		if (g_utf8_collate(config->whitelist_ab_folder, _("Any")) == 0)
+			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(whitelist_ab_folder_combo)->child),
+					config->whitelist_ab_folder);
+		else
+			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(whitelist_ab_folder_combo)->child),
+					config->whitelist_ab_folder);
+	}
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spamd_port_spinbtn), (float) config->port);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(max_size_spinbtn), (float) config->max_size);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(timeout_spinbtn), (float) config->timeout);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(process_emails_checkbtn), config->process_emails);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(save_spam_checkbtn), config->receive_spam);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(whitelist_ab_checkbtn), config->whitelist_ab);
 	if (config->save_folder != NULL)
 		gtk_entry_set_text(GTK_ENTRY(save_spam_folder_entry), config->save_folder);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mark_as_read_checkbtn), config->mark_as_read);
@@ -445,17 +521,16 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 	page->save_folder = save_spam_folder_entry;
 	page->save_folder_select = save_spam_folder_select;
 	page->mark_as_read = mark_as_read_checkbtn;
+	page->whitelist_ab = whitelist_ab_checkbtn;
+	page->whitelist_ab_folder_combo = whitelist_ab_folder_combo;
 
 	active = 0;
 	for (i = 0; i < (sizeof(transports) / sizeof(struct Transport)); i++) {
-		GtkWidget *menuitem;
-
-		menuitem = gtk_menu_item_new_with_label(gettext(transports[i].name));
-		g_object_set_data(G_OBJECT(menuitem), MENU_VAL_ID, &transports[i]);
-		g_signal_connect(G_OBJECT(menuitem), "activate",
-				G_CALLBACK(transport_sel_cb), page);
-		gtk_widget_show(menuitem);
-		gtk_menu_append(GTK_MENU(transport_menu), menuitem);
+		
+		gtk_list_store_append(store, &iter);
+		gtk_list_store_set(store, &iter,
+				   0, gettext(transports[i].name),
+				   1, &transports[i], -1);
 
 		if (config->transport == transports[i].transport) {
 			show_transport(page, &transports[i]);
@@ -471,8 +546,9 @@ static void spamassassin_create_widget_func(PrefsPage * _page,
 				config->enable);
 		}
 	}
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(transport_optmenu), transport_menu);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(transport_optmenu), active);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(transport_optmenu), active);
+	g_signal_connect(G_OBJECT(transport_optmenu), "changed",
+			 G_CALLBACK(transport_sel_cb), page);
 
 	page->page.widget = vbox1;
 }
@@ -530,6 +606,17 @@ static void spamassassin_save_func(PrefsPage *_page)
 	/* mark_as_read */
 	config->mark_as_read = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->mark_as_read));
 
+	/* whitelist_ab */
+	config->whitelist_ab = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->whitelist_ab));
+	g_free(config->whitelist_ab_folder);
+	config->whitelist_ab_folder = gtk_editable_get_chars(
+				GTK_EDITABLE(GTK_BIN(page->whitelist_ab_folder_combo)->child), 0, -1);
+	/* store UNtranslated "Any" */
+	if (g_utf8_collate(config->whitelist_ab_folder, _("Any")) == 0) {
+		g_free(config->whitelist_ab_folder);
+		config->whitelist_ab_folder = g_strdup("Any");
+	}
+
 	if (config->process_emails) {
 		spamassassin_register_hook();
 	} else {
@@ -551,7 +638,7 @@ static void spamassassin_save_func(PrefsPage *_page)
 
 static void gtk_message_callback(gchar *message)
 {
-	statusbar_print_all(message);
+	statusbar_print_all("%s", message);
 }
 
 static struct SpamAssassinPage spamassassin_page;
