@@ -699,7 +699,7 @@ gint pop3_write_uidl_list(Pop3Session *session)
 	}
 	fp = NULL;
 #ifdef G_OS_WIN32
-	g_unlink(path);
+	claws_unlink(path);
 #endif
 	if (g_rename(tmp_path, path) < 0) {
 		FILE_OP_ERROR(path, "rename");
@@ -738,7 +738,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 		if (fprintf(fp, "%s\n", prefix) < 0) {
 			FILE_OP_ERROR(file, "fprintf");
 			fclose(fp);
-			g_unlink(file);
+			claws_unlink(file);
 			return -1;
 		}
 	}
@@ -754,7 +754,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 			FILE_OP_ERROR(file, "fwrite");
 			g_warning("can't write to file: %s\n", file);
 			fclose(fp);
-			g_unlink(file);
+			claws_unlink(file);
 			return -1;
 		}
 
@@ -780,7 +780,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 		FILE_OP_ERROR(file, "fwrite");
 		g_warning("can't write to file: %s\n", file);
 		fclose(fp);
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 	if (data[len - 1] != '\r' && data[len - 1] != '\n') {
@@ -788,14 +788,14 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 			FILE_OP_ERROR(file, "fputc");
 			g_warning("can't write to file: %s\n", file);
 			fclose(fp);
-			g_unlink(file);
+			claws_unlink(file);
 			return -1;
 		}
 	}
 
 	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(file, "fclose");
-		g_unlink(file);
+		claws_unlink(file);
 		return -1;
 	}
 
