@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,6 +47,7 @@
 #include "alertpanel.h"
 #include "base64.h"
 #include "filesel.h"
+#include "combobox.h"
 
 enum {
 	CUSTHDR_STRING,		/*!< display string managed by list store */
@@ -209,15 +210,12 @@ static void prefs_custom_header_create(void)
 			  0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (hdr_label), 0, 0.5);
 	
-	hdr_combo = gtk_combo_new ();
-	gtk_widget_show (hdr_combo);
+	hdr_combo = combobox_text_new(TRUE, "User-Agent", "Face", "X-Face",
+				      "X-Operating-System", NULL);
 	gtk_table_attach (GTK_TABLE (table1), hdr_combo, 0, 1, 1, 2,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  0, 0, 0);
 	gtk_widget_set_size_request (hdr_combo, 150, -1);
-	gtkut_combo_set_items (GTK_COMBO (hdr_combo),
-			       "User-Agent", "Face", "X-Face", "X-Operating-System",
-			       NULL);
 
 	val_label = gtk_label_new (_("Value"));
 	gtk_widget_show (val_label);
@@ -311,7 +309,7 @@ static void prefs_custom_header_create(void)
 	customhdr.cancel_btn = cancel_btn;
 
 	customhdr.hdr_combo  = hdr_combo;
-	customhdr.hdr_entry  = GTK_COMBO (hdr_combo)->entry;
+	customhdr.hdr_entry  = GTK_BIN (hdr_combo)->child;
 	customhdr.val_entry  = val_entry;
 
 	customhdr.list_view   = list_view;
@@ -357,7 +355,7 @@ void prefs_custom_header_read_config(PrefsAccount *ac)
 	fclose(fp);
 }
 
-void prefs_custom_header_write_config(PrefsAccount *ac)
+static void prefs_custom_header_write_config(PrefsAccount *ac)
 {
 	gchar *rcpath;
 	PrefFile *pfile;
@@ -559,9 +557,9 @@ static void prefs_custom_header_val_from_file_cb(void)
 	const gchar *hdr = gtk_entry_get_text(GTK_ENTRY(customhdr.hdr_entry));
 	
 	if (!strcmp(hdr, "Face"))
-		filename = filesel_select_file_open(_("Choose a png file"), NULL);
+		filename = filesel_select_file_open(_("Choose a PNG file"), NULL);
 	else if (!strcmp(hdr, "X-Face"))
-		filename = filesel_select_file_open(_("Choose an xbm file"), NULL);
+		filename = filesel_select_file_open(_("Choose an XBM file"), NULL);
 	else
 		filename = filesel_select_file_open(_("Choose a text file"), NULL);
 
@@ -593,7 +591,7 @@ static void prefs_custom_header_val_from_file_cb(void)
 				}
 				if (g_ascii_strcasecmp("png", gdk_pixbuf_format_get_name(format))) {
 					alertpanel_error(_("The image isn't in the correct format (PNG)."));
-					printf("%s\n", gdk_pixbuf_format_get_name(format));
+					g_print("%s\n", gdk_pixbuf_format_get_name(format));
 					g_free(filename);
 					return;
 				}
@@ -602,19 +600,25 @@ static void prefs_custom_header_val_from_file_cb(void)
 				int i = 0;
 				if (g_ascii_strcasecmp("xbm", gdk_pixbuf_format_get_name(format))) {
 					alertpanel_error(_("The image isn't in the correct format (XBM)."));
-					printf("%s\n", gdk_pixbuf_format_get_name(format));
+					g_print("%s\n", gdk_pixbuf_format_get_name(format));
 					g_free(filename);
 					return;
 				}
 				cmd = g_strdup_printf("compface %s", filename);
 				tmp = get_command_output(cmd);
 				g_free(cmd);
-				if (tmp == NULL || strlen(tmp) == 0){
+				if (tmp == NULL || *tmp == '\0') {
 					alertpanel_error(_("Couldn't call `compface`. Make sure it's in your $PATH."));
 					g_free(filename);
 					g_free(tmp);
 					return;
 				}
+				if (strstr(tmp, "compface:")) {
+					alertpanel_error(_("Compface error: %s"), tmp);
+					g_free(filename);
+					g_free(tmp);
+					return;
+				}
 				while (tmp[i]) {
 					gchar *tmp2 = NULL;
 					if (tmp[i] == ' ') {
