@@ -150,13 +150,16 @@ struct _FolderItemTemplatesPage
 	GtkWidget *window;
 	GtkWidget *table;
 	GtkWidget *checkbtn_compose_with_format;
+	GtkWidget *compose_override_from_format;
 	GtkWidget *compose_subject_format;
 	GtkWidget *compose_body_format;
 	GtkWidget *checkbtn_reply_with_format;
 	GtkWidget *reply_quotemark;
+	GtkWidget *reply_override_from_format;
 	GtkWidget *reply_body_format;
 	GtkWidget *checkbtn_forward_with_format;
 	GtkWidget *forward_quotemark;
+	GtkWidget *forward_override_from_format;
 	GtkWidget *forward_body_format;
 
 	/* apply to sub folders */
@@ -1000,8 +1003,6 @@ static void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 	page->default_alt_dictionary_rec_checkbtn = default_alt_dictionary_rec_checkbtn;
 #endif
 
-	address_completion_start(page->window);
-
 	page->page.widget = table;
 }
 
@@ -1013,7 +1014,6 @@ static void prefs_folder_item_compose_destroy_widget_func(PrefsPage *page_)
 		address_completion_unregister_entry(GTK_ENTRY(page->entry_default_to));
 	if (page->entry_default_reply_to)
 		address_completion_unregister_entry(GTK_ENTRY(page->entry_default_reply_to));
-	address_completion_end(page->window);
 }
 
 /** \brief  Save the prefs in page to folder.
@@ -1153,6 +1153,7 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 	GtkWidget *notebook;
 	GtkWidget *vbox;
 	GtkWidget *page_vbox;
+	GtkWidget *no_save_warning;
 	GtkWidget *new_msg_format_rec_checkbtn;
 	GtkWidget *reply_format_rec_checkbtn;
 	GtkWidget *forward_format_rec_checkbtn;
@@ -1162,6 +1163,17 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 	page_vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (page_vbox);
 	
+	if (!can_save) {
+		no_save_warning = gtk_label_new(
+			_("<i>These preferences will not be saved as this folder "
+			"is a top-level one. However you can use them to set them "
+			"to the whole mailbox tree using \"Apply to subfolders\".</i>"));
+		gtk_label_set_use_markup(GTK_LABEL(no_save_warning), TRUE);
+		gtk_label_set_line_wrap(GTK_LABEL(no_save_warning), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(no_save_warning), 0.0, 0.5);
+		gtk_box_pack_start(GTK_BOX(page_vbox), no_save_warning, FALSE, FALSE, 4);
+	}
+
 	/* Notebook */
 	notebook = gtk_notebook_new();
 	gtk_widget_show(notebook);
@@ -1177,9 +1189,12 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 				window,
 				vbox,
 				&page->checkbtn_compose_with_format,
+				&page->compose_override_from_format,
 				&page->compose_subject_format,
 				&page->compose_body_format,
-				FALSE);
+				FALSE, FALSE);
+	address_completion_register_entry(GTK_ENTRY(page->compose_override_from_format),
+			TRUE);
 
 	new_msg_format_rec_checkbtn = gtk_check_button_new_with_label(
 			_("Apply to subfolders"));
@@ -1195,9 +1210,12 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 				window,
 				vbox,
 				&page->checkbtn_reply_with_format,
+				&page->reply_override_from_format,
 				&page->reply_quotemark,
 				&page->reply_body_format,
-				FALSE);
+				FALSE, FALSE);
+	address_completion_register_entry(GTK_ENTRY(page->reply_override_from_format),
+			TRUE);
 
 	reply_format_rec_checkbtn = gtk_check_button_new_with_label(
 			_("Apply to subfolders"));
@@ -1213,9 +1231,12 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 				window,
 				vbox,
 				&page->checkbtn_forward_with_format,
+				&page->forward_override_from_format,
 				&page->forward_quotemark,
 				&page->forward_body_format,
-				FALSE);
+				FALSE, FALSE);
+	address_completion_register_entry(GTK_ENTRY(page->forward_override_from_format),
+			TRUE);
 
 	forward_format_rec_checkbtn = gtk_check_button_new_with_label(
 			_("Apply to subfolders"));
@@ -1227,6 +1248,8 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->checkbtn_compose_with_format),
 			item->prefs->compose_with_format);
+	pref_set_entry_from_pref(GTK_ENTRY(page->compose_override_from_format),
+			item->prefs->compose_override_from_format);
 	pref_set_entry_from_pref(GTK_ENTRY(page->compose_subject_format),
 			item->prefs->compose_subject_format);
 	pref_set_textview_from_pref(GTK_TEXT_VIEW(page->compose_body_format),
@@ -1236,6 +1259,8 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 			item->prefs->reply_with_format);
 	pref_set_entry_from_pref(GTK_ENTRY(page->reply_quotemark),
 			item->prefs->reply_quotemark);
+	pref_set_entry_from_pref(GTK_ENTRY(page->reply_override_from_format),
+			item->prefs->reply_override_from_format);
 	pref_set_textview_from_pref(GTK_TEXT_VIEW(page->reply_body_format),
 			item->prefs->reply_body_format);
 
@@ -1243,6 +1268,8 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 			item->prefs->forward_with_format);
 	pref_set_entry_from_pref(GTK_ENTRY(page->forward_quotemark),
 			item->prefs->forward_quotemark);
+	pref_set_entry_from_pref(GTK_ENTRY(page->forward_override_from_format),
+			item->prefs->forward_override_from_format);
 	pref_set_textview_from_pref(GTK_TEXT_VIEW(page->forward_body_format),
 			item->prefs->forward_body_format);
 
@@ -1259,7 +1286,14 @@ static void prefs_folder_item_templates_create_widget_func(PrefsPage * page_,
 
 static void prefs_folder_item_templates_destroy_widget_func(PrefsPage *page_) 
 {
-	/* FolderItemTemplatesPage *page = (FolderItemTemplatesPage *) page_; */
+	FolderItemTemplatesPage *page = (FolderItemTemplatesPage *) page_;
+
+	if (page->compose_override_from_format)
+		address_completion_unregister_entry(GTK_ENTRY(page->compose_override_from_format));		
+	if (page->reply_override_from_format)
+		address_completion_unregister_entry(GTK_ENTRY(page->reply_override_from_format));		
+	if (page->forward_override_from_format)
+		address_completion_unregister_entry(GTK_ENTRY(page->forward_override_from_format));		
 }
 
 /** \brief  Save the prefs in page to folder.
@@ -1283,15 +1317,19 @@ static void templates_save_folder_prefs(FolderItem *folder, FolderItemTemplatesP
 	/* save and check formats */
 
 	if (all || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->new_msg_format_rec_checkbtn))) {
+		
 		prefs->compose_with_format =
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_compose_with_format));
+		prefs->compose_override_from_format = pref_get_pref_from_entry(
+				GTK_ENTRY(page->compose_override_from_format));
 		prefs->compose_subject_format = pref_get_pref_from_entry(
 				GTK_ENTRY(page->compose_subject_format));
 		prefs->compose_body_format = pref_get_pref_from_textview(
 				GTK_TEXT_VIEW(page->compose_body_format));
 		quotefmt_check_new_msg_formats(prefs->compose_with_format,
-										prefs->compose_subject_format,
-										prefs->compose_body_format);
+						prefs->compose_override_from_format,
+						prefs->compose_subject_format,
+						prefs->compose_body_format);
 	}
 
 	if (all || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->reply_format_rec_checkbtn))) {
@@ -1299,9 +1337,12 @@ static void templates_save_folder_prefs(FolderItem *folder, FolderItemTemplatesP
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_reply_with_format));
 		prefs->reply_quotemark = gtk_editable_get_chars(
 				GTK_EDITABLE(page->reply_quotemark), 0, -1);
+		prefs->reply_override_from_format = pref_get_pref_from_entry(
+				GTK_ENTRY(page->reply_override_from_format));
 		prefs->reply_body_format = pref_get_pref_from_textview(
 				GTK_TEXT_VIEW(page->reply_body_format));
 		quotefmt_check_reply_formats(prefs->reply_with_format,
+										prefs->reply_override_from_format,
 										prefs->reply_quotemark,
 										prefs->reply_body_format);
 	}
@@ -1311,9 +1352,12 @@ static void templates_save_folder_prefs(FolderItem *folder, FolderItemTemplatesP
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_forward_with_format));
 		prefs->forward_quotemark = gtk_editable_get_chars(
 				GTK_EDITABLE(page->forward_quotemark), 0, -1);
+		prefs->forward_override_from_format = pref_get_pref_from_entry(
+				GTK_ENTRY(page->forward_override_from_format));
 		prefs->forward_body_format = pref_get_pref_from_textview(
 				GTK_TEXT_VIEW(page->forward_body_format));
 		quotefmt_check_forward_formats(prefs->forward_with_format,
+										prefs->forward_override_from_format,
 										prefs->forward_quotemark,
 										prefs->forward_body_format);
 	}
@@ -1557,6 +1601,16 @@ static void register_templates_page(void)
 
 static GSList *prefs_pages = NULL;
 
+static void prefs_folder_item_address_completion_start(GtkWindow * window)
+{
+	address_completion_start(GTK_WIDGET(window));
+}
+
+static void prefs_folder_item_address_completion_end(GtkWindow * window)
+{
+	address_completion_end(GTK_WIDGET(window));
+}
+
 void prefs_folder_item_open(FolderItem *item)
 {
 	gchar *id, *title;
@@ -1576,8 +1630,10 @@ void prefs_folder_item_open(FolderItem *item)
 	title = g_strdup_printf (_("Properties for folder %s"), id);
 	g_free (id);
 	prefswindow_open(title, prefs_pages, item,
-			&prefs_common.folderitemwin_width, &prefs_common.folderitemwin_height);
-	
+			&prefs_common.folderitemwin_width, &prefs_common.folderitemwin_height,
+			prefs_folder_item_address_completion_start,
+			prefs_folder_item_address_completion_end);
+
 	g_free (title);
 }
 
