@@ -936,7 +936,8 @@ static GSList *prefs_pages = NULL;
 void prefs_gtk_open(void)
 {
 	prefswindow_open(_("Preferences"), prefs_pages, NULL,
-			&prefs_common.prefswin_width, &prefs_common.prefswin_height);
+			&prefs_common.prefswin_width, &prefs_common.prefswin_height,
+			NULL, NULL);
 }
 
 void prefs_gtk_register_page(PrefsPage *page)
