@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,8 @@
 #include "gtkutils.h"
 #include "utils.h"
 
+static void prefs_summary_column_set_config(SummaryColumnState *state);
+
 enum {
 	SUMCOL_NAME,
 	SUMCOL_TYPE,
@@ -87,6 +89,7 @@ static const gchar *const col_name[N_SUMMARY_COLS] = {
 	N_("Number"),		/* S_COL_NUMBER  */
         N_("Score"),		/* S_COL_SCORE   */
 	N_("Locked"),		/* S_COL_LOCKED  */
+	N_("Tags"),		/* S_COL_TAGS  */
 };
 
 static SummaryColumnState default_state[N_SUMMARY_COLS] = {
@@ -101,6 +104,7 @@ static SummaryColumnState default_state[N_SUMMARY_COLS] = {
 	{ S_COL_NUMBER , FALSE },
         { S_COL_SCORE  , FALSE },
 	{ S_COL_LOCKED , FALSE },
+	{ S_COL_TAGS   , FALSE },
 };
 
 static void prefs_summary_column_create	(void);
@@ -157,6 +161,12 @@ static void drag_data_received	(GtkTreeView *tree_view,
 				 guint time, 
 				 GtkTreeModel *model);
 
+static void prefs_summary_column_shown_set_btn_sensitivity(void);
+static void prefs_summary_column_shown_set_active(const gboolean active);
+static void prefs_summary_column_stock_set_active(const gboolean active);
+static void prefs_summary_column_shown_sel_changed(void);
+static void prefs_summary_column_stock_sel_changed(void);
+
 void prefs_summary_column_open(void)
 {
 	inc_lock();
@@ -263,6 +273,9 @@ static void prefs_summary_column_create(void)
 				       
 	stock_list_view = prefs_summary_column_list_view_create
 				(_("Hidden columns"));
+	g_signal_connect(G_OBJECT(stock_list_view), "cursor-changed",
+			   G_CALLBACK(prefs_summary_column_stock_sel_changed),
+			   NULL);
 	gtk_widget_show(stock_list_view);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), stock_list_view);
 
@@ -292,6 +305,9 @@ static void prefs_summary_column_create(void)
 
 	shown_list_view = prefs_summary_column_list_view_create
 				(_("Displayed columns"));
+	g_signal_connect(G_OBJECT(shown_list_view), "cursor-changed",
+			   G_CALLBACK(prefs_summary_column_shown_sel_changed),
+			   NULL);
 	gtk_widget_show(shown_list_view);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), shown_list_view);
 
@@ -357,6 +373,9 @@ static void prefs_summary_column_create(void)
 	summary_col.cancel_btn  = cancel_btn;
 	summary_col.stock_list_view = stock_list_view;
 	summary_col.shown_list_view = shown_list_view;
+	
+	prefs_summary_column_shown_set_active(FALSE);
+	prefs_summary_column_stock_set_active(FALSE);
 }
 
 SummaryColumnState *prefs_summary_column_get_config(void)
@@ -384,7 +403,7 @@ SummaryColumnState *prefs_summary_column_get_config(void)
 	return state;
 }
 
-void prefs_summary_column_set_config(SummaryColumnState *state)
+static void prefs_summary_column_set_config(SummaryColumnState *state)
 {
 	SummaryColumnType type;
 	gint pos;
@@ -510,6 +529,8 @@ static void prefs_summary_column_add(void)
 	gtk_tree_selection_select_iter(gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(summary_col.shown_list_view)),
 		 &shown_add);
+	prefs_summary_column_shown_set_active(TRUE);
+	prefs_summary_column_stock_set_active(FALSE);
 }
 
 static void prefs_summary_column_remove(void)
@@ -557,6 +578,8 @@ static void prefs_summary_column_remove(void)
 	gtk_tree_selection_select_iter(gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(summary_col.stock_list_view)),
 		&stock_add);
+	prefs_summary_column_shown_set_active(FALSE);
+	prefs_summary_column_stock_set_active(TRUE);
 }
 
 static void prefs_summary_column_up(void)
@@ -594,6 +617,7 @@ static void prefs_summary_column_up(void)
 	gtk_tree_path_free(prev);
 
 	gtk_list_store_swap(shown_store, &iprev, &isel);
+	prefs_summary_column_shown_set_btn_sensitivity();
 }
 
 static void prefs_summary_column_down(void)
@@ -616,11 +640,14 @@ static void prefs_summary_column_down(void)
 		return;
 
 	gtk_list_store_swap(shown_store, &next, &sel);
+	prefs_summary_column_shown_set_btn_sensitivity();
 }
 
 static void prefs_summary_column_set_to_default(void)
 {
 	prefs_summary_column_set_dialog(default_state);
+	prefs_summary_column_shown_set_active(FALSE);
+	prefs_summary_column_stock_set_active(FALSE);
 }
 
 static void prefs_summary_column_ok(void)
@@ -886,6 +913,83 @@ static void drag_data_received(GtkTreeView *tree_view, GdkDragContext *context,
 		gtk_drag_finish(context, TRUE, FALSE, time);
 	}
 
+	prefs_summary_column_shown_set_active(FALSE);
+	prefs_summary_column_stock_set_active(FALSE);
+	
 	/* XXXX: should we call gtk_drag_finish() for other code paths? */
 }
 
+static void prefs_summary_column_shown_set_btn_sensitivity(void)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(gtk_tree_view_get_model(
+		GTK_TREE_VIEW(summary_col.shown_list_view)));
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(summary_col.shown_list_view));
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	
+	if(!gtk_tree_selection_get_selected(selection, NULL, &iter)) {
+		gtk_widget_set_sensitive(summary_col.remove_btn, FALSE);
+		gtk_widget_set_sensitive(summary_col.up_btn, FALSE);
+		gtk_widget_set_sensitive(summary_col.down_btn, FALSE);
+		return;
+	}
+	
+	path = gtk_tree_model_get_path(model, &iter);
+
+	gtk_widget_set_sensitive(summary_col.up_btn, gtk_tree_path_prev(path));
+	gtk_widget_set_sensitive(summary_col.down_btn,
+				 gtk_tree_model_iter_next(model, &iter));
+	gtk_tree_path_free(path);
+}
+
+static void prefs_summary_column_shown_set_active(const gboolean active)
+{
+	GtkTreeSelection *selection = NULL;
+	
+	gtk_widget_set_sensitive(summary_col.remove_btn, active);
+	
+	if(active == FALSE) {
+		selection = gtk_tree_view_get_selection(
+			GTK_TREE_VIEW(summary_col.shown_list_view));
+		gtk_tree_selection_unselect_all(selection);
+		
+		gtk_widget_set_sensitive(summary_col.up_btn, FALSE);
+		gtk_widget_set_sensitive(summary_col.down_btn, FALSE);
+	} else {
+		prefs_summary_column_shown_set_btn_sensitivity();
+	}
+}
+
+static void prefs_summary_column_stock_set_active(const gboolean active)
+{
+	GtkTreeSelection *selection = NULL;
+	
+	gtk_widget_set_sensitive(summary_col.add_btn, active);
+	
+	if(active == FALSE) {
+		selection = gtk_tree_view_get_selection(
+			GTK_TREE_VIEW(summary_col.stock_list_view));
+		gtk_tree_selection_unselect_all(selection);
+	}
+}
+
+static void prefs_summary_column_stock_sel_changed(void)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(summary_col.stock_list_view));
+	prefs_summary_column_stock_set_active(
+		(selection != NULL) ? TRUE : FALSE);
+	prefs_summary_column_shown_set_active(
+		(selection != NULL) ? FALSE : TRUE);
+}
+
+static void prefs_summary_column_shown_sel_changed(void)
+{
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(
+		GTK_TREE_VIEW(summary_col.shown_list_view));
+	prefs_summary_column_shown_set_active(
+		(selection != NULL) ? TRUE : FALSE);
+	prefs_summary_column_stock_set_active(
+		(selection != NULL) ? FALSE : TRUE);
+}
