@@ -57,6 +57,7 @@ static struct Templates {
 	GtkWidget *list_view;
 	GtkWidget *entry_name;
 	GtkWidget *entry_subject;
+	GtkWidget *entry_from;
 	GtkWidget *entry_to;
 	GtkWidget *entry_cc;	
 	GtkWidget *entry_bcc;
@@ -65,6 +66,26 @@ static struct Templates {
 
 static int modified = FALSE;
 static int modified_list = FALSE;
+
+static struct
+{
+	gchar *label;
+	GtkWidget **entry;
+	gboolean compl;
+	gchar *tooltips;
+} widgets_table[] = {
+	{N_("Name"),	&templates.entry_name,		FALSE,
+		N_("This name is used as the Menu item")},
+	{N_("From"),	&templates.entry_from,		TRUE,
+		N_("Override composing account's From header. This doesn't change the composing account.")},
+	{N_("To"),	&templates.entry_to,		TRUE, 	NULL},
+	{N_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
+	{N_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
+	{N_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
+	{NULL,		NULL,				FALSE,	NULL}
+};
+
+
 /* widget creating functions */
 static void prefs_template_window_create	(void);
 static void prefs_template_window_setup		(void);
@@ -132,7 +153,7 @@ static void prefs_template_window_create(void)
 	GtkWidget *scrolled_window;
 	GtkWidget   *vpaned;
 	GtkWidget     *vbox1;
-	GtkWidget       *table; /* including : entry_[name|to|cc|bcc|subject] */
+	GtkWidget       *table; /* including : entry_[name|from|to|cc|bcc|subject] */
 	GtkWidget       *scroll2;
 	GtkWidget         *text_value;
 	GtkWidget     *vbox2;
@@ -200,43 +221,28 @@ static void prefs_template_window_create(void)
 
 	tooltips = gtk_tooltips_new();
 
-	struct
-	{
-		gchar *label;
-		GtkWidget **entry;
-		gboolean compl;
-		gchar *tooltips;
-	} tab[] = {
-		{_("Name"),	&templates.entry_name,		FALSE,
-			_("This name is used as the Menu item")},
-		{_("To"),	&templates.entry_to,		TRUE, 	NULL},
-		{_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
-		{_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
-		{_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
-		{NULL,		NULL,				FALSE,	NULL}
-	};
-
-	for (i=0; tab[i].label; i++) {
+	for (i=0; widgets_table[i].label; i++) {
 
 		GtkWidget *label;
 
-		label = gtk_label_new(tab[i].label);
+		label = gtk_label_new(widgets_table[i].label);
 		gtk_widget_show(label);
 		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, (i + 1),
 				(GtkAttachOptions) (GTK_FILL),
 				(GtkAttachOptions) 0, 0, 0);
 		gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
-		*(tab[i].entry) = gtk_entry_new();
-		gtk_widget_show(*(tab[i].entry));
-		gtk_table_attach(GTK_TABLE(table), *(tab[i].entry), 1, 2, i, (i + 1),
+		*(widgets_table[i].entry) = gtk_entry_new();
+		gtk_widget_show(*(widgets_table[i].entry));
+		gtk_table_attach(GTK_TABLE(table), *(widgets_table[i].entry), 1, 2, i, (i + 1),
 				(GtkAttachOptions) (GTK_EXPAND|GTK_SHRINK|GTK_FILL),
 				(GtkAttachOptions) 0, 0, 0);
-		gtk_tooltips_set_tip(tooltips, *(tab[i].entry), tab[i].tooltips, NULL);
+		gtk_tooltips_set_tip(tooltips, *(widgets_table[i].entry),
+				widgets_table[i].tooltips, NULL);
 
-		if (tab[i].compl)
+		if (widgets_table[i].compl)
 			address_completion_register_entry(
-				GTK_ENTRY(*(tab[i].entry)), TRUE);
+				GTK_ENTRY(*(widgets_table[i].entry)), TRUE);
 	}
 
 	/* template content */
@@ -261,7 +267,7 @@ static void prefs_template_window_create(void)
 		}
 	}
 	gtk_widget_show(text_value);
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	gtk_widget_set_size_request(text_value, -1, 120);
 #else
 	gtk_widget_set_size_request(text_value, -1, 60);
@@ -441,6 +447,7 @@ static void prefs_template_reset_dialog(void)
 	GtkTextBuffer *buffer;
 
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_from), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc), "");			
@@ -510,6 +517,18 @@ static gboolean prefs_template_key_pressed_cb(GtkWidget *widget,
 	return FALSE;
 }
 
+static void prefs_template_address_completion_end(void)
+{
+	gint i;
+
+	for (i=0; widgets_table[i].label; i++) {
+		if (widgets_table[i].compl)
+			address_completion_unregister_entry(
+				GTK_ENTRY(*(widgets_table[i].entry)));
+	}
+	address_completion_end(templates.window);
+}
+
 static void prefs_template_ok_cb(void)
 {
 	GSList *tmpl_list;
@@ -521,6 +540,9 @@ static void prefs_template_ok_cb(void)
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	} 
+
+	prefs_template_address_completion_end();
+
 	modified = FALSE;
 	modified_list = FALSE;
 	tmpl_list = prefs_template_get_list();
@@ -548,6 +570,9 @@ static void prefs_template_cancel_cb(void)
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
+
+	prefs_template_address_completion_end();
+
 	modified = FALSE;
 	modified_list = FALSE;
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
@@ -588,6 +613,9 @@ static GSList *prefs_template_get_list(void)
 			ntmpl->subject = tmpl->subject && *(tmpl->subject) 
 					 ? g_strdup(tmpl->subject) 
 					 : NULL;
+			ntmpl->from    = tmpl->from && *(tmpl->from)
+					 ? g_strdup(tmpl->from)
+					 : NULL;
 			ntmpl->to      = tmpl->to && *(tmpl->to)
 					 ? g_strdup(tmpl->to)
 					 : NULL;
@@ -608,29 +636,49 @@ static GSList *prefs_template_get_list(void)
 	return tmpl_list;
 }
 
-gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean escaped_string)
+gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean escaped_string, gboolean email)
 {
+	gboolean result = TRUE;
 	if (string && *string != '\0') {
+		gchar *tmp = NULL;
 		gchar *parsed_buf;
 		MsgInfo dummyinfo;
+		PrefsAccount *account = account_get_default();
 
+		if (escaped_string) {
+			tmp = malloc(strlen(string)+1);
+			pref_get_unescaped_pref(tmp, string);
+		} else {
+			tmp = g_strdup(string);
+		}
 		memset(&dummyinfo, 0, sizeof(MsgInfo));
 #ifdef USE_ASPELL
-		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, NULL, escaped_string, NULL);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, account, FALSE, NULL);
 #else
-		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, NULL, escaped_string);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, account, FALSE);
 #endif
-		quote_fmt_scan_string(string);
+		quote_fmt_scan_string(tmp);
 		quote_fmt_parse();
+		g_free(tmp);
 		parsed_buf = quote_fmt_get_buffer();
 		if (!parsed_buf) {
 			if (line)
 				*line = quote_fmt_get_line();
 			return FALSE;
 		}
+		if (email) {
+			const gchar *start = strrchr(parsed_buf, '<');
+			const gchar *end = strrchr(parsed_buf, '>');
+			const gchar *at = strrchr(parsed_buf, '@');
+			const gchar *space = strrchr(parsed_buf, ' ');
+			if (!at)
+				result = FALSE;
+			if (at && space && (!start || !end || end < start || start < space))
+				result = FALSE;
+		}
 		quote_fmt_reset_vartable();
 	}
-	return TRUE;
+	return result;
 }
 
 static gboolean prefs_template_list_view_set_row(gint row)
@@ -639,6 +687,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	Template *tmpl;
 	gchar *name;
 	gchar *subject;
+	gchar *from;
 	gchar *to;
 	gchar *cc;
 	gchar *bcc;	
@@ -659,7 +708,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 			g_free(value);
 		value = NULL;
 		}
-	if (!prefs_template_string_is_valid(value, &line, FALSE)) {
+	if (!prefs_template_string_is_valid(value, &line, TRUE, FALSE)) {
 		alertpanel_error(_("Template body format error at line %d."), line);
 		g_free(value);
 		return FALSE;
@@ -672,6 +721,8 @@ static gboolean prefs_template_list_view_set_row(gint row)
 		g_free(value);
 		return FALSE;
 	}
+	from = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_from),
+				    0, -1);
 	to = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_to),
 				    0, -1);
 	cc = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_cc),
@@ -681,6 +732,10 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	subject = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_subject),
 					 0, -1);
 
+	if (from && *from == '\0') {
+		g_free(from);
+		from = NULL;
+	}
 	if (to && *to == '\0') {
 		g_free(to);
 		to = NULL;
@@ -698,25 +753,31 @@ static gboolean prefs_template_list_view_set_row(gint row)
 		subject = NULL;
 	}
 
-	if (!prefs_template_string_is_valid(to, NULL, FALSE)) {
+	if (!prefs_template_string_is_valid(from, NULL, TRUE, TRUE)) {
+		alertpanel_error(_("Template From format error."));
+		g_free(from);
+		g_free(value);
+		return FALSE;
+	}
+	if (!prefs_template_string_is_valid(to, NULL, TRUE, TRUE)) {
 		alertpanel_error(_("Template To format error."));
 		g_free(to);
 		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(cc, NULL, FALSE)) {
+	if (!prefs_template_string_is_valid(cc, NULL, TRUE, TRUE)) {
 		alertpanel_error(_("Template Cc format error."));	
 		g_free(cc);
 		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(bcc, NULL, FALSE)) {
+	if (!prefs_template_string_is_valid(bcc, NULL, TRUE, TRUE)) {
 		alertpanel_error(_("Template Bcc format error."));	
 		g_free(bcc);
 		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(subject, NULL, FALSE)) {
+	if (!prefs_template_string_is_valid(subject, NULL, TRUE, FALSE)) {
 		alertpanel_error(_("Template subject format error."));	
 		g_free(subject);
 		g_free(value);
@@ -727,6 +788,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	tmpl->load_filename = NULL;
 	tmpl->name = name;
 	tmpl->subject = subject;
+	tmpl->from = from;
 	tmpl->to = to;
 	tmpl->cc = cc;
 	tmpl->bcc = bcc;	
@@ -1132,6 +1194,7 @@ static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path)
 
 	tmpl_def.name = _("Template");
 	tmpl_def.subject = "";
+	tmpl_def.from = "";
 	tmpl_def.to = "";
 	tmpl_def.cc = "";
 	tmpl_def.bcc = "";	
@@ -1142,6 +1205,8 @@ static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path)
 		tmpl = &tmpl_def;
 
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), tmpl->name);
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_from),
+			   tmpl->from ? tmpl->from : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to),
 			   tmpl->to ? tmpl->to : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc),
