@@ -97,6 +97,7 @@ static const gint ToolbarIcons[] =
 	STOCK_PIXMAP_MAIL_REPLY,
 	STOCK_PIXMAP_MAIL_REPLY_TO_ALL,
 	STOCK_PIXMAP_MAIL_REPLY_TO_AUTHOR,
+	STOCK_PIXMAP_MAIL_REPLY_TO_LIST,
 	STOCK_PIXMAP_MAIL_SEND,
 	STOCK_PIXMAP_MAIL_SEND_QUEUE,
 	STOCK_PIXMAP_MAIL_SIGN,
@@ -114,6 +115,7 @@ static const gint ToolbarIcons[] =
 	STOCK_PIXMAP_GO_FOLDERS,
 	STOCK_PIXMAP_MIME_TEXT_PLAIN,
 	STOCK_PIXMAP_MIME_TEXT_HTML,
+	STOCK_PIXMAP_MIME_TEXT_PATCH,
 	STOCK_PIXMAP_MIME_APPLICATION,
 	STOCK_PIXMAP_MIME_IMAGE,
 	STOCK_PIXMAP_MIME_AUDIO,
@@ -137,6 +139,8 @@ static const gint ToolbarIcons[] =
 	STOCK_PIXMAP_MIME_MESSAGE,
 	STOCK_PIXMAP_SPAM_BTN,
 	STOCK_PIXMAP_HAM_BTN,
+	STOCK_PIXMAP_TRASH,
+	STOCK_PIXMAP_DELETE,
 	STOCK_PIXMAP_EMPTY,              /* last entry */
 };
 
@@ -768,7 +772,7 @@ static void prefs_toolbar_create(ToolbarPage *prefs_toolbar)
 	GtkWidget *reg_hbox;
 	GtkWidget *arrow;
 	GtkWidget *btn_hbox;
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	GtkWidget *hbox;
 #endif
 	GtkWidget *reg_btn;
@@ -915,7 +919,7 @@ static void prefs_toolbar_create(ToolbarPage *prefs_toolbar)
 			  prefs_toolbar);
 
 	default_btn = gtk_button_new_with_label(_(" Use default "));
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	gtk_box_pack_end(GTK_BOX(reg_hbox), default_btn, FALSE, TRUE, 0);
 #else
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -1096,7 +1100,7 @@ static GtkWidget *create_set_list_view(ToolbarPage *prefs_toolbar)
 	gtk_tree_view_column_set_title(column, _("Icon"));
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
-	
+	gtk_tree_view_set_reorderable(list_view, TRUE);
 	/* tell pixbuf renderer it is only visible if 
 	 * the icon is not represented by text */
 	gtk_tree_view_column_set_cell_data_func(column, renderer,
