@@ -772,6 +772,9 @@ static gint procheader_scan_date_string(const gchar *str,
 	gchar zone1[3];
 	gchar zone2[3];
 
+	if (str == NULL)
+		return -1;
+
 	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d %5s",
 			weekday, day, month, year, hh, mm, ss, zone);
 	if (result == 8) return 0;
@@ -1003,7 +1006,7 @@ gint procheader_get_header_from_msginfo(MsgInfo *msginfo, gchar *buf, gint len,
 	val = procheader_get_one_field(buf,len, fp, hentry);
 	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(file, "fclose");
-		g_unlink(file);
+		claws_unlink(file);
 		g_free(file);
 		return -1;
 	}
