@@ -117,17 +117,23 @@ void quote_fmt_quote_description(GtkWidget *widget, GtkWidget *pref_window)
 void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_compose_with_format,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_subject_format,
 						GtkWidget **edit_body_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
 	GtkWidget *vbox_format;
 	GtkWidget *hbox_format;
+	GtkWidget *hbox2_format;
+	GtkWidget *label_from = NULL;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *label_subject;
 	GtkWidget *entry_subject;
 	GtkWidget *scrolledwin_format;
 	GtkWidget *text_format;
+	GtkSizeGroup *size_group;
 
 	if (add_info_button)
 		g_return_if_fail(parent_window != NULL);
@@ -138,6 +144,8 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 	g_return_if_fail(edit_subject_format != NULL);
 	g_return_if_fail(edit_body_format != NULL);
 
+	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+
 	if (checkbtn_compose_with_format)
 		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format, 
 				   _("Use template when composing new messages"));
@@ -147,13 +155,36 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 	gtk_container_add(GTK_CONTAINER (parent_box), vbox_format);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_format), 8);
 
+	if (override_from_format) {
+		hbox2_format = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox2_format);
+		gtk_box_pack_start (GTK_BOX (vbox_format), hbox2_format, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_misc_set_alignment(GTK_MISC(label_from), 1, 0.5);
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox2_format), label_from, FALSE, FALSE, 0);
+		gtk_size_group_add_widget(size_group, label_from);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox2_format), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to compose the new message."),
+				NULL);
+	}
+
 	hbox_format = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_format);
 	gtk_box_pack_start (GTK_BOX (vbox_format), hbox_format, FALSE, FALSE, 0);
 
 	label_subject = gtk_label_new (_("Subject"));
+	gtk_misc_set_alignment(GTK_MISC(label_subject), 1, 0.5);
 	gtk_widget_show (label_subject);
 	gtk_box_pack_start (GTK_BOX (hbox_format), label_subject, FALSE, FALSE, 0);
+	gtk_size_group_add_widget(size_group, label_subject);
 
 	entry_subject = gtk_entry_new ();
 	gtk_widget_show (entry_subject);
@@ -188,15 +219,23 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 
 	if (checkbtn_compose_with_format) {
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_subject);
+		if (override_from_format) {
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_from);
+		}
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_subject);
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_format);
 	}
 
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_format);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_format, set_defaults_func);
 
 	if (checkbtn_compose_with_format)
 		*checkbtn_compose_with_format = checkbtn_use_format;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_subject_format = entry_subject;
 	*edit_body_format = text_format;
 }
@@ -204,18 +243,24 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_reply_with_format,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_reply_quotemark,
 						GtkWidget **edit_reply_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
 	GtkWidget *vbox_quote;
 	GtkWidget *hbox1;
 	GtkWidget *hbox2;
+	GtkWidget *hbox3;
 	GtkWidget *label_quotemark;
 	GtkWidget *entry_quotemark;
+	GtkWidget *label_from = NULL;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *scrolledwin_quotefmt;
 	GtkWidget *text_quotefmt;
+	GtkSizeGroup *size_group;
 
 	if (add_info_button)
 		g_return_if_fail(parent_window != NULL);
@@ -226,6 +271,8 @@ void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 	g_return_if_fail(edit_reply_quotemark != NULL);
 	g_return_if_fail(edit_reply_format != NULL);
 
+	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+
 	if (checkbtn_reply_with_format)
 		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format,
 				   _("Use template when replying to messages"));
@@ -235,6 +282,27 @@ void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 	gtk_container_add(GTK_CONTAINER (parent_box), vbox_quote);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
 
+	if (override_from_format) {
+		hbox3 = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox3);
+		gtk_box_pack_start (GTK_BOX (vbox_quote), hbox3, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_misc_set_alignment(GTK_MISC(label_from), 1, 0.5);
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), label_from, FALSE, FALSE, 0);
+		gtk_size_group_add_widget(size_group, label_from);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to reply."),
+				NULL);
+	}
+
 	hbox1 = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
@@ -244,8 +312,10 @@ void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
 
 	label_quotemark = gtk_label_new (_("Quotation mark"));
+	gtk_misc_set_alignment(GTK_MISC(label_quotemark), 1, 0.5);
 	gtk_widget_show (label_quotemark);
 	gtk_box_pack_start (GTK_BOX (hbox2), label_quotemark, FALSE, FALSE, 0);
+	gtk_size_group_add_widget(size_group, label_quotemark);
 
 	entry_quotemark = gtk_entry_new ();
 	gtk_widget_show (entry_quotemark);
@@ -281,33 +351,47 @@ void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 	if (checkbtn_reply_with_format) {
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_quotemark);
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_quotemark);
+		if (override_from_format) {
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_from);
+		}
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_quotefmt);
 	}
 
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_quote);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_quote, set_defaults_func);
 
 	if (checkbtn_reply_with_format)
 		*checkbtn_reply_with_format = checkbtn_use_format;
 	*edit_reply_quotemark = entry_quotemark;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_reply_format = text_quotefmt;
 }
 
 void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_forward_with_format,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_fw_quotemark,
 						GtkWidget **edit_fw_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
 	GtkWidget *vbox_quote;
 	GtkWidget *hbox1;
 	GtkWidget *hbox2;
+	GtkWidget *hbox3;
 	GtkWidget *label_quotemark;
 	GtkWidget *entry_fw_quotemark;
+	GtkWidget *label_from = NULL;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *scrolledwin_quotefmt;
 	GtkWidget *text_fw_quotefmt;
+	GtkSizeGroup *size_group;
 
 	if (add_info_button)
 		g_return_if_fail(parent_window != NULL);
@@ -318,6 +402,8 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	g_return_if_fail(edit_fw_quotemark != NULL);
 	g_return_if_fail(edit_fw_format != NULL);
 
+	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+
 	if (checkbtn_forward_with_format)
 		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format,
 				   _("Use template when forwarding messages"));
@@ -327,6 +413,27 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	gtk_container_add(GTK_CONTAINER (parent_box), vbox_quote);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
 
+	if (override_from_format) {
+		hbox3 = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox3);
+		gtk_box_pack_start (GTK_BOX (vbox_quote), hbox3, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_misc_set_alignment(GTK_MISC(label_from), 1, 0.5);
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), label_from, FALSE, FALSE, 0);
+		gtk_size_group_add_widget(size_group, label_from);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to forward."),
+				NULL);
+	}
+
 	hbox1 = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
@@ -336,8 +443,10 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
 
 	label_quotemark = gtk_label_new (_("Quotation mark"));
+	gtk_misc_set_alignment(GTK_MISC(label_quotemark), 1, 0.5);
 	gtk_widget_show (label_quotemark);
 	gtk_box_pack_start (GTK_BOX (hbox2), label_quotemark, FALSE, FALSE, 0);
+	gtk_size_group_add_widget(size_group, label_quotemark);
 
 	entry_fw_quotemark = gtk_entry_new ();
 	gtk_widget_show (entry_fw_quotemark);
@@ -375,15 +484,23 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	if (checkbtn_forward_with_format) {
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_quotemark);
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_fw_quotemark);
+		if (override_from_format) {
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_from);
+		}
 		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_fw_quotefmt);
 	}
 
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_quote);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_quote, set_defaults_func);
 
 	if (checkbtn_forward_with_format)
 		*checkbtn_forward_with_format = checkbtn_use_format;
 	*edit_fw_quotemark = entry_fw_quotemark;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_fw_format = text_fw_quotefmt;
 }
 
@@ -409,51 +526,84 @@ void quotefmt_add_info_button(GtkWindow *parent_window, GtkWidget *parent_box)
 			 G_CALLBACK(quote_fmt_quote_description), GTK_WIDGET(parent_window));
 }
 
+void quotefmt_add_defaults_button(GtkWindow *parent_window,
+								  GtkWidget *parent_box,
+								  void(*set_defaults_func)(void))
+{
+	GtkWidget *hbox_formatdesc;
+	GtkWidget *btn_formatdesc;
+
+	g_return_if_fail(set_defaults_func != NULL);
+
+	hbox_formatdesc = gtk_hbox_new (FALSE, 32);
+	gtk_widget_show (hbox_formatdesc);
+	gtk_box_pack_start (GTK_BOX (parent_box), hbox_formatdesc, FALSE, FALSE,
+				VBOX_BORDER);
+
+	btn_formatdesc = gtk_button_new_with_mnemonic (_("Defaults"));
+	gtk_button_set_image (GTK_BUTTON(btn_formatdesc),
+		gtk_image_new_from_stock(GTK_STOCK_UNDO, GTK_ICON_SIZE_BUTTON));
+	gtk_widget_show (btn_formatdesc);
+	gtk_box_pack_start (GTK_BOX (hbox_formatdesc), btn_formatdesc, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(btn_formatdesc), "clicked",
+			 G_CALLBACK(set_defaults_func), GTK_WIDGET(parent_window));
+}
 
 void quotefmt_check_new_msg_formats(gboolean use_format,
-									gchar *subject_fmt,
-									gchar *body_fmt)
+				    gchar *override_from_fmt,
+				    gchar *subject_fmt,
+				    gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(subject_fmt, NULL, FALSE))
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, TRUE, TRUE))
+			alertpanel_error(_("New message From format error."));
+
+		if (!prefs_template_string_is_valid(subject_fmt, NULL, TRUE, FALSE))
 			alertpanel_error(_("New message subject format error."));
 
-		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE)) {
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
 			alertpanel_error(_("New message body format error at line %d."), line);
 		}
 	}
 }
 
 void quotefmt_check_reply_formats(gboolean use_format,
-									gchar *quotation_mark,
-									gchar *body_fmt)
+				  gchar *override_from_fmt,
+				  gchar *quotation_mark,
+				  gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(quotation_mark, NULL, FALSE))
+		if (!prefs_template_string_is_valid(quotation_mark, NULL, TRUE, FALSE))
 			alertpanel_error(_("Message reply quotation mark format error."));
 
-		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE)) {
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, TRUE, TRUE))
+			alertpanel_error(_("Message reply From format error."));
+
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
 			alertpanel_error(_("Message reply format error at line %d."), line);
 		}
 	}
 }
 
 void quotefmt_check_forward_formats(gboolean use_format,
-									gchar *quotation_mark,
-									gchar *body_fmt)
+				    gchar *override_from_fmt,
+				    gchar *quotation_mark,
+				    gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(quotation_mark, NULL, FALSE))
+		if (!prefs_template_string_is_valid(quotation_mark, NULL, TRUE, FALSE))
 			alertpanel_error(_("Message forward quotation mark format error."));
 
-fprintf(stderr, "\n%s\n", body_fmt);
-		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE)) {
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, TRUE, TRUE))
+			alertpanel_error(_("Message forward From format error."));
+
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
 			alertpanel_error(_("Message forward format error at line %d."), line);
 		}
 	}
