@@ -70,6 +70,7 @@
 #include "pixmaps/mail_reply.xpm"
 #include "pixmaps/mail_reply_to_all.xpm"
 #include "pixmaps/mail_reply_to_author.xpm"
+#include "pixmaps/mail_reply_to_list.xpm"
 #include "pixmaps/mail_send.xpm"
 #include "pixmaps/mail_send_queue.xpm"
 #include "pixmaps/mail_sign.xpm"
@@ -83,7 +84,7 @@
 #include "pixmaps/queue_open.xpm"
 #include "pixmaps/queue_open_hrm.xpm"
 #include "pixmaps/claws-mail_icon.xpm"
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 #include "pixmaps/claws-mail_logo.xpm"
 #else
 #include "pixmaps/claws-mail_logo-small.xpm"
@@ -104,6 +105,7 @@
 #include "pixmaps/go_folders.xpm"
 #include "pixmaps/mime_text_plain.xpm"
 #include "pixmaps/mime_text_html.xpm"
+#include "pixmaps/mime_text_patch.xpm"
 #include "pixmaps/mime_application.xpm"
 #include "pixmaps/mime_image.xpm"
 #include "pixmaps/mime_audio.xpm"
@@ -142,6 +144,8 @@
 #include "pixmaps/outbox_close.xpm"
 #include "pixmaps/outbox_close_hrm.xpm"
 #include "pixmaps/trash.xpm"
+#include "pixmaps/delete_btn.xpm"
+#include "pixmaps/cancel.xpm"
 #include "pixmaps/trash_btn.xpm"
 #include "pixmaps/trash_hrm.xpm"
 #include "pixmaps/drafts_close.xpm"
@@ -202,10 +206,15 @@ typedef struct _OverlayData OverlayData;
 
 struct _OverlayData
 {
+	gboolean is_pixmap;
 	GdkPixmap *base_pixmap;
 	GdkBitmap *base_mask;
 	GdkPixmap *overlay_pixmap;
 	GdkBitmap *overlay_mask;
+	
+	GdkPixbuf *base_pixbuf;
+	GdkPixbuf *overlay_pixbuf;
+	
 	guint base_height;
 	guint base_width;
 	guint overlay_height;
@@ -279,6 +288,7 @@ static StockPixmapData pixmaps[] =
 	{mail_reply_xpm				, NULL, NULL, "mail_reply", NULL},
 	{mail_reply_to_all_xpm			, NULL, NULL, "mail_reply_to_all", NULL},
 	{mail_reply_to_author_xpm		, NULL, NULL, "mail_reply_to_author", NULL},
+	{mail_reply_to_list_xpm			, NULL, NULL, "mail_reply_to_list", NULL},
 	{mail_send_xpm				, NULL, NULL, "mail_send", NULL},
 	{mail_send_queue_xpm			, NULL, NULL, "mail_send_queue", NULL},
 	{mail_sign_xpm				, NULL, NULL, "mail_sign", NULL},
@@ -330,6 +340,7 @@ static StockPixmapData pixmaps[] =
 	{drafts_open_mark_xpm			, NULL, NULL, "drafts_open_mark", NULL},
 	{mime_text_plain_xpm			, NULL, NULL, "mime_text_plain", NULL},
 	{mime_text_html_xpm			, NULL, NULL, "mime_text_html", NULL},
+	{mime_text_patch_xpm			, NULL, NULL, "mime_text_patch", NULL},
 	{mime_application_xpm			, NULL, NULL, "mime_application", NULL},
 	{mime_image_xpm				, NULL, NULL, "mime_image", NULL},
 	{mime_audio_xpm				, NULL, NULL, "mime_audio", NULL},
@@ -354,8 +365,10 @@ static StockPixmapData pixmaps[] =
 	{mime_message_xpm			, NULL, NULL, "mime_message", NULL},
 	{claws_mail_icon_xpm			, NULL, NULL, "claws_mail_icon", NULL},
 	{read_xpm				, NULL, NULL, "read", NULL},
+	{delete_btn_xpm				, NULL, NULL, "delete_btn", NULL},
+	{cancel_xpm				, NULL, NULL, "cancel_btn", NULL},
 	{trash_btn_xpm				, NULL, NULL, "trash_btn", NULL},
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	{claws_mail_logo_xpm			, NULL, NULL, "claws_mail_logo", NULL},
 #else
 	{claws_mail_logo_small_xpm		, NULL, NULL, "claws_mail_logo_small", NULL},
@@ -384,14 +397,13 @@ static StockPixmapData pixmaps[] =
 /* return newly constructed GtkPixmap from GdkPixmap */
 GtkWidget *stock_pixmap_widget(GtkWidget *window, StockPixmap icon)
 {
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	GdkPixbuf *pixbuf;
 
 	g_return_val_if_fail(window != NULL, NULL);
 	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
 
-	if (stock_pixmap_gdk(window, icon, &pixmap, &mask) != -1)
-		return gtk_image_new_from_pixmap(pixmap, mask);
+	if (stock_pixbuf_gdk(window, icon, &pixbuf) != -1)
+		return gtk_image_new_from_pixbuf(pixbuf);
 	
 	return NULL;
 }
@@ -402,6 +414,9 @@ GtkWidget *stock_pixmap_widget(GtkWidget *window, StockPixmap icon)
 gint stock_pixbuf_gdk(GtkWidget *window, StockPixmap icon, GdkPixbuf **pixbuf)
 {
 	StockPixmapData *pix_d;
+	static const char *extension[]={".png", ".xpm", NULL};
+	int i = 0;
+	gboolean theme_changed = FALSE;
 
 	if (pixbuf)
 		*pixbuf = NULL;
@@ -410,17 +425,23 @@ gint stock_pixbuf_gdk(GtkWidget *window, StockPixmap icon, GdkPixbuf **pixbuf)
 
 	pix_d = &pixmaps[icon];
 
-	if (!pix_d->pixbuf || (strcmp2(pix_d->icon_path, prefs_common.pixmap_theme_path) != 0)) {
+	theme_changed = (strcmp2(pix_d->icon_path, prefs_common.pixmap_theme_path) != 0);
+	if (!pix_d->pixbuf || theme_changed) {
 		GdkPixbuf *pix = NULL;
-	
+		
+		if (theme_changed && pix_d->pixmap) {
+			g_object_unref(pix_d->pixmap);
+			pix_d->pixmap = NULL;
+		}
+
 		if (strcmp(prefs_common.pixmap_theme_path, DEFAULT_PIXMAP_THEME) != 0) {
 			if (is_dir_exist(prefs_common.pixmap_theme_path)) {
 				char *icon_file_name; 
-				
+try_next_extension:				
 				icon_file_name = g_strconcat(prefs_common.pixmap_theme_path,
 							     G_DIR_SEPARATOR_S,
 							     pix_d->file,
-							     ".xpm",
+							     extension[i],
 							     NULL);
 				if (is_file_exist(icon_file_name)) {
 					GError *err = NULL;
@@ -432,6 +453,11 @@ gint stock_pixbuf_gdk(GtkWidget *window, StockPixmap icon, GdkPixbuf **pixbuf)
 					pix_d->icon_path = g_strdup(prefs_common.pixmap_theme_path);
 				}
 				g_free(icon_file_name);
+				if (!pix) {
+					i++;
+					if (extension[i])
+						goto try_next_extension;
+				}
 			} else {
 				/* even the path does not exist (deleted between two sessions), so
 				set the preferences to the internal theme */
@@ -464,6 +490,9 @@ gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 		      GdkPixmap **pixmap, GdkBitmap **mask)
 {
 	StockPixmapData *pix_d;
+	static const char *extension[]={".png", ".xpm", NULL};
+	int i = 0;
+	gboolean theme_changed = FALSE;
 
 	if (pixmap) *pixmap = NULL;
 	if (mask)   *mask   = NULL;
@@ -473,17 +502,23 @@ gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 
 	pix_d = &pixmaps[icon];
 
-	if (!pix_d->pixmap || (strcmp2(pix_d->icon_path, prefs_common.pixmap_theme_path) != 0)) {
+	theme_changed = (strcmp2(pix_d->icon_path, prefs_common.pixmap_theme_path) != 0);
+	if (!pix_d->pixmap || theme_changed) {
 		GdkPixmap *pix = NULL;
-	
+
+		if (theme_changed && pix_d->pixbuf) {
+			g_object_unref(pix_d->pixbuf);
+			pix_d->pixbuf = NULL;
+		}
+
 		if (strcmp(prefs_common.pixmap_theme_path, DEFAULT_PIXMAP_THEME) != 0) {
 			if ( is_dir_exist(prefs_common.pixmap_theme_path) ) {
 				char *icon_file_name; 
-				
+try_next_extension:
 				icon_file_name = g_strconcat(prefs_common.pixmap_theme_path,
 							     G_DIR_SEPARATOR_S,
 							     pix_d->file,
-							     ".xpm",
+							     extension[i],
 							     NULL);
 				if (is_file_exist(icon_file_name))
 					PIXMAP_CREATE_FROM_FILE(window, pix, pix_d->mask, icon_file_name);
@@ -492,6 +527,11 @@ gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 					pix_d->icon_path = g_strdup(prefs_common.pixmap_theme_path);
 				}
 				g_free(icon_file_name);
+				if (!pix) {
+					i++;
+					if (extension[i])
+						goto try_next_extension;
+				}
 			} else {
 				/* even the path does not exist (deleted between two sessions), so
 				set the preferences to the internal theme */
@@ -523,6 +563,7 @@ static void stock_pixmap_find_themes_in_dir(GList **list, const gchar *dirname)
 {
 	struct dirent *d;
 	DIR *dp;
+	static const char *extension[]={".png", ".xpm", NULL};
 	
 	if ((dp = opendir(dirname)) == NULL) {
 		debug_print("dir %s not found, skipping theme scan", dirname?dirname:"(null)");
@@ -538,16 +579,18 @@ static void stock_pixmap_find_themes_in_dir(GList **list, const gchar *dirname)
 		
 		if (strcmp(entry, ".") != 0 && strcmp(entry, "..") != 0 && is_dir_exist(fullentry)) {
 			gchar *filetoexist;
+			gboolean found = FALSE;
 			int i;
-			
-			for (i = 0; i < N_STOCK_PIXMAPS; i++) {
-				filetoexist = g_strconcat(fullentry, G_DIR_SEPARATOR_S, pixmaps[i].file, ".xpm", NULL);
-				if (is_file_exist(filetoexist)) {
-					*list = g_list_append(*list, fullentry);
+			int j;
+			for (i = 0; i < N_STOCK_PIXMAPS && !found; i++) {
+				for (j = 0; extension[j] && !found; j++) {
+					filetoexist = g_strconcat(fullentry, G_DIR_SEPARATOR_S, pixmaps[i].file, extension[j], NULL);
+					if (is_file_exist(filetoexist)) {
+						*list = g_list_append(*list, fullentry);
+						found = TRUE;
+					}
 					g_free(filetoexist);
-					break;
 				}
-				g_free(filetoexist);
 			}
 			if (i == N_STOCK_PIXMAPS) 
 				g_free(fullentry);
@@ -616,9 +659,12 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 	gint left = 0;
 	gint top = 0;
 
-	g_return_val_if_fail(data->base_pixmap != NULL, FALSE);
-	g_return_val_if_fail(data->base_mask != NULL, FALSE);
-
+	if (data->is_pixmap) {
+		g_return_val_if_fail(data->base_pixmap != NULL, FALSE);
+		g_return_val_if_fail(data->base_mask != NULL, FALSE);
+	} else {
+		g_return_val_if_fail(data->base_pixbuf != NULL, FALSE);
+	}
 	gc_pix = gdk_gc_new((GdkWindow *)drawable);
 						 
 	gdk_window_clear_area (drawable, expose->area.x, expose->area.y,
@@ -641,21 +687,22 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 		}
 	}
 
-	gdk_gc_set_tile(gc_pix, data->base_pixmap);
-	gdk_gc_set_ts_origin(gc_pix, data->border_x, data->border_y);
-	gdk_gc_set_clip_mask(gc_pix, data->base_mask);
-	gdk_gc_set_clip_origin(gc_pix, data->border_x, data->border_y);
-	gdk_gc_set_fill(gc_pix, GDK_TILED);
-
-	gdk_draw_rectangle(drawable, gc_pix, TRUE, data->border_x, data->border_y, 
+	if (data->is_pixmap) {
+		gdk_gc_set_tile(gc_pix, data->base_pixmap);
+		gdk_gc_set_ts_origin(gc_pix, data->border_x, data->border_y);
+		gdk_gc_set_clip_mask(gc_pix, data->base_mask);
+		gdk_gc_set_clip_origin(gc_pix, data->border_x, data->border_y);
+		gdk_gc_set_fill(gc_pix, GDK_TILED);
+		gdk_draw_rectangle(drawable, gc_pix, TRUE, data->border_x, data->border_y, 
 			   data->base_width, data->base_height);
+	} else {
+		gdk_draw_pixbuf(drawable, gc_pix, data->base_pixbuf, 
+			0, 0, data->border_x, data->border_y,
+			-1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+	}
+
 
 	if (data->position != OVERLAY_NONE) {
-		g_return_val_if_fail(data->overlay_pixmap != NULL, FALSE);
-		g_return_val_if_fail(data->overlay_mask != NULL, FALSE);
-
-		gdk_gc_set_tile(gc_pix, data->overlay_pixmap);
-		gdk_gc_set_clip_mask(gc_pix, data->overlay_mask);
 
 		switch (data->position) {
 			case OVERLAY_TOP_LEFT:
@@ -701,12 +748,26 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 			default:
 				break;
 		}
+	}
+	if (data->position != OVERLAY_NONE) {
+		if (data->is_pixmap) {
+			g_return_val_if_fail(data->overlay_pixmap != NULL, FALSE);
+			g_return_val_if_fail(data->overlay_mask != NULL, FALSE);
+			gdk_gc_set_tile(gc_pix, data->overlay_pixmap);
+			gdk_gc_set_clip_mask(gc_pix, data->overlay_mask);
 
-		gdk_gc_set_ts_origin(gc_pix, left, top);
-		gdk_gc_set_clip_origin(gc_pix, left, top);
-		gdk_gc_set_fill(gc_pix, GDK_TILED);
-		gdk_draw_rectangle(drawable, gc_pix, TRUE, left, top, 
+			gdk_gc_set_ts_origin(gc_pix, left, top);
+			gdk_gc_set_clip_origin(gc_pix, left, top);
+			gdk_gc_set_fill(gc_pix, GDK_TILED);
+			gdk_draw_rectangle(drawable, gc_pix, TRUE, left, top, 
 				   data->overlay_width, data->overlay_height);
+		} else {
+			g_return_val_if_fail(data->overlay_pixbuf != NULL, FALSE);
+			gdk_draw_pixbuf(drawable, gc_pix, data->overlay_pixbuf, 
+				0, 0, left, top,
+				-1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+
+		}
 	}
 	g_object_unref(gc_pix);
 	
@@ -715,11 +776,18 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 
 static void pixmap_with_overlay_destroy_cb(GtkObject *object, OverlayData *data) 
 {
-	g_object_unref(data->base_pixmap);
-	g_object_unref(data->base_mask);
-	if (data->position != OVERLAY_NONE) {
-		g_object_unref(data->overlay_pixmap);
-		g_object_unref(data->overlay_mask);
+	if (data->is_pixmap) {
+		g_object_unref(data->base_pixmap);
+		g_object_unref(data->base_mask);
+		if (data->position != OVERLAY_NONE) {
+			g_object_unref(data->overlay_pixmap);
+			g_object_unref(data->overlay_mask);
+		}
+	} else {
+		g_object_unref(data->base_pixbuf);
+		if (data->position != OVERLAY_NONE) {
+			g_object_unref(data->overlay_pixbuf);
+		}
 	}
 	g_free(data);
 }
@@ -741,40 +809,69 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 					    StockPixmap overlay, OverlayPosition pos,
 					    gint border_x, gint border_y)
 {
-	GdkPixmap *stock_pixmap;
-	GdkBitmap *stock_mask;
-	GtkWidget *widget;
-	GtkWidget *stock_wid;
-	OverlayData *data;
+	GdkPixmap *stock_pixmap = NULL;
+	GdkBitmap *stock_mask = NULL;
+	GdkPixbuf *stock_pixbuf = NULL;
+	GtkWidget *widget = NULL;
+	GtkWidget *stock_wid = NULL;
+	OverlayData *data = NULL;
 	
 	data = g_new0(OverlayData, 1);
 
 	stock_wid = stock_pixmap_widget(window, icon);
-	gtk_image_get_pixmap(GTK_IMAGE(stock_wid), &stock_pixmap, &stock_mask);
-	g_object_ref(stock_pixmap);
-	g_object_ref(stock_mask);
-	data->base_pixmap = stock_pixmap;
-	data->base_mask   = stock_mask;
-	data->base_height = stock_wid->requisition.height;
-	data->base_width  = stock_wid->requisition.width;
-	gtk_widget_destroy(stock_wid);
 
-	if (pos == OVERLAY_NONE) {
-		data->overlay_pixmap = NULL;
-		data->overlay_mask   = NULL;
-	} else {
-		stock_wid = stock_pixmap_widget(window, overlay);
+	if (gtk_image_get_storage_type(GTK_IMAGE(stock_wid)) == GTK_IMAGE_PIXMAP)
+		data->is_pixmap = TRUE;
+	else
+		data->is_pixmap = FALSE;
+
+	if (data->is_pixmap) {
 		gtk_image_get_pixmap(GTK_IMAGE(stock_wid), &stock_pixmap, &stock_mask);
 		g_object_ref(stock_pixmap);
 		g_object_ref(stock_mask);
-		data->overlay_pixmap = stock_pixmap;
-		data->overlay_mask   = stock_mask;
-		data->overlay_height = stock_wid->requisition.height;
-		data->overlay_width  = stock_wid->requisition.width;
-
+		data->base_pixmap = stock_pixmap;
+		data->base_mask   = stock_mask;
+		data->base_height = stock_wid->requisition.height;
+		data->base_width  = stock_wid->requisition.width;
 		gtk_widget_destroy(stock_wid);
+
+		if (pos == OVERLAY_NONE) {
+			data->overlay_pixmap = NULL;
+			data->overlay_mask   = NULL;
+		} else {
+			stock_wid = stock_pixmap_widget(window, overlay);
+			gtk_image_get_pixmap(GTK_IMAGE(stock_wid), &stock_pixmap, &stock_mask);
+			g_object_ref(stock_pixmap);
+			g_object_ref(stock_mask);
+			data->overlay_pixmap = stock_pixmap;
+			data->overlay_mask   = stock_mask;
+			data->overlay_height = stock_wid->requisition.height;
+			data->overlay_width  = stock_wid->requisition.width;
+
+			gtk_widget_destroy(stock_wid);
+		}
+	} else {
+		data->is_pixmap = FALSE;
+		stock_pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(stock_wid));
+		g_object_ref(stock_pixbuf);
+		data->base_pixbuf = stock_pixbuf;
+		data->base_height = stock_wid->requisition.height;
+		data->base_width  = stock_wid->requisition.width;
+		gtk_widget_destroy(stock_wid);
+		if (pos == OVERLAY_NONE) {
+			data->overlay_pixmap = NULL;
+			data->overlay_mask   = NULL;
+		} else {
+			stock_wid = stock_pixmap_widget(window, overlay);
+			stock_pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(stock_wid));
+			g_object_ref(stock_pixbuf);
+			data->overlay_pixbuf = stock_pixbuf;
+			data->overlay_height = stock_wid->requisition.height;
+			data->overlay_width  = stock_wid->requisition.width;
+
+			gtk_widget_destroy(stock_wid);
+		}
 	}
-	
 	data->position = pos;
 	data->border_x = border_x;
 	data->border_y = border_y;
