@@ -764,7 +764,7 @@ static void recursive_add_parts(TextView *textview, GNode *node)
         mimeinfo = (MimeInfo *) node->data;
         
         textview_add_part(textview, mimeinfo);
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	textview_set_position(textview, 0);
 #endif        
         if ((mimeinfo->type != MIMETYPE_MULTIPART) &&
@@ -879,20 +879,20 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 		TEXTVIEW_INSERT(" (");
 		TEXTVIEW_INSERT(content_type);
 		TEXTVIEW_INSERT(", ");
-		TEXTVIEW_INSERT(to_human_readable(partinfo->length));
+		TEXTVIEW_INSERT(to_human_readable((goffset)partinfo->length));
 		TEXTVIEW_INSERT("):\n\n");
 		
 		g_free(content_type);
 	}
 	TEXTVIEW_INSERT(_("  The following can be performed on this part\n"));
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	TEXTVIEW_INSERT(_("  by right-clicking the icon or list item:"));
 #endif
 	TEXTVIEW_INSERT("\n");
 
 	TEXTVIEW_INSERT(_("     - To save, select "));
 	TEXTVIEW_INSERT_LINK(_("'Save as...'"), "sc://save_as", NULL);
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	TEXTVIEW_INSERT(_(" (Shortcut key: 'y')"));
 #endif
 	TEXTVIEW_INSERT("\n");
@@ -900,7 +900,7 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXTVIEW_INSERT(_("     - To display as text, select "));
 	TEXTVIEW_INSERT_LINK(_("'Display as text'"), "sc://display_as_text", NULL);
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	TEXTVIEW_INSERT(_(" (Shortcut key: 't')"));
 #endif
 	TEXTVIEW_INSERT("\n");
@@ -908,7 +908,7 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXTVIEW_INSERT(_("     - To open with an external program, select "));
 	TEXTVIEW_INSERT_LINK(_("'Open'"), "sc://open", NULL);
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	TEXTVIEW_INSERT(_(" (Shortcut key: 'l')\n"));
 	TEXTVIEW_INSERT(_("       (alternately double-click, or click the middle "));
 	TEXTVIEW_INSERT(_("mouse button)\n"));
@@ -928,7 +928,9 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 	CodeConverter *conv;
 	const gchar *charset, *p, *cmd;
 	GSList *cur;
-	
+	gboolean continue_write = TRUE;
+	size_t wrote = 0, i = 0;
+
 	if (textview->messageview->forced_charset)
 		charset = textview->messageview->forced_charset;
 	else
@@ -953,7 +955,7 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 			tmpfp = g_fopen(filename, "rb");
 			textview_show_html(textview, tmpfp, conv);
 			fclose(tmpfp);
-			g_unlink(filename);
+			claws_unlink(filename);
 		}
 		g_free(filename);
 	} else if (!g_ascii_strcasecmp(mimeinfo->subtype, "enriched")) {
@@ -964,7 +966,7 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 			tmpfp = g_fopen(filename, "rb");
 			textview_show_ertf(textview, tmpfp, conv);
 			fclose(tmpfp);
-			g_unlink(filename);
+			claws_unlink(filename);
 		}
 		g_free(filename);
 #ifndef G_OS_WIN32
@@ -1040,13 +1042,20 @@ textview_default:
 		}
 		fseek(tmpfp, mimeinfo->offset, SEEK_SET);
 		debug_print("Viewing text content of type: %s (length: %d)\n", mimeinfo->subtype, mimeinfo->length);
-		while ((ftell(tmpfp) < mimeinfo->offset + mimeinfo->length) &&
-		       (fgets(buf, sizeof(buf), tmpfp) != NULL)) {
+		while (((i = ftell(tmpfp)) < mimeinfo->offset + mimeinfo->length) &&
+		       (fgets(buf, sizeof(buf), tmpfp) != NULL)
+		       && continue_write) {
 			textview_write_line(textview, buf, conv, TRUE);
 			if (textview->stop_loading) {
 				fclose(tmpfp);
 				return;
 			}
+			wrote += ftell(tmpfp)-i;
+			if (mimeinfo->length > 1024*1024 
+			&&  wrote > 1024*1024
+			&& !textview->messageview->show_full_text) {
+				continue_write = FALSE;
+			}
 		}
 		fclose(tmpfp);
 	}
@@ -1068,6 +1077,12 @@ textview_default:
 		}
 	}
 	
+	if (continue_write == FALSE) {
+		messageview_show_partial_display(
+			textview->messageview, 
+			textview->messageview->msginfo,
+			mimeinfo->length);
+	}
 	GTK_EVENTS_FLUSH();
 }
 
@@ -3009,8 +3024,11 @@ static void mail_to_uri_cb (TextView *textview, guint action, void *data)
 		folder_item = textview->messageview->msginfo->folder;
 		if (folder_item->prefs && folder_item->prefs->enable_default_account)
 			account = account_find_from_id(folder_item->prefs->default_account);
+		
+		compose_new_with_folderitem(account, folder_item, uri->uri+7);
+	} else {
+		compose_new(account, uri->uri + 7, NULL);
 	}
-	compose_new(account, uri->uri + 7, NULL);
 }
 
 static void copy_mail_to_uri_cb	(TextView *textview, guint action, void *data)
