@@ -146,6 +146,8 @@ static void toolbar_draft_cb			(GtkWidget	*widget,
 					 	 gpointer	 data);
 static void toolbar_close_cb			(GtkWidget	*widget,
 					 	 gpointer	 data);
+static void toolbar_preferences_cb		(GtkWidget	*widget,
+					 	 gpointer	 data);
 static void toolbar_open_mail_cb		(GtkWidget	*widget,
 						 gpointer	 data);
 static void toolbar_insert_cb			(GtkWidget	*widget,
@@ -194,6 +196,7 @@ struct {
 	{ "A_PRINT",	     	N_("Print")				   },
 	{ "A_LEARN_SPAM",	N_("Learn Spam or Ham")			   },
 	{ "A_GO_FOLDERS",   	N_("Open folder/Go to folder list")        },
+	{ "A_PREFERENCES",	N_("Preferences")			   },
 
 	{ "A_SEND",          	N_("Send Message")                         },
 	{ "A_SENDL",         	N_("Put into queue folder and send later") },
@@ -227,7 +230,7 @@ struct {
 	{ "toolbar_compose.xml", NULL}, 
   	{ "toolbar_msgview.xml", NULL}
 };
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 static GtkItemFactoryEntry reply_entries[] =
 {
 	{N_("/Reply with _quote"), NULL,    toolbar_reply, COMPOSE_REPLY_WITH_QUOTE, NULL},
@@ -332,7 +335,7 @@ GList *toolbar_get_action_items(ToolbarType source)
 					A_TRASH , A_DELETE_REAL,       A_EXECUTE,       A_GOTO_PREV, 
 					A_GOTO_NEXT,	A_IGNORE_THREAD,  A_WATCH_THREAD,	A_PRINT,
 					A_ADDRBOOK, 	A_LEARN_SPAM, A_GO_FOLDERS, 
-					A_CANCEL_INC };
+					A_CANCEL_INC,   A_PREFERENCES };
 
 		for (i = 0; i < sizeof main_items / sizeof main_items[0]; i++)  {
 			items = g_list_append(items, gettext(toolbar_text[main_items[i]].descr));
@@ -433,6 +436,7 @@ const gchar *toolbar_get_short_text(int action) {
 	case A_WATCH_THREAD: 	return _("Watch thread");
 	case A_PRINT:	 	return _("Print");
 	case A_CLOSE: 		return _("Close");
+	case A_PREFERENCES:	return _("Preferences");
 	case A_SEND: 		return _("Send");
 	case A_SENDL: 		return _("Send later");
 	case A_DRAFT: 		return _("Draft");
@@ -464,10 +468,10 @@ gint toolbar_get_icon(int action) {
 	case A_REPLY_MESSAGE: 	return STOCK_PIXMAP_MAIL_REPLY;
 	case A_REPLY_ALL: 	return STOCK_PIXMAP_MAIL_REPLY_TO_ALL;
 	case A_REPLY_SENDER: 	return STOCK_PIXMAP_MAIL_REPLY_TO_AUTHOR;
-	case A_REPLY_ML: 	return STOCK_PIXMAP_MAIL_REPLY;
+	case A_REPLY_ML: 	return STOCK_PIXMAP_MAIL_REPLY_TO_LIST;
 	case A_FORWARD: 	return STOCK_PIXMAP_MAIL_FORWARD;
 	case A_TRASH: 		return STOCK_PIXMAP_TRASH;
-	case A_DELETE_REAL:	return STOCK_PIXMAP_DELETED;
+	case A_DELETE_REAL:	return STOCK_PIXMAP_DELETE;
 	case A_LEARN_SPAM: 	return STOCK_PIXMAP_SPAM_BTN;
 	case A_GOTO_PREV: 	return STOCK_PIXMAP_UP_ARROW;
 	case A_GOTO_NEXT: 	return STOCK_PIXMAP_DOWN_ARROW;
@@ -475,6 +479,7 @@ gint toolbar_get_icon(int action) {
 	case A_WATCH_THREAD: 	return STOCK_PIXMAP_WATCHTHREAD;
 	case A_PRINT:	 	return STOCK_PIXMAP_PRINTER;
 	case A_CLOSE: 		return STOCK_PIXMAP_CLOSE;
+	case A_PREFERENCES:	return STOCK_PIXMAP_PREFERENCES;
 	case A_SEND: 		return STOCK_PIXMAP_MAIL_SEND;
 	case A_SENDL: 		return STOCK_PIXMAP_MAIL_SEND_QUEUE;
 	case A_DRAFT: 		return STOCK_PIXMAP_MAIL;
@@ -485,7 +490,7 @@ gint toolbar_get_icon(int action) {
 	case A_LINEWRAP_CURRENT:return STOCK_PIXMAP_LINEWRAP_CURRENT;
 	case A_LINEWRAP_ALL:	return STOCK_PIXMAP_LINEWRAP_ALL;
 	case A_ADDRBOOK: 	return STOCK_PIXMAP_ADDRESS_BOOK;
-	case A_CANCEL_INC:	return STOCK_PIXMAP_NOTICE_ERROR;
+	case A_CANCEL_INC:	return STOCK_PIXMAP_CANCEL;
 	case A_EXECUTE:		return STOCK_PIXMAP_EXEC;
 	#ifdef USE_ASPELL
 	case A_CHECK_SPELLING:	return STOCK_PIXMAP_CHECK_SPELLING;
@@ -499,7 +504,7 @@ static void toolbar_set_default_main(void)
 	struct {
 		gint action;
 	} default_toolbar[] = {
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 		{ A_GO_FOLDERS},
 		{ A_OPEN_MAIL},		
 		{ A_SEPARATOR}, 
@@ -510,14 +515,14 @@ static void toolbar_set_default_main(void)
 		{ A_COMPOSE_EMAIL},
 		{ A_SEPARATOR},
 		{ A_REPLY_MESSAGE}, 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		{ A_REPLY_ALL},
 		{ A_REPLY_SENDER},
 #endif
 		{ A_FORWARD},
 		{ A_SEPARATOR},
 		{ A_TRASH},
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 #if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM},
 #endif
@@ -558,7 +563,7 @@ static void toolbar_set_default_compose(void)
 	struct {
 		gint action;
 	} default_toolbar[] = {
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 		{ A_CLOSE},
 		{ A_SEPARATOR}, 
 #endif
@@ -566,7 +571,7 @@ static void toolbar_set_default_compose(void)
 		{ A_SENDL},
 		{ A_DRAFT},
 		{ A_SEPARATOR}, 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		{ A_INSERT},
 #endif
 		{ A_ATTACH},
@@ -606,7 +611,7 @@ static void toolbar_set_default_msgview(void)
 	struct {
 		gint action;
 	} default_toolbar[] = {
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 		{ A_CLOSE},
 		{ A_SEPARATOR}, 
 #endif
@@ -616,7 +621,7 @@ static void toolbar_set_default_msgview(void)
 		{ A_FORWARD},
 		{ A_SEPARATOR},
 		{ A_TRASH},
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 #if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM},
 #endif
@@ -880,7 +885,7 @@ static void activate_compose_button (Toolbar           *toolbar,
 		gtk_tool_button_set_icon_widget(
 			GTK_TOOL_BUTTON(toolbar->compose_mail_btn),
 			toolbar->compose_news_icon);
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(toolbar->compose_mail_btn), GTK_TOOLTIPS(toolbar->tooltips),
 			_("Compose News message"), NULL);
 #endif	
@@ -889,7 +894,7 @@ static void activate_compose_button (Toolbar           *toolbar,
 		gtk_tool_button_set_icon_widget(
 			GTK_TOOL_BUTTON(toolbar->compose_mail_btn),
 			toolbar->compose_mail_icon);
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(toolbar->compose_mail_btn), GTK_TOOLTIPS(toolbar->tooltips),
 			_("Compose Email"), NULL);
 #endif	
@@ -921,7 +926,7 @@ static void activate_learn_button (Toolbar           *toolbar,
 		gtk_tool_button_set_label(
 			GTK_TOOL_BUTTON(toolbar->learn_spam_btn),
 			_("Spam"));
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(toolbar->learn_spam_btn), GTK_TOOLTIPS(toolbar->tooltips),
 			_("Learn spam"), NULL);	
 #endif
@@ -933,7 +938,7 @@ static void activate_learn_button (Toolbar           *toolbar,
 		gtk_tool_button_set_label(
 			GTK_TOOL_BUTTON(toolbar->learn_spam_btn),
 			_("Ham"));
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 		gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(toolbar->learn_spam_btn), GTK_TOOLTIPS(toolbar->tooltips),
 			_("Learn ham"), NULL);
 #endif	
@@ -1345,7 +1350,7 @@ static void toolbar_prev_unread_cb(GtkWidget *widget, gpointer data)
 		
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			MsgInfo * msginfo = summary_get_selected_msg(msgview->mainwin->summaryview);
 		       
 			if (msginfo)
@@ -1392,7 +1397,7 @@ static void toolbar_next_unread_cb(GtkWidget *widget, gpointer data)
 
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			MsgInfo * msginfo = summary_get_selected_msg(msgview->mainwin->summaryview);
 			
 			if (msginfo)
@@ -1524,6 +1529,11 @@ static void toolbar_close_cb(GtkWidget *widget, gpointer data)
 	}
 }
 
+static void toolbar_preferences_cb(GtkWidget *widget, gpointer data)
+{
+	prefs_gtk_open();
+}
+
 static void toolbar_open_mail_cb(GtkWidget *widget, gpointer data)
 {
 	ToolbarItem *toolbar_item = (ToolbarItem*)data;
@@ -1697,6 +1707,7 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 		{ A_DRAFT,		toolbar_draft_cb      		},
 		{ A_OPEN_MAIL,		toolbar_open_mail_cb		},
 		{ A_CLOSE,		toolbar_close_cb		},
+		{ A_PREFERENCES,	toolbar_preferences_cb		},
 		{ A_INSERT,		toolbar_insert_cb     		},
 		{ A_ATTACH,		toolbar_attach_cb     		},
 		{ A_SIG,		toolbar_sig_cb	      		},
@@ -1720,7 +1731,7 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 		}
 	}
 }
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 #define TOOLBAR_ITEM(item,icon,text,tooltip) {								\
 	item = GTK_WIDGET(gtk_tool_button_new(icon, text));						\
 	gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(item), FALSE);					\
@@ -1812,7 +1823,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 	GSList *cur;
 	GSList *toolbar_list;
 	Toolbar *toolbar_data;
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	GtkTooltips *toolbar_tips;
 	
  	toolbar_tips = gtk_tooltips_new();
@@ -1872,12 +1883,16 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Close window"));
 			toolbar_data->close_window_btn = item; 
 			break;
+		case A_PREFERENCES:
+			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Open preferences"));
+			toolbar_data->preferences_btn = item; 
+			break;
 		case A_OPEN_MAIL:
 			TOOLBAR_ITEM(item,icon_wid,toolbar_item->text,_("Open email"));
 			toolbar_data->open_mail_btn = item; 
 			break;
 		case A_COMPOSE_EMAIL:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Compose Email"),
 				_("Compose with selected Account"));
@@ -1913,7 +1928,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			MAKE_MENU(learn_entries,"<LearnSpam>",toolbar_data->learn_spam_btn);
 			break;
 		case A_REPLY_MESSAGE:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Reply to Message"),
 				_("Reply to Message options"));
@@ -1927,7 +1942,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 #endif
 			break;
 		case A_REPLY_SENDER:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Reply to Sender"),
 				_("Reply to Sender options"));
@@ -1941,7 +1956,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 #endif
 			break;
 		case A_REPLY_ALL:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Reply to All"),
 				_("Reply to All options"));
@@ -1955,7 +1970,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 #endif
 			break;
 		case A_REPLY_ML:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Reply to Mailing-list"),
 				_("Reply to Mailing-list options"));
@@ -1969,7 +1984,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 #endif
 			break;
 		case A_FORWARD:
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 			TOOLBAR_MENUITEM(item,icon_wid,toolbar_item->text,
 				_("Forward Message"),
 				_("Forward Message options"));
@@ -2074,13 +2089,13 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 
 	}
 	toolbar_data->toolbar = toolbar;
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	toolbar_data->tooltips = toolbar_tips;
 #endif
 	gtk_widget_show_all(toolbar);
 
 	if (type == TOOLBAR_MAIN) {
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 		MainWindow *mainwin = mainwindow_get_mainwindow();
 		GtkWidget *progressbar = gtk_progress_bar_new();
 		item = GTK_WIDGET(gtk_tool_item_new());
@@ -2131,7 +2146,7 @@ void toolbar_update(ToolbarType type, gpointer data)
 	Compose    *compose = (Compose*)data;
 	MessageView *msgview = (MessageView*)data;
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	switch(type) {
 	case TOOLBAR_MAIN:
 		toolbar_data = mainwin->toolbar;
@@ -2171,9 +2186,9 @@ void toolbar_update(ToolbarType type, gpointer data)
 	default:
 		return;
 	}
-
+#ifdef MAEMO
 	hildon_window_remove_toolbar(HILDON_WINDOW(handlebox), GTK_TOOLBAR(toolbar_data->toolbar));
-
+#endif
 	toolbar_init(toolbar_data);
  	toolbar_data = toolbar_create(type, handlebox, data);
 #endif
@@ -2294,38 +2309,38 @@ void toolbar_main_set_sensitive(gpointer data)
 	}
 	if (toolbar->open_mail_btn) {
 		SET_WIDGET_COND(toolbar->open_mail_btn, 
-			M_TARGET_EXIST);
+			M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 	if (toolbar->reply_btn) {
 		SET_WIDGET_COND(toolbar->reply_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replyall_btn) {
 		SET_WIDGET_COND(toolbar->replyall_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replylist_btn) {
 		SET_WIDGET_COND(toolbar->replylist_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 	if (toolbar->replysender_btn) {
 		SET_WIDGET_COND(toolbar->replysender_btn,
-			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 	if (toolbar->fwd_btn) {
 		SET_WIDGET_COND(toolbar->fwd_btn, 
-			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+			M_HAVE_ACCOUNT|M_TARGET_EXIST|M_SUMMARY_ISLIST);
 	}
 
 	if (prefs_common.next_unread_msg_dialog == NEXTUNREADMSGDIALOG_ASSUME_NO) {
-		SET_WIDGET_COND(toolbar->next_btn, M_MSG_EXIST);
+		SET_WIDGET_COND(toolbar->next_btn, M_MSG_EXIST|M_SUMMARY_ISLIST);
 	} else {
 		SET_WIDGET_COND(toolbar->next_btn, 0);
 	}
 	
 	if (toolbar->trash_btn)
 		SET_WIDGET_COND(toolbar->trash_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE);
+			M_TARGET_EXIST|M_ALLOW_DELETE|M_NOT_NEWS);
 
 	if (toolbar->delete_btn)
 		SET_WIDGET_COND(toolbar->delete_btn,
@@ -2335,13 +2350,9 @@ void toolbar_main_set_sensitive(gpointer data)
 		SET_WIDGET_COND(toolbar->exec_btn, 
 			M_DELAY_EXEC);
 	
-/*	if (toolbar->learn_ham_btn)
-		SET_WIDGET_COND(toolbar->learn_ham_btn,
-			M_TARGET_EXIST|M_CAN_LEARN_SPAM);
-*/
 	if (toolbar->learn_spam_btn)
 		SET_WIDGET_COND(toolbar->learn_spam_btn, 
-			M_TARGET_EXIST|M_CAN_LEARN_SPAM);
+			M_TARGET_EXIST|M_CAN_LEARN_SPAM|M_SUMMARY_ISLIST);
 
 	if (toolbar->cancel_inc_btn)
 		SET_WIDGET_COND(toolbar->cancel_inc_btn,
